#========================================================================================
# package      API::Patch::PatchInstallHandler
# state        public
#----------------------------------------------------------------------------------------
# description  This the install handler for all patch install functions.
#========================================================================================
package API::Patch::PatchInstallHandler;
use base API::Patch::PatchHandler;

use strict;
use API::Patch::PatchInstallHandlerStores;
#use API::Patch::PatchStoreHandler;
#use API::Patch::PatchNonCartridgeFilesHandler;
#use API::Patch::PatchConfigFileHandler;
#use API::Patch::PatchServiceHandler;
#use API::Patch::PatchCartridgeHandler;

#========================================================================================
# function     doPatch_checkPreCondition
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_checkPreCondition($TargetVersion);
#----------------------------------------------------------------------------------------
# description  check pre condition for each handler using information from patch system info handler
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_checkPreCondition{
    my $self = shift;
    my ($TargetVersion) = @_;

    #register start
    $self->registerStart($TargetVersion);

    #ROADWORKS die if any preconditions are not given

    #register end
    $self->registerEnd($TargetVersion);

    return;
}

#========================================================================================
# function     doPatch_universal
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_universal($TargetVersion);
#----------------------------------------------------------------------------------------
# description  standard patch function for handler
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_universal{
    my $self = shift;
    my ($TargetVersion) = @_;

    #register start
    $self->registerStart($TargetVersion);

    $self->doPatch_basic($TargetVersion);

    #register end
    $self->registerEnd($TargetVersion);

    return;
}

#========================================================================================
# function     doPatch_basic
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_basic($TargetVersion);
#----------------------------------------------------------------------------------------
# description  basic patch function for handler, normally call sub patch handler objects
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_basic{
    my $self = shift;
    my ($TargetVersion) = @_;

    #register start
    $self->registerStart($TargetVersion);

    my $PatchInstallHandlerStores = API::Patch::PatchInstallHandlerStores->new($self->getPatchSystemInfoHandler());

    $PatchInstallHandlerStores->doPatch($TargetVersion);
    #call all handler and give them the PatchSystemInfoHandler

    #register end
    $self->registerEnd($TargetVersion);
}

#========================================================================================
# function     doPatch_6_14_2
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_universal();
#----------------------------------------------------------------------------------------
# description  standard patch function for handler
#========================================================================================
sub doPatch_6_14_2{
   my $self = shift;

   my $TargetVersion = '6.14.2';

    #register start
    $self->registerStart($TargetVersion);

    print "\n\nROADWORKS - SpadeWork for PatchInstallHandler::doPatch_6_14_2";

    $self->doPatch_basic($TargetVersion);

    #register end
    $self->registerEnd($TargetVersion);

    print "\n\nROADWORKS - ReWork for PatchInstallHandler::doPatch_6_14_2";

    return;
}

1;