#========================================================================================
# package      API::Patch::PatchPrepareHandler
# state        public
#----------------------------------------------------------------------------------------
# description  This the install handler for all patch install functions.
#========================================================================================
package API::Patch::PatchPrepareHandler;
use base API::Patch::PatchHandler;

use strict;
use API::Patch::PatchPrepareHandlerVersionFolders;
use API::Patch::PatchPrepareHandlerJQueryDictionaryConverter;

#========================================================================================
# function     doPatch_checkPreCondition
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_checkPreCondition($TargetVersion);
#----------------------------------------------------------------------------------------
# description  check pre condition for each handler using information from patch system info handler
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_checkPreCondition{
    my $self = shift;
    my ($TargetVersion) = @_;

    #register start
    $self->registerStart($TargetVersion);

    #ROADWORKS die if any preconditions are not given

    #register end
    $self->registerEnd($TargetVersion);

    return;
}

#========================================================================================
# function     doPatch_universal
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_universal($TargetVersion);
#----------------------------------------------------------------------------------------
# description  standard patch function for handler
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_universal{
    my $self = shift;
    my ($TargetVersion) = @_;

    #register start
    $self->registerStart($TargetVersion);

    #register step
    $self->logPatchHistory(
        $TargetVersion,
        'summary: generate different patch folders, generate cartridges version folders, install store types, convert JQuery dictionary',
        4
    );

    $self->doPatch_basic($TargetVersion);

    #register end
    $self->registerEnd($TargetVersion);

    return;
}

#========================================================================================
# function     doPatch_basic
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_basic($TargetVersion);
#----------------------------------------------------------------------------------------
# description  basic patch function for handler, normally call sub patch handler objects
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_basic{
    my $self = shift;
    my ($TargetVersion) = @_;

    #use now DE_EPAGES::Core::API::File qw (ExistsDirectory CreateDirectory)
    $self->useNow('DE_EPAGES::Core::API::File', qw (ExistsDirectory CreateDirectory) );

    my $PatchSystemInfoHandler = $self->getPatchSystemInfoHandler();
    my $PatchPrepareHandlerVersionFolders = API::Patch::PatchPrepareHandlerVersionFolders->new($PatchSystemInfoHandler);
    my $PatchPrepareHandlerJQueryDictionaryConverter = API::Patch::PatchPrepareHandlerJQueryDictionaryConverter->new($PatchSystemInfoHandler);

    #register start
    $self->registerStart($TargetVersion);

    #step1: create target dir
    my $TargetDir = $PatchSystemInfoHandler->getSystemInfo('PatchTargetDir');

    #register step
    $self->logPatchHistory( $TargetVersion, 'create directory ' . $TargetDir, 3 );

    #create dir
    CreateDirectory($TargetDir) unless ExistsDirectory($TargetDir);

    #step 2: create backup dir
    my $BackupDir = $PatchSystemInfoHandler->getSystemInfo('PatchBackupDir');

    #register step
    $self->logPatchHistory( $TargetVersion, 'create directory ' . $BackupDir, 3 );

    #create dir
    CreateDirectory($BackupDir) unless ExistsDirectory($BackupDir);

    #step 3: call sub handler
    #register step
    $self->logPatchHistory( $TargetVersion, 'call sub handler API::Patch::PatchPrepareHandlerVersionFolders', 3 );

    #call sub handler
    $PatchPrepareHandlerVersionFolders->doPatch($TargetVersion);

    #step 4: version call to install store types
    $self->callSubHandlerWithVersionSources($TargetVersion, 'PatchPrepareHandlerStoreTypes');

    #step 5: version call to convert dictionaries
    $self->callSubHandlerWithVersionSources($TargetVersion, 'PatchPrepareHandlerJQueryDictionaryConverter');

    #register end
    $self->registerEnd($TargetVersion);
}

#example versionized function
##========================================================================================
## function     doPatch_6_14_2
## state        public
##----------------------------------------------------------------------------------------
## syntax       $Handler->doPatch_universal();
##----------------------------------------------------------------------------------------
## description  standard patch function for handler
##========================================================================================
#sub doPatch_6_14_2{
#    my $self = shift;
#
#    my $TargetVersion = '6.14.2';
#
#    #register start
#    $self->registerStart($TargetVersion);
#
#
#    $self->doPatch_basic($TargetVersion);
#    #or $self->doPatch_basic_6_14_2($TargetVersion);
#
#    #register end
#    $self->registerEnd($TargetVersion);
#
#    print "\n\nROADWORKS - ReWork for PatchInstallHandler::doPatch_6_14_2";
#
#    return;
#}



1;