#========================================================================================
# package      API::Patch::PatchPrepareJQueryDictionaryConverter
# state        public
#----------------------------------------------------------------------------------------
# description  This the install handler for all patch install functions.
#========================================================================================
package API::Patch::PatchPrepareHandlerJQueryDictionaryConverter;
use base API::Patch::PatchHandler;

use strict;

#========================================================================================
# function     doPatch_checkPreCondition
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_checkPreCondition($TargetVersion);
#----------------------------------------------------------------------------------------
# description  check pre condition for each handler using information from patch system info handler
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_checkPreCondition{
    my $self = shift;
    my ($TargetVersion) = @_;

    #register start
    $self->registerStart($TargetVersion);

    #use now DE_EPAGES::Core::API::File qw (ExistsDirectory);
    #$self->useNow('ROADWORKS', qw (ROADWORKS) );

    #get patch system info handler
    my $PatchSystemInfoHandler = $self->getPatchSystemInfoHandler();

    #get info
    #ROADWORKS

    #check
    #ROADWORKS

    #register end
    $self->registerEnd($TargetVersion);

    return;
}

#========================================================================================
# function     doPatch_universal
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_universal($TargetVersion);
#----------------------------------------------------------------------------------------
# description  standard patch function for handler
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_universal{
    my $self = shift;
    my ($TargetVersion) = @_;

    #register start
    $self->registerStart($TargetVersion);

    #register step
    $self->logPatchHistory(
        $TargetVersion,
        'summary: convert the JQuery dictionary',
        4
    );

    $self->doPatch_basic($TargetVersion);

    #register end
    $self->registerEnd($TargetVersion);

    return;
}

#========================================================================================
# function     doPatch_basic
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_basic($TargetVersion);
#----------------------------------------------------------------------------------------
# description  basic patch function for handler, normally call sub patch handler objects
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_basic{
    my $self = shift;
    my ($TargetVersion) = @_;

    #register start
    $self->registerStart($TargetVersion);

    #use now
    $self->useNow('DE_EPAGES::XML::API::XML::CartridgeInstallerImportHandler', qw (GetCartridges));
    $self->useNow('DE_EPAGES::Database::API::Config', qw (BUStoreTypes));
    $self->useNow('DE_EPAGES::JQuery::API::DictionaryConverter',
        qw (
            ConvertMainDictionary
            ConvertCartridgeDictionary
            ConvertSubDictionary
        )
    );

    #get patch system info handler
    my $PatchSystemInfoHandler = $self->getPatchSystemInfoHandler();

    #get storetype info
    my $aStoreTypes = $PatchSystemInfoHandler->getStoreTypes();

    #check for install all or only a few store types
    my $AllStoreTypes = 0;
    $AllStoreTypes = 1 if (scalar @$aStoreTypes == 0);

    #get info about cartridges installed in store type
    my $haStoreTypeCartridgeInfo = GetCartridges();

    #use all store types if no store type is givven
    $aStoreTypes = BUStoreTypes() if ($AllStoreTypes);

    #placeholder for dictionary converter info
    my $hInfo = {'version' => $TargetVersion };

    #for every given store type
    foreach my $StoreType ( @$aStoreTypes ) {
        #get cartridges defined for store type
        my $aCartridges = $PatchSystemInfoHandler->getStoreTypeCartridges($StoreType);

        #fill info
        $hInfo->{'storetype'} = $StoreType;

        #check for install all or only a few cartridges for store types
        my $AllCartridges = 0;
        $AllCartridges = 1 if (scalar @$aCartridges == 0);

        #generate log line
        my $LogLine = "convert JQuery dictionary, storetype '$StoreType' for all included cartridges";
        $LogLine =~ s/\sall\sincluded/ specific/ unless ($AllCartridges);

        #log patch history
        $self->logPatchHistory( $TargetVersion, $LogLine, 2 );

        #convert main dictionary
        ConvertMainDictionary($hInfo);

        #use all cartridges if no cartridges is given for store type
        $aCartridges = $haStoreTypeCartridgeInfo->{$StoreType} if ($AllCartridges);

        #for each cartridge
        foreach my $Cartridge (@$aCartridges){

            #fill info
            $hInfo->{'cartridge'} = $Cartridge;

            #generate log line
            $LogLine = "convert JQuery dictionary, storetype '$StoreType' for cartridge '$Cartridge'";

            #log patch history
            $self->logPatchHistory( $TargetVersion, $LogLine, 1 );

            #convert dictionaries
            ConvertCartridgeDictionary($hInfo);
            ConvertSubDictionary($hInfo);
        }
    }

    #register end
    $self->registerEnd($TargetVersion);
}

#example versionized function
##========================================================================================
## function     doPatch_6_14_2
## state        public
##----------------------------------------------------------------------------------------
## syntax       $Handler->doPatch_universal();
##----------------------------------------------------------------------------------------
## description  standard patch function for handler
##========================================================================================
#sub doPatch_6_14_2{
#    my $self = shift;
#
#    my $TargetVersion = '6.14.2';
#
#    #register start
#    $self->registerStart($TargetVersion);
#
#    print "\n\nROADWORKS - SpadeWork for PatchInstallHandler::doPatch_6_14_2";
#
#    $self->doPatch_basic($TargetVersion);
#
#    #register end
#    $self->registerEnd($TargetVersion);
#
#    print "\n\nROADWORKS - ReWork for PatchInstallHandler::doPatch_6_14_2";
#
#    return;
#}

#----------------------------------------------------Patch Functions--------------------------

1;