#========================================================================================
# package      API::Patch::PatchPrepareHandlerStoreTypes
# state        public
#----------------------------------------------------------------------------------------
# description  ROADWORKS
#========================================================================================
package API::Patch::PatchPrepareHandlerStoreTypes;
use base API::Patch::PatchHandler;

use strict;
#========================================================================================
# function     doPatch_checkPreCondition
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_checkPreCondition($TargetVersion);
#----------------------------------------------------------------------------------------
# description  check pre condition for each handler using information from patch system info handler
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_checkPreCondition{
    my $self = shift;
    my ($TargetVersion) = @_;

    #register start
    $self->registerStart($TargetVersion);

    #ROADWORKS die if any preconditions are not given

    #register end
    $self->registerEnd($TargetVersion);

    return;
}

#========================================================================================
# function     doPatch_universal
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_universal($TargetVersion);
#----------------------------------------------------------------------------------------
# description  standard patch function for handler
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_universal{
    my $self = shift;
    my ($TargetVersion) = @_;

    #register start
    $self->registerStart($TargetVersion);

    #register step
    $self->logPatchHistory(
        $TargetVersion,
        'summary: install store types',
        4
    );

    #do basic
    $self->doPatch_basic($TargetVersion);

    #register end
    $self->registerEnd($TargetVersion);

    return;
}

#========================================================================================
# function     doPatch_basic
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_basic($TargetVersion);
#----------------------------------------------------------------------------------------
# description  basic patch function for handler, normally call sub patch handler objects
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_basic{
    my $self = shift;
    my ($TargetVersion) = @_;

    #use now
    $self->useNow('DE_EPAGES::XML::API::XML::CartridgeInstallerImportHandler', qw (GetCartridges));
    $self->useNow('DE_EPAGES::Database::API::Config', qw (BUStoreTypes));
    $self->useNow('DE_EPAGES::Installer::API::Install', qw( InstallStoreTypeDir ));

    #get patch system info handler
    my $PatchSystemInfoHandler = $self->getPatchSystemInfoHandler();

    #register start
    $self->registerStart($TargetVersion);

    #get storetype info
    my $aStoreTypes = $PatchSystemInfoHandler->getStoreTypes();

    #check for install all or only a few store types
    my $AllStoreTypes = 0;
    $AllStoreTypes = 1 if (scalar @$aStoreTypes == 0);

    #get info about cartridges installed in store type
    my $haStoreTypeCartridgeInfo = GetCartridges();

    #use all store types if no store type is givven
    $aStoreTypes = BUStoreTypes() if ($AllStoreTypes);

    #for every given store type
    foreach my $StoreType ( @$aStoreTypes ) {

        #get cartridges defined for store type
        my $aCartridges = $PatchSystemInfoHandler->getStoreTypeCartridges($StoreType);

        #check for install all or only a few cartridges for store types
        my $AllCartridges = 0;
        $AllCartridges = 1 if (scalar @$aCartridges == 0);

        #use all cartridges if no cartridges is given for store type
        $aCartridges = $haStoreTypeCartridgeInfo->{$StoreType} if ($AllCartridges);

        #generate log line
        my $LogLine = "install storetype '$StoreType' for all included cartridges";
        $LogLine =~ s/\sall\sincluded/ specific/;

        #log patch history
        $self->logPatchHistory( $TargetVersion, $LogLine, 2 );

        #install store type
        InstallStoreTypeDir( $StoreType, $aCartridges, $TargetVersion, { update => 1 } );
    }

    #register end
    $self->registerEnd($TargetVersion);
}

#example versionized function
##========================================================================================
## function     doPatch_6_14_2
## state        public
##----------------------------------------------------------------------------------------
## syntax       $Handler->doPatch_universal();
##----------------------------------------------------------------------------------------
## description  standard patch function for handler
##========================================================================================
#sub doPatch_6_14_2{
#    my $self = shift;
#
#    my $TargetVersion = '6.14.2';
#
#    #register start
#    $self->registerStart($TargetVersion);
#
#    print "\n\nROADWORKS - SpadeWork for PatchInstallHandler::doPatch_6_14_2";
#
#    $self->doPatch_basic($TargetVersion);
#
#    #register end
#    $self->registerEnd($TargetVersion);
#
#    print "\n\nROADWORKS - ReWork for PatchInstallHandler::doPatch_6_14_2";
#
#    return;
#}

1;