use Test::More;
use strict;
use warnings;

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite  );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Amazon::API::Object::PagerAWSItems;
use DE_EPAGES::Amazon::API::Object::RemoteAWS;
use Data::Dumper;

sub testPager {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
          SKIP: {
                my $Portal     = LoadObjectByPath('/Portals/Amazon');
                my $PortalSite = $Portal->path('Amazon_de_DE');

                my $RemoteAWS = DE_EPAGES::Amazon::API::Object::RemoteAWS->new(
                    'AWSURL'         => $PortalSite->get('AWSURL'),
                    'AWSAccessKeyId' => $Portal->get('AWSKeyID'),
                    'AWSSecretKeyId' => $Portal->get('AWSSecretKeyID'),
                    'Logging'        => $Portal->get('Logging'),
                );

                my $Pager = testSearch( "Deuter", $RemoteAWS );
                cmp_ok( scalar @{ $Pager->items() }, '>=', 1, 'search for "Deuter" returns at least one result' ) or diag( Dumper( $Pager->page() ) );
                $Pager = testASINLookup( 'B0009VC8MI', $RemoteAWS );
                cmp_ok( scalar @{ $Pager->items() }, '>=', 1, 'search for Campingaz Kocher ASIN returns at least one result' )
                  or diag( Dumper( $Pager->page() ) );
                $Pager = testASINLookup( 'B007226JCW', $RemoteAWS );
                cmp_ok(
                    scalar @{ $Pager->items() }, '>=', 2,
                    'search for OhPlah Beissring (variation product) returns at least 2 variations.'
                ) or diag( Dumper( $Pager->page() ) );
                $Pager = testASINLookup( 'B00XXXXXXX', $RemoteAWS );
                cmp_ok( scalar @{ $Pager->items() }, '==', 0, 'search for invalid ASIN returns 0 results.' ) or diag( Dumper( $Pager->page() ) );
            }
        }
    );
}

sub testSearch {
    my ( $SearchString, $RemoteAWS ) = @_;

    my $Pager = DE_EPAGES::Amazon::API::Object::PagerAWSItems->new( {
            'ObjectID'     => 1,
            'RemoteAWS'    => $RemoteAWS,
            'ASIN'         => undef,
            'UPCEAN'       => undef,
            'SearchString' => $SearchString,
            'ProductName'  => $SearchString,
            'Page'         => 1,
        }
    );

    $Pager->page();
    return $Pager;
}

sub testASINLookup {
    my ( $ASIN, $RemoteAWS ) = @_;

    my $Pager = DE_EPAGES::Amazon::API::Object::PagerAWSItems->new( {
            'ObjectID'  => 1,
            'RemoteAWS' => $RemoteAWS,
            'ASIN'      => $ASIN,
            'UPCEAN'    => undef,
        }
    );
    if ( defined( $Pager->{'XMLResponse'}->{'OperationRequest'}->{'Errors'}->{'Error'} )
        && $Pager->{'XMLResponse'}->{'OperationRequest'}->{'Errors'}->{'Error'}->[0]->{'Code'} eq 'AWS.TimestampExceedsTimeToLive' )
    {
        diag('AWS.TimestampExceedsTimeToLive - wrong clock! ASIN search will fail.');
    }

    $Pager->page();
    return $Pager;
}

sub Main {
    testPager();
    done_testing();
    return;
}

RunScript( Sub => \&Main );

1;
