#========================================================================================
# package      MooseObj
# state        private
#========================================================================================
package MooseObj;
use strict;
use Moose;
has 'MyBool', 'isa' => 'Bool', 'is' => 'rw';

no Moose;
__PACKAGE__->meta->error_class('DE_EPAGES::Core::API::MooseError');
__PACKAGE__->meta->make_immutable;

#========================================================================================
# package      main
# state        private
#========================================================================================
package main;    ## no critic (ProhibitMultiplePackages)

use Test::More;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use strict;

sub Main {
    testOk();
    testMooseException();
    testRegularException();
    done_testing;
}

sub testMooseException {
    my $Obj = MooseObj->new();
    my $errorLine;
    eval {
        $errorLine = __LINE__; $Obj->MyBool('Str');
        fail('should throw exception.');
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        is( $Error->code, 'MooseDie', 'MooseDie is error code.' );
        like(
            $Error->shortMessage,
            ## no critic (ProhibitComplexRegexes, RequireExtendedFormatting, ProhibitEscapedMetacharacters)
            qr{^Attribute \(MyBool\) does not pass the type constraint because: Validation failed for 'Bool' with value Str$},
            'error message.'
        );
        cmp_ok( scalar @{ $Error->callers }, '==', 6, 'contains caller stack.' );
        like( $Error->message, qr{MooseErrorTest.t line $errorLine\.$}, 'message contains exact error occurence' );
    }
}

sub testRegularException {
    eval {
        die 'die';
        fail('should throw an exception.');    ## no critic (ProhibitUnreachableCode)
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        is( $Error->code,         'NativeDie', 'error code' );
        is( $Error->shortMessage, 'die',       'message' );
    }
}

sub testOk {
    my $Obj = MooseObj->new();
    $Obj->MyBool(1);
    is( $Obj->MyBool(), 1, 'setting boolean is ok.' );
}

RunScript( Sub => \&Main, Debug => 1 );
