/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.

	epages.event.selectables $Revision: 1.31.28.1 $

*/
dojo.provide('epages.event.selectables');
dojo.require('epages.lang.array');
dojo.require('epages.html');
dojo.require('epages.browser');
dojo.require('epages.html.element');

epages.event.Selectables = epages.Class();

epages.event.Selectables.prototype = {
	selectables:        undefined,
	currentSelected:    undefined,
	currentHighlighted: undefined,
	callback:           undefined,
	strHighlighted: 'Highlighted',
	strUnselected:  'Unselected',
	strSelected:    'Selected',
	selectionGuides : undefined,

	initialize: function(/*ArrayLike*/enumselectables, doc) {
		if(doc === undefined){
			doc = document;
		}
		var win = epages.getDocumentWindow(doc);
		if (this.selectables === undefined){
			this.selectables = new Array();
		}
		var sel = this.selectables;
		var existsUndefined = -1;
		var counter = -1;
		this.selectionGuides = {};

		this.selectionGuides.North = doc.createElement("div");
		this.selectionGuides.South = doc.createElement("div");
		this.selectionGuides.West  = doc.createElement("div");
		this.selectionGuides.East  = doc.createElement("div");
		for(var item in this.selectionGuides) {
			var div = this.selectionGuides[item];
			div.className="SelectionGuide "+item;
			doc.getElementsByTagName("body")[0].appendChild(div);
		}

		$A(enumselectables).each(function(el) {
			counter++;
			if (el) {
				sel.push(el);

					var contentDiv = win.dojo.query(".ContentAreaInner", el)[0];
					var effectLayer = el.ownerDocument.createElement("div");
					effectLayer.className = "EffectLayer";

					if(el.tagName.toLowerCase() == "td") {
						effectLayer.style.marginLeft = "-" + el.getElementsByTagName("div")[0].offsetLeft + "px";
						effectLayer.style.marginTop = "-" + el.getElementsByTagName("div")[0].offsetTop + "px";
					}

					if(contentDiv && epages.Browser.engine == "MSIE" || el.attributes["class"].nodeValue.indexOf("ContentArea") > -1 && el.tagName.toLowerCase() == "div"){
						el.style.position = "relative";
						dojo.style(effectLayer, {
							left : "0",
							top  :"0",
							marginLeft : "0",
							marginTop  : "0"
						});
					}
					el.insertBefore(effectLayer, el.firstChild);

					//ugly way to check if element has been resized...do not have a better idea for now
					var tempResizeCheck = function() {
						if(parseInt(effectLayer.style.height) != el.offsetHeight) {
							effectLayer.style.height = el.offsetHeight + "px";
						}
						if(parseInt(effectLayer.style.width) != el.offsetWidth) {
							effectLayer.style.width = el.offsetWidth + "px";
						}
					};

					dojo.connect(window, "resize", tempResizeCheck);

			}else if (existsUndefined == -1) {
				existsUndefined = counter;
			}
		});
		if (existsUndefined != -1){
			console.warn('element ' + existsUndefined + ' is not defined and can\'t be selected');
		}
		dojo.connect(window, "resize", this,"setSelectionGuides");
		dojo.connect($("EmbeddedStorefront"), "resize", this, "setSelectionGuides");

		this.preparePage();
	},

	unselect: /*void*/function(/*HTMLElement*/sel) {
		var className = sel.className.replace(this.strHighlighted,this.strUnselected).replace(this.strSelected,this.strUnselected);
		if ( !className.match( this.strUnselected )){
			className += ' ' + this.strUnselected;
		}
		sel.className = className;
	},

	reEnterElement: /*void*/function (/*HTMLElement*/sel) {
		this.unselect(sel);
		this.appendEvents(sel);
		if(sel == this.currentSelected){
			this.currentSelected = undefined;
		}
		this.selectables.push(sel);
	},

	destroy: /*void*/function () {
		for( var i=0,l=this.selectables.length ; i<l ; i++ ) {
			var className = this.selectables[i].className;
			[this.strHighlighted, this.strUnselected, this.strSelected].each(function (str) {
				className.replace(str,'');
			});
			this.selectables[i].className = className;
		}
	},

	appendEvents: /*void*/function(/*HTMLElement*/sel) {
		if(sel.tagName == "BODY" && epages.Browser.engine == "MSIE") {
			sel.onmouseover = function() {
				this.setCapture();
			};
			sel.onmouseout = function() {
				this.releaseCapture();
			};
		}
		dojo.connect(sel, 'onmouseover', this, 'onMouseOver');
		dojo.connect(sel, 'onmousedown', this, 'onClick');
		dojo.connect(sel, 'onclick', this, 'onClick');
	},

	preparePage: /*void*/function() {
		for( var i=0,l=this.selectables.length ; i<l ; i++ ) {
			var sel = this.selectables[i];
			this.unselect(sel);
			this.appendEvents(sel);
		}
	},

	onMouseOver: /*void*/function(/*Event*/evt) {
		dojo.stopEvent(evt);
		if(!$("EmbeddedStorefront").contentWindow.epages.html.getElementsByClassName("IsDragging", evt.currentTarget)[0]) {
			var newHighlighted = evt.currentTarget;
			if(!this.oldHighlighted) {
				this.oldHighlighted = newHighlighted;
			}
			if(this.oldHighlighted != newHighlighted) {
				epages.html.replaceClass(this.oldHighlighted, this.strUnselected, this.strHighlighted);
				epages.html.replaceClass(newHighlighted, this.strHighlighted, this.strUnselected);
				this.oldHighlighted = newHighlighted;
			}
		}
	},

	onClick: /*void*/function(/*Event*/evt) {
		dojo.stopEvent(evt);
		var newSelected = evt.currentTarget;
		if(newSelected != this.currentSelected) {
			var oldSelected = this.currentSelected;
			if (oldSelected !== undefined) {
				epages.html.replaceClass(oldSelected, this.strUnselected, this.strSelected);
				epages.html.replaceClass(oldSelected, this.strUnselected, this.strHighlighted);
			}
			epages.html.replaceClass(newSelected, this.strSelected, this.strUnselected);
			epages.html.replaceClass(newSelected, this.strSelected, this.strHighlighted);
			this.currentSelected = newSelected;
			if (this.callback !== undefined){
				this.callback(oldSelected, newSelected, evt);
			}
			this.setSelectionGuides();
		}
	},

	setSelectionGuides: function() {
		if(this.currentSelected) {
			var el = $E(this.currentSelected);
			var pos = el.getAbsolutePosition();

			pos.width = this.currentSelected.offsetWidth;
			pos.height = this.currentSelected.offsetHeight;

			//insert some jquery magic because dojo magic is calcuating wrong in this case
			var posJq = jq(this.currentSelected).offset();
			pos.x = posJq.left;
			pos.y = posJq.top;

			pos.x2 = pos.x + pos.width -1;
			pos.y2 = pos.y + pos.height -1;

			this.selectionGuides.East.style.top = pos.y+"px";
			this.selectionGuides.East.style.height = pos.height+"px";
			this.selectionGuides.East.style.left = pos.x+"px";

			this.selectionGuides.West.style.top = pos.y+"px";
			this.selectionGuides.West.style.height = pos.height+"px";
			this.selectionGuides.West.style.left = pos.x2+"px";

			this.selectionGuides.North.style.top = pos.y+"px";
			this.selectionGuides.North.style.width = pos.width+"px";
			this.selectionGuides.North.style.left = pos.x+"px";

			this.selectionGuides.South.style.top = pos.y2+"px";
			this.selectionGuides.South.style.width = pos.width+"px";
			this.selectionGuides.South.style.left = pos.x+"px";
		}
	}
};
