/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.

	epages.html.undo $Revision: 1.25.28.1 $

*/
dojo.provide("epages.html.undo");
dojo.require("epages.html.element");
dojo.require("epages.lang.hash");

epages.html.UnDo = function(container) {
	this.errorStack    = [];
	this.historyStack  = [];
	this.forwardStack  = [];
	this.currentValues = $H();
	this.defaultValues = $H();

	if (container === undefined){
		container = document;
	}

	this.registerContainerInputs(container);
};

epages.html.UnDo.prototype = {
	POS_OBJECT:  0,
	POS_NEW   :  1,
	POS_OLD   :  2,
	historyStack:   undefined,
	forwardStack:   undefined,
	currentValues:  undefined,
	defaultValues:  undefined,
	isUndoCall:     false,
	lastSavePoint:  0,
	isTyping:		false,
	undoNavigationElementWidgetStack: [],

	registerContainerInputs: function(container) {
		var iterator = this.registerInput;
		dojo.forEach(container.getElementsByTagName("select"),   iterator, this);
		dojo.forEach(container.getElementsByTagName("input"),    iterator, this);
		dojo.forEach(container.getElementsByTagName("textarea"), iterator, this);
	},

	registerInput: function(el) {
		// ignore all inputs with ep_noundo = 1 and without name
		if (el.name != '' & el.getAttribute("ep_noundo")!='1') {
			this.setDefaultValue(el);
			var me = this;
			$E(el).connectChangeEvent(function () { me.addInput(el); });
		}
	},

	setDefaultValue: function (/*HTMLObject*/object) {
		// summary: set default value
		this.defaultValues.set(object.name, $E(object).get());
	},

	_lastValue: function(identifier) {
		// returns: last Value
		var entry = this.currentValues.get(identifier);
		if (entry !== undefined){
			return entry;
		}

		entry = this.defaultValues.get(identifier);
		
		return entry;
	},

	addInput: function(object, currentValue) {
			// alert('addInput:' + object.name);
		if (! this.isUndoCall) { // ignore self fired events
			var lastValue = this._lastValue(object.name);
			if (currentValue == null){
				currentValue = $E(object).get();
			}
			this.currentValues.set(object.name, currentValue);
			if (lastValue !== null && currentValue != lastValue) {
				if (this.forwardStack.length > 0 && this.historyStack.length < this.lastSavePoint){
					this.lastSavePoint = -1; // save is always false
				}
				this.historyStack.push([ object, currentValue, lastValue ]);
				this.forwardStack = [];
			}
		}
		this.validateInput(object);
	},

	validateInput: function(object) {
		if (typeof(object.getAttribute) !== 'function'){
			return;
		}

		var isError=object.getAttribute("ep_error");
		var inputPosition=$A(this.errorStack).find(object);
		if (isError == "1" && inputPosition == null){
			this.errorStack.push(object);
		}else if (isError == "0" && inputPosition != null){
			this.errorStack.splice([inputPosition],1);
		}
	},

	show: function(object) {
		object.innerHTML  = '<p>history: '       + this.historyStack.flatten()  + '</p>';
		object.innerHTML += '<p>forward: '       + this.forwardStack.flatten()  + '</p>';
		object.innerHTML += '<p>currentValues: ' + this.currentValues.flatten() + '</p>';
		object.innerHTML += '<p>defaultValues: ' + this.defaultValues.flatten() + '</p>';
	},

	_setValue: function (object, value) {
		this.currentValues.set(object.name, value);
		var htmlElement = $E(object);
		htmlElement.set(value);
		this.isUndoCall = true;
		htmlElement.callOnChange();
		this.isUndoCall = false;
	},

	undo: function() {
		if (this.historyStack.length == 0){
			return;
		}
		var entry = this.historyStack.pop();
		this.forwardStack.push(entry);
		this._setValue(entry[this.POS_OBJECT], entry[this.POS_OLD]);
		
		// undo stack for dialogs
		for(var i=0, iLength=this.undoNavigationElementWidgetStack.length; i<iLength; i++){
			this.undoNavigationElementWidgetStack[i].resetEditDialogInputValues();
		}
	},

	redo: function() {
		if (this.forwardStack.length == 0){
			return;
		}
		var entry = this.forwardStack.pop();
		this.historyStack.push(entry);
		this._setValue(entry[this.POS_OBJECT], entry[this.POS_NEW]);
		
		// undo stack for dialogs
		for(var i=0, iLength=this.undoNavigationElementWidgetStack.length; i<iLength; i++){
			this.undoNavigationElementWidgetStack[i].resetEditDialogInputValues();
		}
	},

	existsChanges: function () {
		return this.historyStack.length != this.lastSavePoint;
	},

	clear: function() {
		this.historyStack = [];
		this.forwardStack = [];
		this.lastSavePoint = 0;
		this.defaultValues = this.currentValues;
		this.currentValues = $H();
	},

	getChanged: function () {
		return $H(this.currentValues);
	},

	setLastSavePoint: function(){
		this.lastSavePoint = this.historyStack.length;
	},

	checkTyping: function(isTyping) {
		if (this.isTyping != isTyping) {
			this.isTyping = isTyping;
			this.onChangeTyping();
		}
	},

	
	onChangeTyping: function() {
		// summary: gets called, when the user starts or stops typing
		// description: callback stub
		// tags: extendable
	}

};

// this class is an adapter to epages.html.UnDo to be used by tinymce instead of its custom undomanager
epages.html.TinymceUndoManagerAdapter = function(undoObject, input, onChangeCallback) {
	this.__undoObject = undoObject;
	this.__input = input;
	this.__onChange = onChangeCallback;
};

epages.html.TinymceUndoManagerAdapter.prototype = {
	typing : 0, // Integer - is user currently typing

	__undoObject: null, // Object - reference to epages.html.UnDo object
	__input: null,
	
	__onChange: null, // function - callback for onChange events

	add : function(info) {
		if (info && info.initial) {
			this.__undoObject.setDefaultValue(this.__input);
		} else {
			//this.__undoObject.addInput(this.__input);
			this.__onChange();
		}
	},

	undo : function() {
		if (this.typing){
			this.add();
		}
		this.__undoObject.undo();
		this.typing = 0;
	},

	redo : function() {
		this.__undoObject.redo();
	},

	clear : function() { this.__undoObject.clear(); },

	hasUndo : function() {
		return this.typing || this.__undoObject.historyStack.length > 0;
	},

	hasRedo : function() {
		return this.__undoObject.forwardStack.length > 0;
	}

};