/*
	Copyright (c) 2006-2010, ePages GmbH
	All Rights Reserved.

	epages.widget.InlineTinyMCE $Revision: 1.16.4.1 $

*/
dojo.provide("epages.widget.InlineTinyMCE");
dojo.require("epages.widget.LocalizedWidget");
dojo.require("epages.string");

dojo.declare("epages.widget.InlineTinyMCE",
	[epages.widget.LocalizedWidget],
	{
		/**
		 * public properties
		 */
		useAbsolutePath : false,
		editorNodeId    : '',
		editorNodeName  : '',
		toolMediaGalleryId : '',
		defaultPColor   : '',
		zoomLevel       : 1,
		editorWidth: 470,
		editorHeight: 200,
		EDITOR_MAX_HEIGHT: 500,
		autoHeight: false,
		isHidden: false,
		isActive:false,
		hoverMode: false,
		autofocus: true,

		/**
		 * private properties
		 */
		_translation     : '',
		_editorNodeContent : '',
		_toolMediaGallery: undefined,
		_tinyMCEButtons: [],
		_editor : undefined,
		_isOpen: false,
		_tinyMCEBookmark: undefined,
		CLOSED_EDITOR_HEIGHT: 50,

		/**
		 * dojo widget properties
		 */
		templatePath    : dojo.moduleUrl('epages.widget', "templates/InlineTinyMCE.html"),
		translationName : dojo.moduleUrl('epages.widget', "templates/InlineTinyMCE"),
		imageRoot : epages.vars.StoreRoot,
		widgetsInTemplate: true,

		showEditMode: function() {
		if(epages.vars.currentInlineTinyMCE){
			epages.vars.currentInlineTinyMCE.hideEditMode();
		}
		epages.vars.currentInlineTinyMCE = this;
		tinyMCE.activeEditor = this._editor;
		this._showToolbar();
		this._changeEditorHeight(true,true);
		this._isOpen = true;
		},

		hideEditMode: function() {
			epages.vars.currentInlineTinyMCE = undefined;
			this._hideToolbar();
			this._changeEditorHeight(false,true);
			this._isOpen = false;
		},

		_showToolbar:function(){
			dojo.fx.wipeIn({ node: this.toolbarNode,  duration: 200}).play();
		},

		_hideToolbar:function(){
			dojo.fx.wipeOut({ node: this.toolbarNode, duration: 200}).play();
		},

		postMixInProperties: function(){
			this.inherited("postMixinProperties", arguments);
			this.useAbsolutePath = epages.string.toBoolean(this.useAbsolutePath);
			this.isActive = epages.string.toBoolean(this.isActive);
			this.hoverMode = epages.string.toBoolean(this.hoverMode);
			this.autofocus = epages.string.toBoolean(this.autofocus);
		},

		postCreate: function() {
			this.inherited("postCreate", arguments);
			this.connect(this.closeButtonNode,"onclick", "hideEditMode");
			this.connect(this.advimageButtonNode,"onclick", "_onImageButtonClick");
			this.connect(this.advmediaButtonNode,"onclick", "_onMediaButtonClick");
			if(this.hoverMode){
				this.connect(this.editorContainerNode,"onmouseover", "_onMouseOverEditor");
				this.connect(this.editorContainerNode,"onmouseout", "_onMouseOutEditor");
			}
			this.connect(this.advimageButtonNode,"onmousedown", function(){
				this._saveSelection(this.advimageButtonNode);
			});
			this.connect(this.advmediaButtonNode,"onmousedown", function(){
				this._saveSelection(this.advmediaButtonNode);
			});

			this.connect(this.epeditlinkNode,"onclick", "_onLinkButtonClick");

			if(this.srcNodeRef && this.srcNodeRef.innerHTML){
				this._editorNodeContent = this.srcNodeRef.innerHTML;
				this.srcNodeRef.innerHTML = '';
				this.editorNode.innerHTML = this._editorNodeContent;
			}

			dojo.subscribe("tinymce_inline/buttonsChanged", this,"_onTinyMCEButtonsChanged");
			dojo.subscribe('tinyMCEControlManager/setActive',this,'onTinyMCEControlManagerSetActive');

			if(this.toolMediaGalleryId != '' && $$(this.toolMediaGalleryId)){
				this._toolMediaGallery = $$(this.toolMediaGalleryId);
			}

			//register tinyMCEButtons
			this._tinyMCEButtons.push(this.tinyMCEButtonBold);
			this._tinyMCEButtons.push(this.tinyMCEButtonItalic);
			this._tinyMCEButtons.push(this.tinyMCEButtonUnderline);
			this._tinyMCEButtons.push(this.tinyMCEButtonJustifyLeft);
			this._tinyMCEButtons.push(this.tinyMCEButtonJustifyCenter);
			this._tinyMCEButtons.push(this.tinyMCEButtonJustifyRight);
			this._tinyMCEButtons.push(this.tinyMCEButtonJustifyFull);
			this._tinyMCEButtons.push(this.tinyMCEButtonInsertUnorderedList);
			this._tinyMCEButtons.push(this.tinyMCEButtonInsertOrderedList);

		},

		startup:function(){
			this.inherited("startup", arguments);
			this.editorNode.style.width = this.editorWidth + 'px';
			this.editorNode.style.height = this.editorHeight + 'px';
			if(epages.InlineTinyMCEController && this.isHidden == false){
				epages.InlineTinyMCEController.addToStack(this);
			}
			else{
				dojo.subscribe("InlineTinyMCE/showEditor",this,"_onPublishShowEditor");
			}
		},

		_onPublishShowEditor: function(editorId){
			if(editorId && editorId == this.editorNodeName){
				this._buildEditor();
			}
		},

		_buildEditor:function(){
			this.editorContainerNode.style.visibility = '';
			var settings = dojo.mixin(epages.vars.tinyMCESettings, {
				width: this.editorWidth,
				height: 30//this.editorHeight
			});
			this._editor = new tinymce.Editor(this.editorNodeId,settings);
			// BEGIN: overload focus of tinymce to prevent focus onstartup if autofocus disabled
			var _autofocus	= this.autofocus,
				_focus		= this._editor.focus;
			this._editor.focus = function(){
				if( _autofocus ){
					_focus.apply(this,arguments);
				}
			};
			// END: overload focus of tinymce to prevent focus onstartup if autofocus disabled
			this._editor.parentWidgetId = this.id;
			this._editor.onClick.add(function(ed, e) {
				var parentWidget = $$(this.parentWidgetId);
				if(parentWidget && !parentWidget._isOpen){
					parentWidget.showEditMode();
				}
				ed.focus();
			});
			var onChangeFunction = function(editor,undoLevel){
				var parentWidget = $$(this.parentWidgetId);
				var content = editor.getContent();
				if(parentWidget && parentWidget.editorNode && content){
					parentWidget.editorNode.value = content;
					var SaveButton = $$(dojo.query(".SaveButton input"));
					if(SaveButton && SaveButton[0] && $$(SaveButton[0])){
						$$(SaveButton[0].id).enable();
					}
				}
				if(parentWidget._isOpen){
					//save and restore bookmark when changing height of editor to avoid writing problems
					parentWidget._tinyMCEBookmark = editor.selection.getBookmark('simple');
					parentWidget._changeEditorHeight(true);
					editor.selection.moveToBookmark(parentWidget._tinyMCEBookmark);
				}
			};

			this._editor.onChange.add(onChangeFunction);
			this._editor.onKeyDown.add(onChangeFunction);


			this._editor.onInit.add(function(ed, cm) {
				var parentWidget = $$(this.parentWidgetId);
				if(parentWidget.isActive){
					parentWidget.showEditMode();
				}
				else{
					parentWidget.hideEditMode();
					ed.contentAreaContainer.style.height = parentWidget.CLOSED_EDITOR_HEIGHT;
					ed.contentAreaContainer.style.overflow = 'hidden';
				}
			});

			this._editor.render();
		},

		_onTinyMCEButtonsChanged:function(buttons,editor){
			if(!editor || editor.editorId != this.editorNodeId){
				return;
			}
			for(var el in buttons) {
				var value = buttons[el];
				var node = this[el+'ButtonNode'];
				if(node){
					node.setAttribute("ep_state", value);
					// button display
					if(value == 0) {              // disable button
						dojo.addClass(node, "DisabledButton");
					}
					else if(value == 1) {       // normal state / insert mode
					dojo.removeClass(node, "DisabledButton");
					}
					else if(value == 2) {       // item selected / edit mode
						dojo.removeClass(node, "DisabledButton");
					}
				}
			}
		},

		_onImageButtonClick: function() {
			// summary: call image properties or insert image via toolmediagallery
			if(!this._toolMediaGallery){
				return;
			}
			var ep_state =  this.advimageButtonNode.getAttribute("ep_state");
			if(ep_state == 1) {
				this._toolMediaGallery.setFileTypes("jpe;jpg;jpeg;gif;png");
				this._toolMediaGallery.openDialog();
			}
			else if(ep_state == 2){
				this._tinyMCEexecCommand('mceAdvImage');
			}
		},

		_onMediaButtonClick: function() {
			// summary: call media object properties or insert media object via toolmediagallery
			if(!this._toolMediaGallery){
				return;
			}
			var ep_state =  this.advmediaButtonNode.getAttribute("ep_state");
			if(ep_state == 1) {
				this._toolMediaGallery.setFileTypes("");
				this._toolMediaGallery.openDialog();
			}
			else if(ep_state == 2){
				this._tinyMCEexecCommand('mceMedia');
			}
		},

		_onLinkButtonClick: function(){
				if(this.useAbsolutePath){
					tinyMCE.execCommand('epeditlink',false,true);
				}
				else{
					tinyMCE.execCommand('epeditlink',false);
				}
		},

		_tinyMCEexecCommand:function(command, option) {
			if(tinyMCE && tinyMCE.activeEditor){
				tinyMCE.activeEditor.selection.moveToBookmark(tinymce.WindowManager.bookmark);
				tinyMCE.execCommand(command, option);
			}
		},

		_saveSelection:function(buttonNode){
			if(!buttonNode){
				return;
			}
			var ep_state =  buttonNode.getAttribute("ep_state");
			if(ep_state == 1){
				this._saveBookmark(true);
			}
			else{
				this._saveBookmark(false);
			}
		},

		_saveBookmark:function(insertNew){
			if(typeof(tinymce) != "undefined" && tinyMCE){
				if(insertNew) { // workaround: insert img, element in MSIE  -> insert empty string to keep cursor position
					tinyMCE.activeEditor.execCommand('mceInsertContent', false, '');
				}
				tinymce.WindowManager.bookmark = tinyMCE.activeEditor.selection.getBookmark('simple');
			}
		},

		onTinyMCEControlManagerSetActive: function(controlName,state){
			if(tinyMCE && tinyMCE.activeEditor.editorId == this.editorNodeId){
				for( var i=0,iLength=this._tinyMCEButtons.length ; i<iLength ; i++ ){
					var currentButton = this._tinyMCEButtons[i];
					var buttonName = currentButton.getAttribute('name');
					buttonName = buttonName.replace(this.id+'_',"");
					if(buttonName == controlName){
						if(state){
							dojo.removeClass(currentButton,'Normal');
							dojo.addClass(currentButton,'Selected');
						}
						else{
							dojo.removeClass(currentButton,'Selected');
							dojo.addClass(currentButton,'Normal');
						}
					}
				}
			}
		},

		_changeEditorHeight: function(/*boolean*/isOpen, /*boolean*/useAnimation){
			//summary: change the Height of the editor depending on the given parameter isOpen
			// isOpen true opens the editor
			//useAnimation activates a wipe animation
			var contentHeight = this._editor.contentDocument.height? this._editor.contentDocument.height : this._editor.contentDocument.activeElement.clientHeight;
			var contentContainer = this._editor.contentAreaContainer;
			var iFrame = contentContainer.firstChild;
			var calculatedHeight = 0;
			var overflowValue = 'auto';
			if(isOpen){
				if(contentHeight > this.EDITOR_MAX_HEIGHT && this.editorHeight < this.EDITOR_MAX_HEIGHT){
					calculatedHeight = this.EDITOR_MAX_HEIGHT + 20;
				}
				else if(contentHeight && contentHeight > contentContainer.clientHeight){
					calculatedHeight = contentHeight + 20;
				}
				else{
					calculatedHeight = contentContainer.clientHeight;
					overflowValue = 'hidden';
				}
			}
			else{
				calculatedHeight = this.CLOSED_EDITOR_HEIGHT;
				overflowValue = 'hidden';
			}
			if(calculatedHeight + 'px' == contentContainer.style.height){
				return;
			}
			if(useAnimation){
				dojox.fx.wipeTo({
					node: contentContainer,
					duration: 500,
					height: calculatedHeight,
					onEnd : function(node){
						node.style.overflow = overflowValue;
					}
				}).play();
				dojox.fx.wipeTo({
					node: iFrame,
					duration: 500,
					height: calculatedHeight,
					onEnd : function(node){
						node.style.overflow = overflowValue;
					}
				}).play();
			}
			else{
				contentContainer.style.height = calculatedHeight + 'px';
				iFrame.style.height = calculatedHeight + 'px';
				contentContainer.style.overflow = overflowValue;
				iFrame.style.overflow = overflowValue;
			}
		},

		_onMouseOverEditor: function(evt){
			if(!this._isOpen){
				this._changeEditorHeight(true,true);
			}
		},

		_onMouseOutEditor: function(evt){
			if(!this._isOpen){
				this._changeEditorHeight(false,true);
			}
		}
	}
);
