/*
 * @copyright		© Copyright 2006-2011, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.colorpicker
 *
 * @revision		$Revision: 1.10.2.1 $
 */

jQuery.plugin( 'ep.ui.colorpicker', {
	plugin:	['jQuery.tmpl', 'jQuery.metaparse', 'jQuery.ui.widget', 'jQuery.ui.draggable', 'ep.core', 'ep.dict', 'ep.color', 'ep.colorpicker', 'ep.fn.contextOrientation', 'ep.ui.input'],
	dict:	'ep.dict',
	tmpl:	'ep.ui.colorpicker'
}, function($){

/*
 * @dictionary		ep.dict
 *
 * @translation		{ColorCode}
 *					{Transparent}
 *					{Restore}
 *					{UserColors}
 *					{MatchingColors}
 */
	var html			= $('html'),

		epColor			= ep.color,

		combinedColors	= function( color ){
			var hsb = epColor.stringToHsb(color),
				hsb1 = {},
				hsb2 = {},
				hsb3 = {},
				hsb4 = {};

			// set hue
			hsb1.h = hsb.h<340 ? hsb.h+20 : (hsb.h+20)-360;
			hsb2.h = hsb1.h;
			hsb3.h = hsb.h<240 ? hsb.h+120 : (hsb.h+120)-360;
			hsb4.h = hsb.h<260 ? hsb.h+100 : (hsb.h+100)-360;

			// set saturation
			hsb1.s = hsb.s>50 ? hsb.s-40 : hsb.s+40;
			hsb2.s = hsb.s>90 ? hsb.s-10 : hsb.s+10;
			hsb3.s = hsb.s>35 ? hsb.s-25 : hsb.s+25;
			hsb4.s = hsb2.s;

			// set brightness
			hsb1.b = hsb.b>60 ? hsb.b-40 : hsb.b+40;
			hsb2.b = hsb.b>80 ? hsb.b-20 : hsb.b+20;
			hsb3.b = 100;
			hsb4.b = hsb2.b;

			return {
					baseColor0 : '#'+epColor.stringToHex(color),
					baseColor1 : '#'+epColor.hsbToHex(hsb1),
					baseColor2 : '#'+epColor.hsbToHex(hsb2),
					baseColor3 : '#'+epColor.hsbToHex(hsb3),
					baseColor4 : '#'+epColor.hsbToHex(hsb4)
				};
		};

	$.widget( 'ep.ui.Colorpicker', {

		options: {
		//	extended:			{},
			autoOpen:			false,
			orientation:		'right',
			orientationAdjust:	[0,0],
			color:				'#fff',
			autoApply:			false,
			callback:			$.noop
		},

		_create: function(){
			var self = this;

			self.elem = self.element
				.on( 'click', self.Proxy('open') )
				.sub(ep);

			self.originalColor = self._getColor().color;
		},

		_init: function(){
			var self = this
				o = self.options;

			self._colorSelect( undefined, true );

			if( o.autoOpen ){
				self.open();
			}
			if( o.autoApply && self.applyButton ){
				self.applyButton.hide();
			}
			if( o.extended && typeof o.extended.baseColors == 'string' ){
				o.extended.baseColors = combinedColors( o.extended.baseColors );
			}
		},

		_build: function(){
			if( !this.dialog ){
				var self = this,
					o = self.options,
					color = self._getColor(),
					colorpicker = self.colorpicker = new ep.Colorpicker({
						margin:		10,
						size:		210,
						hex:		color.hex,
						callback:	function( hex ){
							self._setColor( hex );
						}
					});

				// load template and push named tmpl elements to current instance
				$.extend( self, $.tmpl( 'ep.ui.colorpicker', $.extend({transparent:color.string=='transparent'},o.extended) ).tmplItem('elements') );

				self.dialog
					.hide()
					.appendTo('body')
					.dictParse('ep.dict',true)
					.draggable({
						containment:	'window',
						handle:			'.ep-uiColorpicker-head'
					})
					.find('.ep-metaparse')
						.metaparse();

				self.pickerBox
					.append(colorpicker.elem);
				self.closeButton
					.on( 'click', self.Proxy('close') );
				self.active = self.previewBox
					.on('click',function(){
						self._setActive();
					})
					.css('background','#'+o.color);
				self.hexInput
					.on( 'changeValue', function(){
						var val = $(this).val();
						if( val = (epColor.stringToName(val) || epColor.stringToHex(val)) ){
							self._colorSelect( val );
						}
					});
				self.transparentInput
					.on('change',function(){
						self._colorSelect();
					});
				self.applyButton
					.on('click',function(){
						self._colorApply();
						self.close();
					});

				if( self.userColors ){
					self.userColors
						.children('li')
						.on('click',function(){
							var elem = $(this),
								input =  elem.children('input'),
								val = input.val();

							self._setActive( elem, function( color ){
								elem.css( 'background-color', color );
								input.val( color );
							});

							self._colorSelect( val, true );
						})
						.on('dblclick',function(){
							self.previewBox.triggerHandler('click');
						});
				}

				if( self.baseColors ){
					self.baseColors
						.children('li')
						.on('click',function(){
							var elem = $(this),
								input =  elem.children('input'),
								val = input.val();

							self._colorSelect( val );
						});
				}

				if( self.originalBox ){
					self.originalBox
						.on( 'click', function(){
							var elem = $(this),
								color = '#' + ep.color.stringToHex( elem.css('background-color') );

							self._colorSelect( color, true );
						});
				}

				self.previewBox.triggerHandler('click');

				self._init();
			}
		},

		_setActive: function( elem, callback ){
			elem = elem || this.previewBox;
			if( elem ){
				this.active.removeClass('active');
				this.active = elem.addClass('active');
				this.activeCallback = callback || $.noop;
			}
		},

		_getColor: function( color ){
			color = color || this.options.color;

			var ret =
				(this.transparentInput && this.transparentInput.is(':checked')) || color=='transparent' ?
				{
					hex:	'ffffff',
					name:	'transparent'
				} :
				{
					hex:	epColor.stringToHex( color ),
					name:	epColor.stringToName( color )
				};

			ret.string = ret.name || '#'+(ret.hex || 'ffffff');

			return ret;
		},

		_setColor: function( color, original ){
			var self = this,
				o = self.options;

			o.color = color = self._getColor(color).string;

			self.elem.trigger('colorSelect',[color]);

			if( original && self.originalBox ){
				self.originalBox
					.css( 'background', color )
					.children()
						.css( 'color', ep.color.stringToHsb(color).b < 60 ? '#fff' : '#000' );
			}
			if( self.previewBox ){
				self.previewBox.css('background',color);
				self.activeCallback(color);
			}
			if( self.hexInput ){
				self.hexInput.val(color);
			}
			if( o.autoApply ){
				self._colorApply(color);
			}
		},

		_colorSelect: function( color, original ){
			color = this._getColor(color);
			if( color ){
				this._setColor(color.string, original);
				if( this.colorpicker ){
					this.colorpicker.setHex( color.hex, true );
				}
			}
		},

		_colorApply: function( color ){
			if( this.transparentInput ){
				this.transparentInput.attr('checked', color=='transparent' );
			}
			if( color = this._getColor(color).string ){
				this.originalColor = o.color = color;
				this.options.callback(color);
				this.elem.trigger('colorApply',[color]);
			}
		},

		_colorTransparent: function( color ){
			if( this.transparentInput ){
				this.transparentInput.attr('checked', color=='transparent'  );
			}
		},

		colorSelect: function( color, original ){
			this._setActive();
			this._colorTransparent( color );
			this._colorSelect( color, original );
		},

		colorApply: function( color ){
			this._setActive();
			this._colorTransparent( color );
			this._colorApply( color );
		},

		colorReset: function(){
			this.elem.trigger('colorRest',[this.originalColor]);
			this._setColor( this.originalColor );
		},

		open: function(){
			var self = this
				o = self.options;

			// close other colorpicker
			$('.ep-uiColorpicker-dialog').hide();

			self._build();
			self.dialog
				.show()
				.contextOrientation( self.elem, o.orientation, o.orientationAdjust );

			html.on( 'click.uiColorpicker', self.Proxy('_close') );

			this.elem.trigger('open');
		},

		close: function(){
			var self		= this,
				extended	= self.options.extended;

			if( this.dialog ){
				this.dialog.hide();
				if( extended && extended.userColorsSave ){
					extended.userColorsSave( this.userColorsForm.serializeArray() );
				}
				html.off( 'click.uiColorpicker', self.Proxy('_close') );
				this.elem.trigger('close');
			}
		},

		_close: function( event ){
			var self	= this,
				elem	= self.element,
				target	= $(event.originalTarget); 
			
			if( !jQuery.contains( elem[0], target[0] ) && target[0]!=elem[0] && !target.is('.ep-uiColorpicker-dialog, .ep-uiColorpicker-dialog *') ){
				this.close();
			}
		},

		destroy: function(){
			if( this.dialog ){
				this.dialog.remove()
			}
			this.Inherited(arguments);
		}

	});

});