/*
 * @copyright		© Copyright 2006-2011, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.contentlist
 *
 * @revision		$Revision: 1.10.2.4 $
 */
jQuery.plugin('ep.ui.contentlist',
	['ep.core', 'jQuery.ui.widget', 'jQuery.event.special.load', 'ep.fn.contextOrientation', 'ep.ui.textedit', 'ep.ui.validate'],
	function($, undefined){

	$.widget('ep.ui.contentlist', {
		STR_EDIT_ICON: '<i class="Sprite ico_s_edit"></i>',

		options: {
		//	grossPrice:	'c2',						// format for net/gross prices
			editIcon: 	true,						// indicator if a edit icon should be shown
			alterIcon: 	false,						// flag if the edit icon should be shown always right (false) or depending by align (true)
			region:		$.i18n.settings.region,		// region string
			currency:	$.i18n.settings.currency,	// currency acronym
			saveAction: 'Save'						// save method
		},

		_create: function(){
			var self = this,
				o = self.options;

			if(self.element[0]==undefined){
				return;
			}

			// get the currency format (default 'c2': two decimal places)
			o.grossPrice = (o.grossPrice===undefined)? 'c2' : o.grossPrice;

			self._rows = self.element[0].rows;
			self.element.addClass('RowSelection');

			// iterate over all rows
			for(var i=0, iLength=self._rows.length; i<iLength; i++){
				var aCells = self._rows[i].cells || [];

				// check all cell
				if(i==0){
					$('input', aCells[0]).on('change.uiContentlist', $.proxy(self._checkAll, $('input', aCells[0])));
				}

				// bind click handler for the current row
				ep(self._rows[i]).on('click.uiContentlist', $.proxy(self._handleRowClick, self._rows[i]));

				// iterate over all cells
				for(var j=0, jLength=aCells.length; j<jLength; j++){
					var $cell = $(aCells[j]);

					// if checkbox-cell -> add class
					if($cell.hasClass('Checkbox') && ($('input.Checkbox', $cell).length>0)){
						$cell.addClass('RowSelectionInputCell');
					}

					// if the current cells content should be editable
					if($cell.data('epEditable')==true){
						// element which should be editable
						var $oEditWrapper = ep('*[data-ep-attribute]', $cell);

						if($oEditWrapper.length>0){
							$oEditWrapper.addClass('epEditWrapper ClickIcon');
							// if an edit icon should be shown
							if(o.editIcon){
								if(!o.alterIcon || $cell.css('text-align')!="right"){
									$oEditWrapper
										.append(self.STR_EDIT_ICON)
										.data('epTextAlign', 'left');
									if($cell.css('text-align')=="right"){
										$cell.addClass('ep-Textedit-right');
									}
								}else{
									$oEditWrapper
										.prepend(self.STR_EDIT_ICON)
										.data('epTextAlign', 'right');
									$cell.addClass('ep-Textedit-right');
								}
							}
							switch($oEditWrapper.data('epAttribute').toLowerCase()){
								case "name":
									$oEditWrapper.css({
										display: 'inline-block'
									});
									break;
								// if price and stocklevel wrapper need special formatting later
								/*case "price":
									break;
								case "stocklevel":
									break;*/
							}
						}
					}

					// bind a click handler to the current cell
					$cell.on('click.uiContentlist', $.proxy(self._handleCellClick, {cell: $cell, _this: self}));
				}
			}
		},

		_handleCellClick: function(evt){
			// this: object/hash {cell: current cell, _this: reference to the widget}

			var $widget = this._this,
				o = $widget.options,
				$cell = this.cell,
				$currentRow = $cell.parent();

			// if the cell contents an editable element
			if($cell.data('epEditable')){
				if( $(evt.target).is('input') ){
					evt.stopPropagation();
					return;
				}

				// element which should be editable
				var $span = ep(".epEditWrapper", $cell);

				// if the element is not editable yet
				if(ep('input', $cell).length==0){
					// object/hash for the uiValidate widget
					var oValidate = {
						valid: false
					};
					// mark that the element is just created
					$span.prop('justCreated', true);

					switch($span.data('epAttribute')){
						case 'Name':						// value of the data-ep-attribute correlate to the Save attribute
							oValidate.maxlength = 100;
							break;
						case 'Price':						// value of the data-ep-attribute correlate to the Save attribute
							oValidate.type = 'number';
							oValidate.format = o.grossPrice;
							oValidate.region = o.region;
							oValidate.currency = o.currency;
							oValidate.min = -2147483648;
							oValidate.max = 2147483647;
							oValidate.needFormat = true;
							break;
						case 'StockLevel':					// value of the data-ep-attribute correlate to the Save attribute
							oValidate.type = 'number';
							oValidate.format = 'n2';
							oValidate.region = o.region;
							oValidate.min = -2147483648;
							oValidate.max = 2147483647;
							break;
					}

					// object/hash for the uiTextedit widget
					var oTextEditOptions = {
						action		: $widget.options.saveAction, //'Save',																	// SaveAction
						objectId	: ep('.RowSelectionInputCell input.Checkbox', $cell.parent()).val(),	// objectId
						name		: $span.data('epAttribute'),												// name/value pair (from data-ep-attribute)
						stripHTML	: true,
						validate	: oValidate,

						// callback called before element becomes editable
						onBeforeOpen: function(input){
							// remove edit icon if necessary
							if($widget.options.editIcon){
								ep('.Sprite', $span).remove();
							}
							// show label element
							input.parent().show();
							input.attr('disabled', false);

							$span.css('width','98%');
							$cell.find('*.hideOnEditable').hide();
						},

						onAfterOpen: function(input){
							// if element was just created
							if($span.prop('justCreated')){
								$span.prop('justCreated', false);
								// prevent an invalid message if input is empty
								if(input.val().length==0){
									input[0].formInvalid = false;
									input.blur();
									$span.trigger('click.uiTextedit');
									//input.focus();
								}
							}
						},

						// callback called before data get saved
						formatOnWriteBack: function(input, evt){
							// if enter or escape were pressed -> prevent default action
							if((typeof evt!="undefined") && ((evt.keyCode==13)||(evt.keyCode==27))){
								$span.hide();

								// check if input was valid
								var invalid = input.is(':input:invalid');

								// short delay for webkit-engines to prevent default action
								window.setTimeout(function(){
									$widget['_setContent'](input, $span, (invalid ? false : true));

									if(!invalid){
										$span.show();
										input.parent().hide();
									}else{
										$span.hide();
										input.show();
									}
								}, 10);

								evt.preventDefault();
							}

							// format inputs value
							var value = (oValidate.needFormat===true && oValidate.type=="number" && !isNaN( $.i18n.parseNumber(input.val())))
								? $.i18n.formatNumber($.i18n.parseNumber(input.val()), oValidate.format, {currency: $widget.options.currency, region: $widget.options.region})
								: input.val();
							input.val(value);
						},

						// called after element becomes uneditable
						onAfterClose: function(input){
							$widget['_setContent'](input, $span, true);

							// hide label element
							input.parent().hide();

							$span.css('width','');
							$cell.find('*.hideOnEditable').show();
						}
					};

					// if additional parameters are needed for the SaveAction
					if($span.data('epAdditional')!==undefined){
						oTextEditOptions.additional = {};
						// e.g. oTextEditOptions.additional['CurrencyID'] = "EUR"
						oTextEditOptions.additional[$span.data('epAdditional')] = this._this.options.currency;
					}

					// if additional parameters are needed for the SaveAction
					if($span.data('epOwnprice')!==undefined){
						oTextEditOptions.additional = (oTextEditOptions.additional===undefined)? {} : oTextEditOptions.additional;
						// e.g. oTextEditOptions.additional['HasSubOwnPrices'] = 1
						oTextEditOptions.additional[$span.data('epOwnprice')] = 1;
					}

					// build the uiTextedit widget
					var textEdit = $span.uiTextedit(oTextEditOptions);

				}

				// trigger click event to show the editable element
				$span.trigger('click.uiTextedit');

				evt.stopPropagation();
			}
		},

		_handleRowClick: function(evt){
			// this: current table row

			var $row = ep(this);
			var realTargetTag = evt.target.tagName.toLowerCase();

			$row.toggleClass('RowSelected');

			if((ep(evt.target).hasClass("RowSelector") && !ep(evt.target).hasClass("NoRowSelector")) || (
				//!someThingSelected &&
				realTargetTag != "a" &&
				realTargetTag != "input" &&
				realTargetTag != "select" &&
				realTargetTag != "option" &&
				realTargetTag != "textarea" &&
				realTargetTag != "img" &&
				realTargetTag != "label" &&
				!ep(evt.target).hasClass("NoRowSelector") &&
				!ep(evt.target).hasClass("Sprite"))
			){
				var $checkbox = ep('input.Checkbox', $row);
				// if checkbox is not disabled -> toggle check attribute
				if(!$checkbox.prop('disabled')){
					$checkbox.attr({'checked': ($row.hasClass('RowSelected')? true : false)});
				}
			}
		},

		_setContent: function(input, $span, isDisabled){
			var self = this;

			input.attr({
				'disabled': isDisabled
			});
			// if an edit icons should be shown
			if(self.options.editIcon){
				if(!self.options.alterIcon || $span.data('epTextAlign')!="right"){
					$span.append(self.STR_EDIT_ICON);
				}else{
					$span.prepend(self.STR_EDIT_ICON);
				}
			}
		},

		_checkAll: function(evt){
			// this: table cell which fires the event

			var form = ep(this)[0].form;
			var isChecked = ep(this).prop('checked');
			ep('tbody:first .RowSelectionInputCell input[type=checkbox]', form).attr({
				checked: isChecked
			});
			//toggle row highlight class
			ep('tbody:first>tr[class!=NewEntry]', form)[(isChecked? 'addClass' : 'removeClass')]('RowSelected');
		}
	});
});