/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.form
 *
 * @revision		$Revision: 1.9.4.2 $
 */

jQuery.plugin( 'ep.ui.form', ['ep.core', 'ep.ajax', 'jQuery.fn.form', 'jQuery.ui.widget'], function( $, undefined ){

	$.widget('ep.ui.Form',{

		options: {
			compareDiff: false
		},

		_create: function(){
			this.elem = this.element.sub(ep);
			
			this.timeStamp = $.now();
		},

		_init: function(){
			this.elem
				.off('.uiForm')
				.on( 'submit.uiForm', this.Proxy('_submit') )
				.on( 'change.uiForm', this.Proxy('_change') )
				.on( 'validate.uiForm', this.Proxy('_validate') )
				.on( 'reset.uiForm', this.Proxy('_reset') );
		},

		_validate: function(event){
			if( !this.elem.attr('formnovalidate') && event.target === this.elem[0] ){
				var elems = this.elem
					.formInput()
					.trigger('validate')
					.filter(':invalid:first')
					.focus()
					.trigger('validate');

				this._change();
			}
		},

		isValid: function(){
			return !this.elem[0].formInvalid;
		},

		setValid: function( stat ){
			this.elem[0].formInvalid = !stat;
		},

		_isValid: function( elems ){
			var valid = !elems
				.filter(':invalid')
				.length;

			this.elem[0].formInvalid = !valid;

			return valid;
		},

		_isChanged: function( elems ){
			return !!elems
				.filter(':changed')
				.length;
		},
		
		_submitCheck: function( event ){
			if( event.isDefaultPrevented() ){
				this.elem.trigger('submitFail');
			}
			this.elem.off('submit.uiForm', this.Proxy('_submitCheck'));
		},

		_submit: function( event ){
			// control last event handle of submit call
			if( !event.uiFormTriggert ){
				event.stopImmediatePropagation();
				event.preventDefault();
				this.elem
					.on('submit.uiForm', this.Proxy('_submitCheck'))
					.trigger({ type:'submit', uiFormTriggert:true });
				return;
			}
			
			// trigger validation
			this.elem.trigger('validate');

			var valid = this._isValid( this.elem.formInput() ),
				action = this.elem.attr('action'),
				ajax = this.options.ajax;
				
			if( !action || !valid || ajax || (event.timeStamp - this.timeStamp)<300 ){
				event.preventDefault();
			}
			this.timeStamp = event.timeStamp;

			if( !valid ){
				return;
			}

			if( ajax ){
				ep.ajax(
					$.extend(true,{
						url:		this.elem.attr('action') || '',
						type:	this.elem.attr('method') || 'post',
						data:		this.elem.serializeArray()
					},ajax)
				);
			}
		},

		_reset: function( event ){
			this.elem
					.formInput()
					.each(function(){
						$(this).triggerHandler('reset');
					});
		},

		_change: function(){
			var elems = this.elem.formInput(),
				unchanged = !this._isChanged(elems),
				invalid = !this._isValid(elems);

			elems
				.filter(':submit')
				.attr( 'disabled', invalid || (unchanged && this.options.compareDiff) );
		}

	});

});
