/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.input
 *
 * @revision		$Revision: 1.23.8.1 $
 */

jQuery.plugin( 'ep.ui.input',{
	plugin: [
		'ep.core',
		'ep.dict',
		'jQuery.support.placeholder',
		'jQuery.support.multipleUpload',
		'jQuery.fn.form',
		'jQuery.fn.class',
		'ep.ui.tooltip'
	],
	dict: [
		'ep.dict'
	]
}, function( $, undefined ){

/*
 * @dictionary		ep.dict
 *
 * @translation		{SelectFile}
 */

	$.widget('ep.ui.Input',{

		options: {
		//	placeholder:		'text',
		//	info:				'text',
			autofocus:			false,
			big:				false
		},

		_create: function(){
			// get elem with ep API
			this.elem = this.element
				.data( 'ep-uiInput', this )
				.sub(ep);
			this.stack = $([]);

			// get instance of dict
			this.dict = ep.dict('ep.dict');

			// get type
			var type = this.elem[0].tagName.toLowerCase()+':';
			type = this.type = type==='input:' || type==='button:' ? type+this.elem[0].type : type;

			// create a label as wrap
			this.wrap = ep('<label>')
				.insertBefore(this.elem)
				.append(this.elem);
			// add wrap to stack
			this.stack.push(this.wrap[0]);

			var self = this,
				// state event
				activeStart = function(){
					self.addClass('active');
					self.wrap
						.on( 'mouseenter.uiInput', activeEnter )
						.on( 'mouseleave.uiInput', activeLeave );
					$('html').on( 'mouseup.uiInput', activeEnd );
				},
				activeEnter = function(){
					self.addClass('active');
				},
				activeLeave = function(){
					self.removeClass('active');
				},
				activeEnd = function(){
					self.removeClass('active');
					self.wrap
						.off( 'mouseenter.uiInput', activeEnter )
						.off( 'mouseleave.uiInput', activeLeave );
					$('html').off( 'mouseup.uiInput', activeEnd );
				},
				// classes
				elemClasses = '',
				wrapClasses = 'ep-uiInput-wrap'
					+ ' ep-uiInput-wrap'+$.camelCase( '-'+( /((?:[a-z]+$)|^(?:a|button|select|textarea))/.exec(type) || ['',''] )[1] )
					+ ( this.elem.is(':disabled') ? ' disabled' : '' )
					+ ( this.elem.is(':readonly') ? ' readonly' : '' )
					+ ( this.elem.is(':checked') ? ' checked' : '' );

			this.elem
				.on( 'changeAttr.uiInput', this.Proxy('_changeAttr') )
				.on( 'reset.uiInput', this.Proxy('_change') )
				.on( 'focusin.uiInput', this.Proxy(function(){
					this.addClass('focused');
					if( this.tooltip ){
						this.tooltip[ this.tooltip.is(':empty') ? 'hide' : 'show' ]();
					}
				}) )
				.on( 'focusout.uiInput', this.Proxy(function(){
					this.removeClass('focused'); this._leave();
					if( this.tooltip ){
						this.tooltip.hide();
					}
				}) );

			if( !(/hidden$/i).test(type) ){
				this.wrap
					.on( 'mousedown.uiInput', activeStart );
			}

			if( (/file$/i).test(type) ){
				elemClasses += ' ep-uiInput-hidden';
				// create a input to display selected file name
				this.fileText = $('<span>')
					.attr({
						'class':	'ep-uiInput ep-uiInput-base ep-uiInput-text epWidth140 epWidth-4',
						'text':		this.elem.val()
					},true)
					.insertBefore(this.elem);
				// insert no-break space if no content is available
				if(this.elem.val()==""){
					this.fileText.html('&nbsp;');
				}
				// create a button
				this.fileButton = ep('<span>')
					.attr({
						'class':	'ep-uiInput ep-uiInput-base ep-uiInput-button',
						'text':		this.dict.translate('SelectFile')+'...'
					},true)
					.insertBefore(this.elem);
				// add to stack
				this.stack.push( this.fileText[0], this.fileButton[0] );
			}

			if( this.options.big ){
				wrapClasses += ' ep-uiInput-big';
			}
			if( (/(^(textarea:|select:)|(text|password|email|url|tel|date|datetime|time|number)$)/).test(type) ){
				elemClasses += ' ep-uiInput ep-uiInput-base ep-uiInput-field';
			}

			if( (/checkbox$/).test(type) ){
				wrapClasses += ' ep-uiInput ep-uiInput-base ep-uiInput-field ep-uiInput-checkbox';
				elemClasses += ' ep-uiInput-hidden';
			}
			else if( (/radio$/).test(type) ){
				wrapClasses += ' ep-uiInput ep-uiInput-radio';
				elemClasses += ' ep-uiInput-hidden';
			}
			else if( (/^select:/).test(type) ){
				elemClasses += ' ep-uiInput-select ';
			}
			else if( (/(^(button:|a:)|(submit|reset|button|image)$)/).test(type) ){
				elemClasses +=		(/image$/).test(type) ? '' : ' ep-uiInput ep-uiInput-base ep-uiInput-button';
				// handle submit and error on form
				if( (/(submit|reset|image)$/).test(type) ){
					this.elem
						.on('click.uiInput',function(event){
							event.preventDefault();
							$(this.form)[ (/(submit|image)$/).test(type) ? 'submit' : 'formReset' ]();
						});
				}
			}
			else if( (/^textarea:$/).test(type) ){
				elemClasses += ' ep-uiInput-textarea epWidth-2';
			}
			else if( (/(text|password|email|url|tel|date|datetime|time|number)$/i).test(type) ){
				elemClasses += ' ep-uiInput-text epWidth-4';
			}
			// change event
			if( (/(^(select:)|(file|checkbox|radio)$)/).test(type) ){
				this.elem.on( 'change.uiInput', this.Proxy('_change') );
			}
			// key event
			if( (/(^(textarea:)|(text|password|checkbox|radio|email|url|tel|date|datetime|time|number)$)/).test(type) ){
				this.elem.on( 'keyup.uiInput', this.Proxy('_change') );
			}

			// add classes for styling
			this.elem.addClass( elemClasses );
			this.wrap.addClass( wrapClasses );

			// save value state
			this.lastState = [ this.elem[0].value ,this.elem[0].checked, this.elem[0].selectedIndex ];
		},
		// init binds
		_init: function(){

			// init placholder
			this._placeholderInit();

			// init tooltip
			this._tooltipInit();

			// handle autofocus for this elem
			if( this.elem.attr('autofocus') || this.options.autofocus ){
				if( this.placeholder ) {
					this._placeholderHide();
				}
				this.elem[0].focus();
			}
		},
		// handle addClass to the wrap
		addClass: function( name ){
			if( !(/(hover|active|focused)/).test(name) || !this.elem.is(':disabled,:readonly') ){
				this.wrap.addClass(name);
			}
		},
		// handle removeClass to the wrap
		removeClass: function( name ){
			this.wrap.removeClass(name);
		},

		_placeholderInit: function(){
			if( (/(text|password|email|url|tel|date|datetime|time|number)$/i).test(this.type) ){
				var valOpt = this.options.placeholder,
					valAttr = this.elem.attr('placeholder'),
					val = valOpt || valAttr;

				if( !$.support.placeholder ){
					if( !this.placeholder && val ){
						// bind
						this.elem
							.on( 'focus.uiInput', this.Proxy('_placeholderHide'))
							.on( 'blur.uiInput', this.Proxy('_placeholderShow') );
						// create a placeholder if not support by browser
						this.placeholder = $('<input>') // <input type="text"> does not work for ie default of input is text
							.attr({
								'class': this.elem.attr('class') + ' ep-uiInput-placeholder',
								'style': this.elem.attr('style')
							})
							.hide()
							.on( 'focus.uiInput', this.Proxy('_placeholderHide') )
							.insertAfter(this.elem);
						// add to stack
						this.stack.push(this.placeholder[0]);
					}
					if( this.placeholder ){
						this.placeholder
							.prop({
								value:			val || '',
								defaultValue:	val || ''
							});
						this._placeholderShow();
					}
				}
				else{
					if( val !== valAttr ){
						this.elem.attr( 'placeholder', val );
					}
				}
			}
		},

		_placeholderHide: function( event ){
			var self = this;
			if( this.placeholder ){
				var hidden = this.elem.is(':hidden');
				this.placeholder
					.hide()
					.attr('disabled',true);
				this.elem.show();

				if( event && hidden ){
					// prevent conflict of event trigger with validate #EPG-18640
					setTimeout(function(){
						self.elem[0].focus();
					},1);
				}
			}
		},

		_placeholderShow: function( event ){
			if( this.placeholder && this.placeholder.is(':hidden') ){
				if( !this.elem.val() ){
					this.elem.hide();
					this.placeholder
						.attr('disabled',false)
						.show();
				}
				else{
					this._placeholderHide( event );
				}
			}
		},

		_tooltipInit: function( buildManual ){
			if( !this.tooltip && (this.options.info || buildManual) ){
				// create a placeholder if not support by browser
				this.tooltip = ep('<div>')
					.addClass('ep-uiInput-info')
					.uiTooltip({
						interactive:	true,
						event:			'focus',
						context:		this.elem,
						orientation:	'right',
						offsetAdjust:	[3,-2]
					});
				if( this.options.info ){
					this.tooltip.html(this.options.info);
				}
				// add to stack
				this.stack.push(this.tooltip[0]);
			}
		},

		_setOption: function( key, value ) {
			if( key === 'placeholder' ){
				this.elem.attr( 'placeholder', value );
			}

			return this.Inherited(arguments);
		},

		_radioGroupCheck: function(){
			this.elem
				.formGroup(':radio')
				.uiInput('removeClass','checked')
				.filter(':checked')
					.uiInput('addClass','checked');
		},

		// actions if change attr by javascript
		_changeAttr: function( event, attr ){
			if( attr.hasOwnProperty('checked') ){
				// change class to all radios on the current set
				if( (/radio$/i).test(this.type) ){
					this._radioGroupCheck();
				}
				// change class for checkbox
				else{
					this[ this.elem.is(':checked') ? 'addClass' : 'removeClass' ]('checked');
				}
			}
			// change disabled class
			if( attr.hasOwnProperty('disabled') ){
				if( this.elem.is(':disabled') ){
					this.addClass('disabled');
					this.removeClass('focused');
				}
				else{
					this.removeClass('disabled');
				}
			}
			// change readonly class
			if( attr.hasOwnProperty('readonly') ){
				this[ this.elem.is(':readonly') ? 'addClass' : 'removeClass' ]('readonly');
			}
			if( attr.hasOwnProperty('placeholder') ){
				this._placeholderInit(true);
			}
			if( attr.hasOwnProperty('value') ){
				this._change({});
			}
		},

		_stateChanged: function(){
			var oldState = this.lastState,
				newState = [ this.elem[0].value ,this.elem[0].checked, this.elem[0].selectedIndex ],
				ret = !( oldState[0] === newState[0] && oldState[1] === newState[1] && oldState[2] === newState[2] );

			this.lastState = newState;

			return ret;
		},

		_change: function( event ){
			if( event.keyCode!==9 ){
				// special for checkbox
				if( (/checkbox$/i).test(this.type) ){
					this.elem
						.trigger( 'changeAttr', {'checked':this.elem.is(':checked')} );
				}
				// special for radios
				else if( (/radio$/i).test(this.type) ){
					this.elem
						.formGroup(':radio')
							.filter(':checked')
								.trigger( 'changeAttr', {'checked':false} )
							.end()
							.filter(':not(:checked)')
								.trigger( 'changeAttr', {'checked':true} );
				}
				// display selected file
				else if( (/file$/i).test(this.type) ){
					if(this.elem.val()!=""){
						this.fileText.text( this.elem.val().replace(/^.*?fakepath[\/\\]?/,'') );
					}else{
						this.fileText.html('&nbsp;');
					}
				}
			}

			this._leave();
		},

		_leave: function(){
			this[ this.elem.is(':changed') ? 'addClass' : 'removeClass']('changed');

			if( this._stateChanged() ){
				this.elem.trigger('changeValue');
				// trigger changed to form
				$(this.elem[0].form).trigger('change');
			}
		},

		destroy: function(){
			this.elem
				.insertBefore( this.wrap )
				.removeClass(/(ep-uiInput[\w\d-]*|epWidth-\d+)/g);
			this.stack.remove();

			return this.Inherited(arguments);
		}
	});

});
