jQuery.plugin( 'ep.ui.magnifier', ['ep.core', 'jQuery.ui.widget', 'jQuery.event.special.load', 'ep.fn.contextOrientation'], function( $, undefined ){

	$.widget('ep.ui.Magnifier', {
		
		// indicates whether the mouseenter event must be triggered manually at the end of _setImageDetails
		// (necessary when preview src changed without using setImage)
		trigger: false,
		_isPlaying: 0,				// indicates whether the combined slideshow is playing
		_magWrapper: undefined,
		_magTop: undefined,
		_magLeft: undefined,
		_magImg: undefined,			// var for the temporary magnified image
		_magMove: undefined,		// var for the moveable magnifier element
		_magDiv: undefined,			// var for the div where the magnified image is in
		_magDistance: 15,			// distance between product image to magnified image
		_correct: 0,				// summand to correct movers vertical position
		_ratio: 0,					// ratio for vertical calculation
		
		// default options
		options: {
		  // source-path to image
			magnifiedPicture:	'',
			// current position of mouse in image
			mouseX		:	0,
			mouseY		:	0,
			// properties of image
			imgWidth	:	0,
			imgHeight	:	0,
			imgWidthMag	:	0,
			imgHeightMag:	0,
			imgPosLeft	:	0,
			imgPosTop	:	0,
			sPattern	:	'_m',						// part of src which identify a preview image (remove to get big product image)
			// properties of magnified image
			magWidth 	: 200,
			magHeight	: 200,
			fadeDelay	: 200,
			magId		: 'epMagnifier_'+new Date().getTime(),
			magBorder	: '1px solid #aaa',
			magBG		: '#fff',
			magOpac		: 0.6,
			// properties of magnifier
			moveId		: 'epMagMover_'+new Date().getTime(),
			_src		: '',
			moveCorrect : 0,
			// styles/properties for magWrapper
			zIndex	: 9999,
			dir		: 'right',
			// slideshow (ext. widget)
			slideshow: null		// may contain a combined slide show widget object
		},

		_create: function() {
			
			if( this.options.magnifiedPicture ) {
				var self = this,
					o = this.options;
				
				o.magId += o.id;
				o.moveId += o.id;
				
				// calculation of the correction term was moved to self._magImg.on('load.magnifier', ...) (due to a timing problem in safari)
				//self._correct = o.moveCorrect;

				// wrapper for the magnifier
				self._magWrapper = $('body');
				
				if(o.slideshow != null){
					o.dir = o.slideshow.options.magDir;
				}

				this._createTempMagImg(true);
			}
		},
		
		_createTempMagImg: function(/* BOOLEAN */ initial){
			var self = this,
			o = this.options;
			
			ep(self._magImg).remove();
			
			// create temporary node for magnified image
			self._magImg = $('<img src="'+o.magnifiedPicture+'" id="img_'+o.magId+'"/>').css({display: '', position: 'absolute', top: -10000, left: -10000}).appendTo('body');

			// fires when magnified image is loaded
			self._magImg.on('load.magnifier', function(){
				self._setImagePreferences(initial);
				
				// if there are top margins in the first elements -> correct term for the magnifier mover
				self._correct = 0;
				if (!($.browser.msie  && parseInt($.browser.version) <= 7)){
					// if document is not in an (i)frame
					if(window.parent === window){
						var divCollection = ep('div'),
						    correctMarginTop;
						// check the first and second div (wrapper and header)
						for(var i=0; i<2; i++){
							correctMarginTop = parseInt(divCollection.eq(i).css('margin-top'));
							self._correct -= isNaN(correctMarginTop)? 0 : correctMarginTop;
						}
					}
				}
			});
		},

		_setImagePreferences: function(/* BOOLEAN */ initial){
			
			var self = this,
				elem = self.element;
			
			// img is already loaded
			if(elem[0].height>0 && elem[0].width>0){  // complete width
				self._setImageDetails(initial);
			// img is not loaded yet
			}else{
				elem.on('load.magnifier', $.proxy(this._setImageDetails(initial), this));
				/*elem.on('load.magnifier', function(){
					self._setImageDetails(initial);
				});*/
			}
		},

		_setImageDetails : function(/* BOOLEAN */ initial){
			
			var self = this,
				o = self.options,
				elem = self.element;

			o['_src'] = elem.attr('src');
			// position & dimensions of image
			o.imgWidth = elem.width();
			o.imgHeight = elem.height();
			var tmpOffset = elem.offset();
			o.imgPosLeft = Math.round(tmpOffset.left);
			o.imgPosTop = Math.round(tmpOffset.top);
		
			// position & dimensions of magnifier image
			o.imgWidthMag = $(self._magImg).width();
			o.imgHeightMag = $(self._magImg).height();
			self._magImg.remove();
	
			// ratio mag-img/src-img
			o.ratio = o.imgWidthMag/o.imgWidth;
			o.ratioX = o.imgWidthMag/o.imgWidth;
			o.ratioY = o.imgHeightMag/o.imgHeight;
	
			// dimensions for mover-div
			o.moverWidth  = Math.floor(o.magWidth/o.ratioX);
			o.moverHeight = Math.floor(o.magHeight/o.ratioY);
			
			if(o.moverWidth>o.imgWidth){
				var hlpRatio = o.imgWidth/o.moverWidth;
				o.moverWidth = o.imgWidth;
				o.moverHeight *= hlpRatio;
			}
			if(o.moverHeight>o.imgHeight){
				var hlpRatio = o.imgHeight/o.moverHeight;
				o.moverHeight = o.imgHeight;
				o.moverWidth *= hlpRatio;
			}
			
			// vertical ratio
			self._ratio = (o.imgHeightMag-o.magHeight)/(o.imgHeight-o.moverHeight);
			
			// create magnifier
			this._buildMagnifier();
		
			elem.on('mouseenter.magnifier', $.proxy(this._showMagnifier, this));
			
			// fire event when preview src changed without using setImage
			if(this.trigger){
				this.trigger = false;
				elem.trigger('mouseenter.magnifier');
			}
		},

		_buildMagnifier: function() {
			
			var self = this,
				elem = self.element,
				o = self.options;

			if(self._magDiv===undefined){
				self._magDiv = $('<div>')
					.hide()
					.attr({
						'id' : o.magId
					})
					.css({
					  'position': 'absolute',
					  'width': o.magWidth,
					  'height': o.magHeight,
					  'background': "#fff url(" + o.magnifiedPicture + ") 0 0 no-repeat",
					  'border': '1px solid #ccc',
					  'box-shadow': '2px 2px 4px #ccc',
					  'z-index': o.zIndex
					})
					.appendTo('body');
			}else{
				self._magDiv.css({
					'background': "#fff url(" + o.magnifiedPicture + ") 0 0 no-repeat"
				});
			}

			// left position of the magnified image
			if((this._magLeft === undefined) && !elem.is(':hidden')){
				var $context = (o.slideshow != null)? elem.parent() : elem;
				var contextOffset = $context.offset();

				self._magLeft = -parseInt(contextOffset.left+$context.width()+((o.dir == 'left')
					?(-o.imgWidth-self._magDistance)
					:((o.dir=='center')
						? 0.5*(o.magWidth-o.imgWidth)
						: self._magDiv.width()+self._magDistance)));
			}

			// top position for the magnified image
			if((self._magTop === undefined) && !elem.is(':hidden')){
				if(o.slideshow!=null){
					self._magTop = o.slideshow.element.offset().top + self._correct + ((o.dir=='center')? o.slideshow['_slideshowBox'].height() : 0);
				}else{
					self._magTop = contextOffset.top + self._correct + ((o.dir=='center')? elem.height() : 0);
				}
			}

			// positioning of magnified image
			ep(self._magDiv).contextOrientation(self._magWrapper, 'left', [self._magLeft, self._magTop]);
		
			if(self._magMove!==undefined){
				self._magMove.remove();
				self._magMove = undefined;
			}
				
			self._magMove = $('<div>')
				.hide()
				.attr({
					'id' : o.moveId
				})
				.css({
				  'position': 'absolute',
				  'top': -10000,
				  'left': -10000,
				  'width': Math.floor(o.moverWidth)-1,
				  'height': Math.floor(o.moverHeight)-1,
				  'z-index' : o.zIndex-1,
				  'border'	: o.magBorder,
				  'opacity'	: o.magOpac,
				  'background': o.magBG
				})
				.on('mouseleave', function(evt){
					if(evt.relatedTarget != elem){
						$(this).hide();
						self._magDiv.stop(true, true).fadeOut(o.fadeDelay);
						
						elem.one('mousemove.magnifier', $.proxy(self._showMagnifier, self));
						$(document).off('mousemove.magnifier');
					}
				})
				.appendTo('body');
			
			// bind click event to magnifier
			if(o.slideshow != null){
				self._magMove.on('click', function(evt){
					// hide magnifier elements
					self._magDiv.hide();
					self._magMove.hide();
					// show popup image
					o.slideshow['_onClickPopoutButton'](evt);
				});
			}
		},
		
		hideMover: function(){
			this._magMove.hide();
			this._magDiv.hide();
		},

		_showMagnifier: function(/* OBJECT */ event) {

			var self = this,
				elem = self.element,
				o = self.options;
			
			// content is still the same
			if(o['_src'] === elem.attr('src') ) {
				$(document).on('mousemove.magnifier',function(event){self._moveMagnifier(event);});
				elem.off('mouseenter.magnifier');

				self._magDiv.stop(true, true).fadeIn(o.fadeDelay);
				self._magMove.show();
			} else {
				// content of img-node has changed
				this.trigger = true;
				
				var strSrc = elem.attr('src'),
					strNewMagSrc = "";
				
				if(this.options.slideshow!=null){
					strNewMagSrc = o.slideshow['_imagesData'][o.slideshow['_currentImgId']].large;
				}else{
					var intPos = strSrc.lastIndexOf(o.sPattern);
					strNewMagSrc = strSrc.substring(0, intPos) + strSrc.substring(intPos+o.sPattern.length);
				}
				
				this.setImage(elem.attr('src'), strNewMagSrc);
				o['_src'] = strSrc;
				if($('#'+o.moveId).is(':hidden')){
					self._magMove.show();
				}
			}
		},

		_hideMagnifier: function(/* OBJECT */event){

			var self = this,
				elem = self.element,
				o = self.options;
				
			if($.contains(event.target, elem[0])){
				self._magDiv.hide();
				self._magMove.hide();
				
				elem.on('mouseenter.magnifier', $.proxy(this._showMagnifier, this));
				$(document).off('mousemove.magnifier');
			}
		},

		_checkMagnifier: function(/* OBJECT */ event, /* OBJECT */ objThis){
			
			var elem = objThis.element,
				o = objThis.options;

			// get current offset of image
			var tmpOffset = elem.offset();
			o.imgPosLeft = Math.floor(tmpOffset.left);
			o.imgPosTop = Math.floor(tmpOffset.top);
			// get position of mouse in picture
			o.mouseX = event.pageX - o.imgPosLeft;
			o.mouseY = event.pageY - o.imgPosTop;

			// mouse is over source image
			if( o.mouseX > 0 && o.mouseX < o.imgWidth
			 && o.mouseY > 0 && o.mouseY < o.imgHeight ) {
				return  true;
			} else {
				return false;
			}
		},

		_moveMagnifier: function(/* OBJECT */ event) {
			
			var self = this,
				o = self.options;

			if(this._checkMagnifier(event, this)){
				// Mouse is still over the picture
				// position in magnified image

				// Absolute vertical mouseposition in document
				var intTmp = o.imgPosLeft + o.mouseX;
				var tmpHalfW = o.moverWidth/2;
				// movers vertical position inside product image
				var left = (intTmp < o.imgPosLeft+o.imgWidth-o.moverWidth)? intTmp : o.imgPosLeft+o.imgWidth-o.moverWidth;
				left = (o.mouseX<tmpHalfW)? o.imgPosLeft : ((o.mouseX>=o.imgWidth-tmpHalfW)? o.imgPosLeft+o.imgWidth-o.moverWidth : intTmp-parseInt(tmpHalfW));
				
				// absolute horizontal mouseposition in document
				intTmp = o.imgPosTop + o.mouseY;
				var tmpHalfH = o.moverHeight/2;
				// movers horizontal position inside product image
				var top = (intTmp < o.imgPosTop+o.imgHeight-o.moverHeight)? intTmp : o.imgPosTop+o.imgHeight-o.moverHeight;
				top = (o.mouseY<=tmpHalfH)? o.imgPosTop : ((o.mouseY>=o.imgHeight-tmpHalfH)? o.imgPosTop+o.imgHeight-o.moverHeight : intTmp-parseInt(tmpHalfH));
				top += this._correct;
				
				// calculate background positions for magnified image
				var bgPosLeft = o.mouseX-tmpHalfW;
				bgPosLeft = (bgPosLeft < 0)
					? 0 
					: ((o.mouseX>o.imgWidth-tmpHalfW)
						? (o.imgWidthMag-o.magWidth < 0)? 0 : (o.imgWidth-o.moverWidth)
						: bgPosLeft);
				
				bgPosLeft = Math.round(bgPosLeft*o.ratioX);
				
				var bgPosTop = (o.mouseY-tmpHalfH)*self._ratio;
				bgPosTop = (o.mouseY-tmpHalfH < 0)
					? 0 
					: ((o.mouseY>o.imgHeight-tmpHalfH)
						? (o.imgHeightMag-o.magHeight < 0)? 0 : (o.imgHeightMag-o.magHeight) 
						: bgPosTop);
				bgPosTop = Math.round(bgPosTop);
				
				var backgroundPosition = '-' + bgPosLeft + 'px -' + bgPosTop + 'px';

				self._magMove.css({
					'left' : left + "px",
					'top' : top + "px"
				});
				self._magDiv.css({
					 'backgroundPosition' : backgroundPosition
				});
			} else {
				this._hideMagnifier(event);
			}
		},

		setImage: function( /* STRING */ newImgSource, /* STRING */ newMagImgSource) {
			
			if(newImgSource) {
				var self = this
					elem = self.element,
					o = self.options;

				// remove next line when using with slideshow
				elem.attr('src', newImgSource);
				if(newMagImgSource){
					o.magnifiedPicture = newMagImgSource;
				}else{
					o.magnifiedPicture = '';
				}
				
				// unregister load-event for temporary magnified image
				self._magImg.off('load.magnifier');

				this._createTempMagImg(false);
			}
		}
	});

});