/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.slideshow
 *
 * @revision		$Revision: 1.14.4.2 $
 */

jQuery.plugin( 'ep.ui.slideshow', {
	plugin: ['ep.core','jQuery.tmpl', 'jQuery.ui.widget','ep.fn.busy','ep.ui.dialog','jQuery.event.special.load','jQuery.effects.core','jQuery.effects.fade','jQuery.effects.slide','ep.ui.magnifier'],
	tmpl: [
		'ep.ui.slideshow',
		'ep.ui.slideshowPopup',
		'ep.ui.slideshowFisheye',
		'ep.ui.slideshowList',
		'ep.ui.slideshowMoviestrip'
	]
}, function( $, undefined ){

	$.widget('ep.ui.Slideshow',{
		_imagesData:null,
		_bindStack: null,
		_imageLoader:null,

		_slideshowContainerNode: null,
		_slideshowCreated: null,
		_slideshowBox: null,

		_imageFrontNode:null,
		_imageFrontImg: null,
		_imageBackNode: null,
		_imageBackImg: null,
		_currentImgId: null,
		_currentPopupImgId: null,

		_thumbsNode: null,

		_controlsNode: null,
		_prevButton: null,
		_playButton: null,
		_popoutButton: null,
		_nextButton: null,

		_popupNode: null,
		_popupImg: null,
		_popupPrevButton: null,
		_popupNextButton: null,

		_topNode: null,
		_descNode: null,
		_locNode: null,
		_bottomNode: null,
		_countNode: null,
		_dateNode: null,

		_isPlaying: null,
		_playInterval: null,
		_isLoading: null,

		_thumbnailClasses: {
			'small' : 'ep-uiSlideshow-ThumbnailSizeSmall',
			'medium': 'ep-uiSlideshow-ThumbnailSizeMedium',
			'large' : 'ep-uiSlideshow-ThumbnailSizeLarge'
		},

		options: {
			type : 'big',					//type of controls
			width: '300px',				//width of slideshow
			height: '450px',			//height of slideshow
			images: null,					//array of image data consists of hashes iwth the data for every imagee
														//example for an image {
														//									small: 'PathToSmallImage',
														//									medium: 'PathToMediumImage',
														//									large: 'PathToLargeImage',
														//									description : 'DesciptionOfImage',
														//									location: 'LocationOfImage',
														//									date: 'DateImageWasTaken'
														//								}
			thumbnails: null,			//should thumbnails be displayed
			showDelay: 3500,			//how long are the images visible in play mode
			controls: true,			//show controls or not
			start: false,					//start automatically
			showDescription: true,	//show Description of images if they are set
			magnifier: false,		// true if magnifier should be shown, false otherwise
			magDir: 'right'				// direction of the magnified image
		},

		_create: function(){
			this.Inherited(arguments);
			//set local variables
			this._imagesData = this.options.images;
			this._bindStack = $();
			this._currentImgId = 0;
			this._isPlaying = 0;
			this._imageLoader = new Image();
			this._imageLoader.onload = $.proxy(this,'_imageLoaderLoaded');
			this._slideshowCreated = false;

			//build Popup
			$.tmpl( 'ep.ui.slideshowPopup').appendTo(this.element);
			this._popupNode =  this.element.children('.ep-uiSlideshow-popup');
			this._popupNodeImgContainer = this._popupNode.children('.ep-uiSlideshow-popup-imgContainer');
			this._popupImg = this._popupNodeImgContainer.children('img');
			this._popupPrevButton = this._popupNode.find('.ep-uiSlideshow-popup-controlPrev');
			this._popupNextButton = this._popupNode.find('.ep-uiSlideshow-popup-controlNext');
			if(this._imagesData.length > 1){
				this._bindStack.push(
					this._popupPrevButton[0],
					this._popupNextButton[0]
				);
				this._popupPrevButton.on('click',$.proxy(this,'PopupShowPrevImage'));
				this._popupNextButton.on('click',$.proxy(this,'PopupShowNextImage'));
			}
			else{
				this._popupPrevButton.hide();
				this._popupNextButton.hide();
			}
			//create popup
			var self = this;
			ep(this._popupNode).uiDialog({
				modal: true,
				closeOnClickOut: true,
				autoOpen: false,
				dialogClass: 'ep-uiSlideshow-popup-dialog',
				resize: 'auto',
				position:'center',
				width: 'auto'
			});

		},
		_init: function(){
			if(this.options.magnifier){
				this.options.controls = false;
				this.options.start = false;
			}
		},
//internal helper functions

		_createSlideshow:function(){
			//set template
			var thumbs = [];
			if(this.options.thumbnails){
				for(var i = 0, iLength = this._imagesData.length; i < iLength; i++){
					var currentImg = this._imagesData[i];
					thumbs.push({
						img : currentImg.small
					});
				}
			}
			this._slideshowContainerNode.empty();
			$.tmpl( 'ep.ui.slideshow',[{
				image1 : this._imagesData[0].medium,
				image2 : this._imagesData[0].medium,
				thumbs: thumbs.length ? thumbs : undefined,
				baseUrl: ep.config.javascriptRoot
			}]).appendTo(this._slideshowContainerNode);

			//connect nodes & bind events
			this._slideshowBox = this._slideshowContainerNode.children('.ep-uiSlideshow-box');
			this._imageFrontNode = this._slideshowBox.children('.ep-uiSlideshow-front');
			this._imageFrontImg = this._imageFrontNode.children('img');
			this._imageBackNode =  this._slideshowBox.children('.ep-uiSlideshow-back');
			this._imageBackImg = this._imageBackNode.children('img');
			this._thumbsNode =  this._slideshowContainerNode.children('.ep-uiSlideshow-thumbbox');

			this._bindStack.push(
				this._slideshowBox[0],
				this._imageFrontNode[0],//first Element in jQuery array like object is dom node it self
				this._imageFrontImg[0],
				this._imageBackNode[0],
				this._imageBackImg[0],
				this._slideshowContainerNode[0]
			);

			var self = this;

			// init magnifier
			if(this.options.magnifier){
				var magnifierOptions = {
					'magnifiedPicture':this._imagesData[0].large,
					'magWidth': this.options.magWidth,
					'magHeight': this.options.magHeight,
					'magBG': this.options.magBG,
			        'magOpac': this.options.magOpac,
					'moveCorrect': -8,
					slideshow: this};
				magnifierOptions.id = 1;
				ep(this._imageFrontImg).uiMagnifier(magnifierOptions);
				magnifierOptions.id = 2;
				ep(this._imageBackImg).uiMagnifier(magnifierOptions);
			}

			//set inital styles
			this._imageBackNode.hide();
			this._imageFrontImg.one('load',function(){
				self._correctImgPosition(self._imageFrontImg,self._imageFrontNode,self._imageFrontImg.width(),self._imageFrontImg.height());
				self._setTexts();
				if(self.options.start){
					self._playButton.trigger('click');
				}
			});
			//set height and width of slideshow
			this._slideshowBox.css({
				width: this.options.width,
				marginBottom: (this.options.magnifier)? '3px' : '0',
				height: (parseInt(this.options.height) - ((this.options.magnifier)?3:0))+'px'
			});

			this._controlsNode =this._slideshowBox.children('.ep-uiSlideshow-controls'); //use children

			if(this.options.magnifier){
				this._controlsNode.children('ul:first').css({
					marginTop: '0px'
				});
			}

			if(this.options.type == 'big'){
				this._controlsNode.addClass('ep-uiSlideshow-controls-big');
			}
			if(this.options.controls){
				this._prevButton = this._controlsNode.find('.ep-uiSlideshow-controlPrev'); // use controlsNode and children
				this._playButton = this._controlsNode.find('.ep-uiSlideshow-controlPlay');
				this._popoutButton = this._controlsNode.find('.ep-uiSlideshow-controlPopout');
				this._nextButton = this._controlsNode.find('.ep-uiSlideshow-controlNext');

				this._bindStack.push(
					this._prevButton[0],
					this._playButton[0],
					this._popoutButton[0],
					this._nextButton[0]
				);

				this._prevButton.on('click',$.proxy(this,'_onClickPrevButton'));
				this._playButton.on('click',$.proxy(this,'_onClickPlayButton'));
				this._popoutButton.on('click',$.proxy(this,'_onClickPopoutButton'));
				this._nextButton.on('click',$.proxy(this,'_onClickNextButton'));
				this._slideshowBox.on('mouseenter',$.proxy(this,'_onMouseEnterSlideshowBox'));
				this._slideshowBox.on('mouseleave',$.proxy(this,'_onMouseLeaveSlideshowBox'));
				this._controlsNode.hide();
			}
			else{
				this._controlsNode.hide();
				//this._slideshowBox.on('click',$.proxy(this,'popoutCurrent'));
			}

			//enable description
			if(this.options.showDescription){
				this._topNode = this._slideshowContainerNode.find('.ep-uiSlideshow-top');
				this._descNode =this._slideshowContainerNode.find('.ep-uiSlideshow-description');
				this._locNode = this._slideshowContainerNode.find('.ep-uiSlideshow-location');
				this._bottomNode = this._slideshowContainerNode.find('.ep-uiSlideshow-bottom');
				this._countNode = this._slideshowContainerNode.find('.ep-uiSlideshow-count');
				this._dateNode = this._slideshowContainerNode.find('.ep-uiSlideshow-date');

				this._bindStack.push(
					this._topNode[0],
					this._descNode[0],
					this._locNode[0],
					this._bottomNode[0],
					this._countNode[0],
					this._dateNode[0]
				);
			}
			else{
				this._slideshowContainerNode.find('.ep-uiSlideshow-top').hide();
				this._slideshowContainerNode.find('.ep-uiSlideshow-bottom').hide();
			}

			this._slideshowBox.on('click','img',$.proxy(this,'popoutCurrent'));

			//connect thumbnails
			if(this.options.thumbnails && thumbs.length){
				this._thumbsNode.find('img').each(function(index,node){
					var elm = $(node);
					elm.on('click',$.proxy(self,'_onClickThumbnail'));
				});
			}
			else{
				this._thumbsNode.remove();
			}
			this._slideshowContainerNode.show();

		},

		_correctImgPosition:function(imgNode,imgContainerNode,imgSrcWidth,imgSrcHeight){
			//correct back image position
			imgContainerNode.css('top','0px');
			imgContainerNode.css('left','0px');
			imgNode.width('');
			imgNode.height('');
			var imgHeight = imgSrcHeight;
			var boxHeight = this._slideshowBox.height();
			var imgWidth = imgSrcWidth;
			var boxWidth = this._slideshowBox.width();
			var correctWidthOrHeight = undefined;
			//check if we have to correct the size of the displayed image and choose which side we have to correct
			if(imgHeight > boxHeight && imgWidth > boxWidth){
				//case 1 width and height are to big, check which side has the most overlap in order to correct the exact side
				if((imgHeight - boxHeight) > (imgWidth - boxWidth)){
					//case 1 correct height
					correctWidthOrHeight = 'height';
				}
				else{
					//case 1 correct width
					correctWidthOrHeight = 'width';
				}
			}
			else if(imgHeight > boxHeight && imgWidth <= boxWidth){
				//case 2 height is to big
				correctWidthOrHeight = 'height';
			}
			else if(imgHeight <= boxHeight && imgWidth > boxWidth){
				//case 3 width is to big
				correctWidthOrHeight = 'width';
			}

			if(correctWidthOrHeight == 'height'){
				//correct img height
				imgNode.height(boxHeight);
				imgWidth = imgWidth * ((boxHeight/imgHeight));
			}
			else if(correctWidthOrHeight == 'width'){
				//correct img width
				imgNode.width(boxWidth);
				imgHeight = imgHeight * ((boxWidth/imgWidth));
			}

			if(imgHeight < boxHeight){
				//if image is smaller than box we position it to center
				imgContainerNode.css('top',(boxHeight - imgHeight)/2 + 'px');
				this._correct = (boxHeight - imgHeight)/2;
			}
			if(imgWidth < boxWidth){
				//if image is smaller than box we position it to center
				imgContainerNode.css('left',(boxWidth - imgWidth)/2 + 'px');
			}
		},

		_changeBackAndFront:function(evt,src,imgWidth,imgHeight){

			this._isLoading = false;
			//make back image visible to get the height of the image
			//the user is not relay seeing it because after calcualtion we hide it again
			this._imageBackImg.attr('src',src);
			this._correctImgPosition(this._imageBackImg,this._imageBackNode,imgWidth,imgHeight);
			//fade images
			var self = this;
			this._imageFrontNode.fadeOut(500);
			this._imageBackNode.fadeIn(500,function(){
				//replace front with back image
				var tmpNode = self._imageBackNode;
				var tmpImg = self._imageBackImg;
				self._imageBackNode = self._imageFrontNode;
				self._imageFrontNode = tmpNode;
				self._imageBackImg = self._imageFrontImg;
				self._imageFrontImg = tmpImg;
				self._setTexts();

				// if magnifier is defined -> hide magnifier movers after change is complete
				if(self.options.magnifier){
					ep(self._imageFrontImg).uiMagnifier('hideMover');
					ep(self._imageBackImg).uiMagnifier('hideMover');
					ep(ep(this).parent()).uiMagnifier('setImage', self._imagesData[self._currentImgId].medium, self._imagesData[self._currentImgId].large);
				}
			});

		},

		_setTexts:function(){
			if(this.options.showDescription){
				var imageData = this._imagesData[this._currentImgId];
				if(imageData.description){
					this._descNode.html(imageData.description);
				}
				if(imageData.location){
					this._locNode.html(imageData.location);
				}
				if(imageData.description || imageData.location){
					this._topNode.show('slide',{direction: 'up'},1000);
				}
				this._countNode.html((this._currentImgId+1) + '&nbsp;|&nbsp;' + this._imagesData.length);
				if(imageData.date){
					this._dateNode.html(imageData.date);
				}
				this._bottomNode.show('slide',{direction: 'down'},1000);
			}
		},

		_imageLoaderLoaded: function(evt){
			$(this._imageLoader).trigger('loaded',[this._imageLoader.src,this._imageLoader.width,this._imageLoader.height]);
		},

		_setPopup:function(evt,src,imgWidth,imgHeight){
			var epPoupNode = ep(this._popupNode);
			if(imgWidth && imgHeight){
				epPoupNode.uiDialog('open');
				this._popupImg.height('');
				this._popupImg.width('');
				this._popupNodeImgContainer.width('');
				this._popupNodeImgContainer.height('');
				this._popupImg.attr('src',src);
				
				var dialogHeight = epPoupNode.parent().height(),
					viewportHeight = $(window).height(),
					viewportWidth = $(window).width(),
					viewportPadding = 100;	// horizontal and vertical padding (means at least 50px left, 50px right, 50px top, 50px bottom)
				
				if((imgHeight + viewportPadding) > viewportHeight || (imgWidth + viewportPadding) > viewportWidth){
					var factorWidth = ((viewportWidth-viewportPadding) / imgWidth),
						factorHeight = ((viewportHeight-viewportPadding) / imgHeight),
						factorMult = (factorWidth < factorHeight)? factorWidth : factorHeight, // get the relevant factor
						calcHeight = Math.round(factorMult * imgHeight),
						calcWidth = Math.round(factorMult * imgWidth);

					// resize image
					this._popupImg.height(calcHeight);
					this._popupImg.width(calcWidth);
					// resize popup container
					this._popupNodeImgContainer.height(calcHeight);
					this._popupNodeImgContainer.width(calcWidth);
				}
				else{
					this._popupNodeImgContainer.width(imgWidth);
					this._popupNodeImgContainer.height(imgHeight);
				}
			}
			epPoupNode.busy('hide')
				.uiDialog('open')
				.uiDialog("option", "position", "center");
		},

//Event Handlers
		_onClickPopoutButton:function(evt){
			if(evt){
				evt.stopPropagation();
			}
			this.stopPlaying();
			this.popoutCurrent();
		},

		_onClickThumbnail:function(evt){
			var currentImageId = $(evt.currentTarget).data('imgid');
			this.showImage(currentImageId);
		},

		_onClickPlayButton: function(evt){
			if(evt){
				evt.stopPropagation();
			}
			if(this._isPlaying){
				this.stopPlaying();
			}
			else{
				this.startPlaying();
			}
		},

		_onMouseEnterSlideshowBox:function(){
			this._controlsNode.fadeIn(500);
		},

		_onMouseLeaveSlideshowBox: function(){
			this._controlsNode.fadeOut(500);
		},

		_onClickNextButton: function(evt){
			this.stopPlaying();
			if(evt){
				evt.stopPropagation();
			}
			this.showNextImage();
		},

		_onClickPrevButton:function(evt){
			this.stopPlaying();
			if(evt){
				evt.stopPropagation();
			}
			this.showPrevImage();
		},

		_onClickOverviewImage: function(evt){
			var currentImageId = $(evt.currentTarget).data('imgid');
			this.stopPlaying();
			if(this._slideshowCreated){
				this.showImage(currentImageId);
			}
			else{
				this.popupImage(currentImageId);
			}
		},

		showNextImage:function(){
			if(!this._isLoading){
				if(this._currentImgId >= (this._imagesData.length-1)){
					this.showImage(0);
				}
				else{
					this.showImage(this._currentImgId + 1);
				}
			}
		},

		showPrevImage:function(){
			if(this._currentImgId == 0){
				this.showImage(this._imagesData.length-1);
			}
			else{
				this.showImage(this._currentImgId - 1);
			}
		},

		showImage: function(imageNumber){
			var imageData = this._imagesData[imageNumber];
			if(imageData){
				this._currentImgId = imageNumber;
				if(this._topNode){
					this._topNode.hide();
				}
				if(this._bottomNode){
					this._bottomNode.hide();
				}
				this._isLoading = true;
				$(this._imageLoader).one('loaded',$.proxy(this,'_changeBackAndFront'));
				$(this._imageLoader).attr('src',imageData.medium);
			}
		},

		popupImage:function(imageNumber){
			var imageData = this._imagesData[imageNumber];
			if(imageData){
				if(this._popupImg.attr('src').match(imageData.large)){
					this._setPopup();
				}
				else{
					ep(this._popupNode).busy();
					this._currentPopupImgId = imageNumber;
					$(this._imageLoader).one('loaded',$.proxy(this,'_setPopup'));
					$(this._imageLoader).attr('src',imageData.large);
				}
			}
		},

		PopupShowNextImage:function(){
			if(this._currentPopupImgId >= (this._imagesData.length-1)){
				this.popupImage(0);
			}
			else{
				this.popupImage(this._currentPopupImgId + 1);
			}
		},

		PopupShowPrevImage:function(){
			if(this._currentPopupImgId == 0){
				this.popupImage(this._imagesData.length-1);
			}
			else{
				this.popupImage(this._currentPopupImgId - 1);
			}
		},

		popoutCurrent:function(){
			this.stopPlaying();
			this.popupImage(this._currentImgId);
		},

		startPlaying: function(){
			this._isPlaying = 1;
			//we need this interval to cycle through the images
			this._playInterval = setInterval($.proxy(this,'showNextImage'),this.options.showDelay);
			this._playButton.removeClass('ep-sprite-controlPlay');
			this._playButton.addClass('ep-sprite-controlStop');
		},

		stopPlaying: function(){
			if(this._isPlaying){
				this._isPlaying = 0;
				clearInterval(this._playInterval);
				this._playButton.removeClass('ep-sprite-controlStop');
				this._playButton.addClass('ep-sprite-controlPlay');
			}
		},
		createSlideshow: function(targetNode){
			if(!this._slideshowCreated){
				this._slideshowCreated = true;
				this._slideshowContainerNode = targetNode;
				this._createSlideshow();
			}
		},
		addImageOverview: function(targetNode, overviewType, thumbnailSize){
			targetNode.empty();

			thumbnailSize = (thumbnailSize===undefined)? 100 : thumbnailSize;
			var strThumbnailSize = (thumbnailSize==300)
	           ? "large"
	           : (thumbnailSize==150)
	         	 ? "medium"
	         	 : "small";
			var cssClassName = this._thumbnailClasses[strThumbnailSize];
			var liNodes;

			switch(overviewType){
				case 'Fisheye':
					$.tmpl( 'ep.ui.slideshowFisheye',[{
						images: this._imagesData,
						sliderSize: strThumbnailSize,
						size: 'small' // strThumbnailSize
					}]).appendTo(targetNode);
					liNodes = targetNode.find('li');
					$(liNodes).addClass(cssClassName);
					break;
				case 'Moviestrip':
					$.tmpl( 'ep.ui.slideshowMoviestrip',[{
						images: this._imagesData,
						sliderSize: strThumbnailSize,
						size: 'small' // strThumbnailSize
					}]).appendTo(targetNode);
					liNodes = targetNode.find('li');
					$(liNodes).addClass(cssClassName);

					var calculatedWidth = liNodes.length * liNodes.outerWidth(true);
					var targetNodeWidth = targetNode.width();
					var endPointLeft = 0;
					var endPointRight = targetNodeWidth - calculatedWidth;
					var imageContainer = targetNode.find('.ep-Slideshow-MovieStrip-ImageContainer');
					var duration = calculatedWidth;
					imageContainer.width(calculatedWidth);

					if(calculatedWidth <= targetNodeWidth){
						imageContainer.css({
							'margin-left': (targetNodeWidth - calculatedWidth) / 2
						});
						break;
					}
					targetNode.children('.ep-Slideshow-MovieStrip-Slider').on('mousemove',function(evt){

						var currentLayer = $(this);
						var currentLayerX = evt.pageX - currentLayer.offset().left;
						var currentLayerData = currentLayer.data('ep-Slideshow-Moviestrip');
						if(currentLayerX <= (targetNodeWidth * 1/3)){
							if(currentLayerData != 'left'){
								currentLayer.data('ep-Slideshow-Moviestrip','left');
								imageContainer
									.stop(true)
									.animate({
										left: endPointLeft
									},duration
								);
								return;
							}
						}
						else if(currentLayerX > (targetNodeWidth * 1/3) && currentLayerX <= (targetNodeWidth * 2/3)){
							if(currentLayerData){
								currentLayer.data('ep-Slideshow-Moviestrip','');
								imageContainer.stop(true);
								return;
							}
						}
						else if(currentLayerX > (targetNodeWidth * 2/3)){
							if(currentLayerData != 'right'){
								currentLayer.data('ep-Slideshow-Moviestrip','right');
								imageContainer
									.stop(true)
									.animate({
										left: endPointRight
									},duration
								);
								return;
							}
						}
					});
					break;
				case 'List':
					$.tmpl( 'ep.ui.slideshowList',[{
						images: this._imagesData,
						sliderSize: strThumbnailSize,
						size: 'small' // strThumbnailSize
					}]).appendTo(targetNode);
					liNodes = targetNode.find('li');
					$(liNodes).addClass(cssClassName);
					break;
				default:
					$.tmpl( 'ep.ui.slideshowMoviestrip',[{
						images: this._imagesData,
						sliderSize: strThumbnailSize,
						size: 'small' // strThumbnailSize
					}]).appendTo(targetNode);
					liNodes = targetNode.find('li');
					$(liNodes).addClass(cssClassName);

					break;
			}
			targetNode.on('click', 'li', $.proxy(this,'_onClickOverviewImage'));
		},

		destroy: function(){
			this.Inherited(arguments);
		}

	});
});