/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.spinner
 *
 * @revision		$Revision: 1.16.22.1 $
 */

jQuery.plugin( 'ep.ui.spinner', ['ep.core', 'ep.ui.validate', 'ep.date', 'ep.sprite'], function( $, undefined ){

	var is = function( val ){
			return !!( val===0 || (!isNaN(val) && val) );
		},
		html = $('html');

	$.widget('ep.ui.Spinner',ep.ui.Validate,{

		options: {
	// ep.ui.Input
		//	big:				true,
		//	autofocus:			false,
		//	placeholder:		'text',
		//	info:				'text',

	// ep.ui.Validate
		//	accept:				'',
		//	min:				1,
		//	max:				20,
		//	minlength:			1,
		//	maxlength:			4,
		//	pattern:			[\w]+,
		//	required:			true,
		//	tooltipOffset:		[0,-7],
		//	valid:				true,
		//	type:				'basic',
		//	format:				'',

	// ep.ui.Spinner
		//	unit:				'GByte',
			type:				'number',
			format:				'n0',
			step:				1,
			stepType:			'hours'
		},

		_create: function(){
			this.Inherited();

			var self = this;

			this.options = $.extend({
					step:			parseInt( this.elem.attr('step'), 10 )
				},this.options);

			this.wrap.addClass('ep-uiSpinner');

			this.elem
				.addClass('epWidth-29')
				.on( 'keydown', function(event){
					if( event.keyCode === 38 || event.keyCode === 40 ){
						event.preventDefault();
						if( !self._interval ){
							self._start( event.keyCode === 38 ? 1 : -1 );
						}
					}
				})
				.on( 'keyup', function(event){
					if( event.keyCode === 38 || event.keyCode === 40 ){
						event.preventDefault();
						self._stop();
					}
				});

			if( this.placeholder ){
				this.placeholder.addClass('epWidth-29');
			}

			this.stepper = $('<span>')
				.addClass('ep-uiInput ep-uiInput-base ep-uiInput-button ep-uiSpinner-stepper')
				.appendTo(this.wrap);
			this.stepUp = ep( this.dict.parse('<a href="javascript:;">+</a>') )
				.addClass('ep-uiSpinner-stepUp')
				.attr({
					'tabIndex': -1
				})
				.on( 'mousedown.uiSpinner', function(){
					self._start(1);
				})
				.appendTo(this.stepper);
			this.stepDown = ep( this.dict.parse('<a href="javascript:;">&ndash;</a>') )
				.addClass('ep-uiSpinner-stepDown')
				.attr({
					'tabIndex': -1
				})
				.on( 'mousedown.uiSpinner', function(){
					self._start(-1);
				})
				.appendTo(this.stepper);

			this.stack.push( this.stepper[0], this.stepUp[0], this.stepDown[0] );
		},

		_init: function(){
			this.Inherited();
			this._unitInit();

			var o = this.options;

			if( o.type!=='date' ){
				var format = (/(c|n|p)(\d{0,})/).exec(o.format),
					step = (o.step+'').split('.'),
					fDecimals = parseInt( format[2] || 0,10 ),
					sDecimals = step[1] ? step[1].length : 0;
				this.options.format = format[1]+Math.max(fDecimals,sDecimals);
			}
		},

		_unitInit: function(){
			if( !this.unit && this.options.unit ){
				this.unit = $('<span>')
					.addClass('ep-uiSpinner-unit')
					.insertBefore(this.stepper);
				this.stack.push(this.unit[0]);
			}
			if( this.unit ){
				this.unit.text(this.options.unit);
				var padding = (this.unit.width()+4)+'px';
				this.elem.css( 'padding-right', padding );
				if( this.placeholder ){
					this.placeholder.css( 'padding-right', padding );
				}
			}
		},

		_tooltipInit: function(){
			this.Inherited(arguments);
			if( this.tooltip ){
				this.tooltip.uiTooltip( 'option', 'offsetAdjust', [30,-1] );
			}
		},

		_changeAttr: function( event, attr ){
			var self = this;
			$.each( attr, function( name, value ){
				if( (/^(accept|pattern|required)$/i).test(name) ){
					self.options[ name ] = value;
				}
				else if( (/^(min|max|minlength|maxlength|step)$/i).test(name) ){
					self.options[ name ] = parseInt( value, 10 );
				}
			});
		},

		_parseValue: function(){
			var val = this.elem.attr('value'),
				o = this.options,
				region = {region: o.region};

			if( o.type === 'date' ){
				if( !is(this._val) ){
					this._val = is(o.min) ? new ep.Date(o.min) : new ep.Date();
				}
				val = $.i18n.parseDate( val, o.format, region );
				this._val = is(val) ? new ep.Date(val) : this._val;
			}
			else{
				if( !is(this._val) ){
					this._val = is(o.min) ? o.min : 0;
				}
				region.currency = o.currency;

				val = $.i18n.parseNumber( val, 10, region );
				this._val = is(val) ? val : this._val;
			}
		},

		_start: function( inc ){
			html.on( 'mouseup.uiSpinner', this.Proxy('_stop') );

			this._placeholderHide();
			this._parseValue();

			this._type = '_spin' + (this.options.type==='date' ? 'Date' : 'Number');
			this._step = this.options.step * inc;
			this._count = 0;
			this[this._type]();
			// set interval for auto count up/down
			this._interval = setInterval( this.Proxy('_spin'), 250 );
		},

		_spin: function(){
			this._count++;
			if( (/^(10|20|30)$/).test(this._count+'') ){
				clearInterval(this._interval);
				// update interval speed for auto count up/down
				this._interval = setInterval( this.Proxy('_spin'), 150/this._count );
			}
			this[this._type]();
		},

		_spinNumber: function(){
			var o = this.options;

			this._val += this._step;

			if( is(o.min) && o.min > this._val ){
				this._val = o.min;
				this._stop();
			}
			else if( is(o.max) && o.max < this._val ){
				this._val = o.max;
				this._stop();
			}

			this.elem.val( $.i18n.formatNumber( this._val , o.format, {region:o.region,currency:o.currency} ) );
		},

		_spinDate: function( add ){
			var o = this.options;

			this._val[ $.camelCase('add-'+o.stepType) ]( this._step );

			if( is(o.min) && o.min > this._val.getTime() ){
				this._val = new ep.Date(o.min);
				this._stop();
			}
			else if( is(o.max) && o.max < this._val.getTime() ){
				this._val = new ep.Date(o.max);
				this._stop();
			}

			this.elem.val( $.i18n.formatDate( this._val, o.format, {region:o.region} ) );
		},

		_stop: function(){
			html.off( 'mouseup.uiSpinner', this.Proxy('_stop') );

			this._interval = clearInterval(this._interval);

			this._placeholderShow();

			this.elem
				.filter(':visible')
					.focus()
				.end()
				.trigger('changeValue');
		},

		destroy: function(){
			this.elem
				.removeClass('epWidth-29')
				.off('.uiSpinner');
			this.wrap
				.removeClass(/ep-uiSpinner[\w\d-]*/g);

			this.Inherited(arguments);
		}
	});

});
