/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.textedit
 *
 * @revision		$Revision: 1.7.8.1 $
 */

jQuery.plugin( 'ep.ui.textedit', ['ep.core', 'ep.ajax', 'ep.ui.input', 'jQuery.ui.core'], function( $, undefined ){

	$.widget( "ep.ui.Textedit",/* my.parent.widget,*/ {
		MIN_WIDTH: 50,
		options: {
		//	validate: 			undefined,
			additional:			undefined,												// additional save parameters (e.g. CurrencyID)
			type:				'input',												// textarea or input
			action:				null,													// save action
			objectId:			ep.config.objectId !== 0 ? ep.config.objectId : null,	// object id for save
			name:				null,													// name for save
			convertLineBreaks:	false,													// new line to br
			stripHTML:			false,													// remove html tags
			xhrType:			'POST',													// request type
			onAfterClose:		null,
			onBeforeOpen:		null,
			onAfterOpen:		null,
			formatOnWriteBack:	null
		},

		_create: function(){
			var o = this.options;

			this.elem = this.element.sub(ep);
			
			this.elem
				.on('click.uiTextedit', this.Proxy('_open'));
				/*.on( 'mouseover.uiTextedit', this.Proxy('_onmouseover') )
				.on( 'mouseout.uiTextedit', this.Proxy('_onmouseout') );*/
		},
		
		_build: function(){
			var self = this,
				o = self.options;

			this.elem
				.on( 'mouseover.uiTextedit', this.Proxy('_onmouseover') )
				.on( 'mouseout.uiTextedit', this.Proxy('_onmouseout') );

			this.uiInput = ep( o.type === 'textarea' ? '<textarea>' : '<input type="text">' )
				.hide()
				.insertAfter( this.elem );
			if(o.validate){
				this.uiInput.uiValidate(o.validate);
			}else{
				this.uiInput.uiInput();
			}
			this.uiInput
				//.uiInput()
				.uiInput('addClass', 'ep-uiTextedit-wrap')
				.on( 'blur.uiTextedit', this.Proxy('_close') )
				.on( 'keypress.uiTextedit', this.Proxy('_onkeypress') )
				.on( 'keyup.uiTextedit', this.Proxy('_onkeyup') );

			//if we have a action we also need a name to save content to the server
			if( o.action ){
				if( o.name || name ){
					this._name = o.name || name;
				}
				else{
					this._name = this.elem.attr('name');
					if( !this._name ){
						console.warn('ep.ui.textedit: unable to save: action is defined but no name is defined');
					}
				}
			}
		},

		_open:function( event ){
			// build textedit construct if necessary
			if(this.uiInput === undefined){
				this._build();
			}
			
			if((this.options.onBeforeOpen!=null) && (typeof this.options.onBeforeOpen=="function")){
				this.options.onBeforeOpen(this.uiInput);
			}


			this.content = this.elem.html();
			var content = this.content;

			this.uiInput.width( Math.max( this.elem.width(), this.MIN_WIDTH ) );
			this.elem.hide();
			this.uiInput
				.show()
				.css( 'display', this.elem.data("olddisplay") );

			if( this.options.convertLineBreaks ){
				content = content.replace(/(<br>|<br \/>)/g, '\n');
			}

			this.uiInput
				.attr({
					//'disabled': false,
					'value': content 
				})
				.focus()
				.select();
			
			if((this.options.onAfterOpen!=null) && (typeof this.options.onAfterOpen=="function")){
				this.options.onAfterOpen(this.uiInput);
			}

			event.stopPropagation();
		},

		_close:function(evt){
			if(this.uiInput.is(':input:invalid')){
				evt.preventDefault();
				//this.uiInput.focus();
				return;
			}
			this._writeContentBack();
			
			//this.uiInput.attr('disabled', true);
			
			if((this.options.onAfterClose != null) && (typeof this.options.onAfterClose == "function")){
				this.options.onAfterClose(this.uiInput);
			}
		},

		_onkeypress: function( event ){
			/*if( event.keyCode === 27 ){
				this.uiInput.attr( 'value', this.content );
				this._writeContentBack(event);
			}*/
			if( this.options.type === 'input' && event.keyCode === 13 ){
				this._writeContentBack(event);
				//this._close(event);
			}
		},

		_onkeyup: function( event ){
			if( event.keyCode === 27 ){
				this.uiInput.attr( 'value', this.content );
				this._writeContentBack(event);
			}
		},

		_onmouseover: function(){
			this.elem.addClass('ep-uiTextedit-Hover');
		},

		_onmouseout: function(){
			this.elem.removeClass('ep-uiTextedit-Hover');
		},

		_writeContentBack: function(event){
			var o = this.options;

			this.elem.show();
			this.uiInput.hide();
			
			// FIRST
			if((o.formatOnWriteBack!=null) &&(typeof o.formatOnWriteBack=="function")){
				o.formatOnWriteBack(this.uiInput, event);
			}
			
			this.content = this.uiInput.attr('value');
			if( o.stripHTML){
				this.content = ep.stripTags(this.content);
			}
			if( o.convertLineBreaks){
				this.content = this.content.replace( /(\n)/g, '<br />' );
			}
			/*if((o.formatOnWriteBack!=null) &&(typeof o.formatOnWriteBack=="function")){
				o.formatOnWriteBack(this.uiInput, event);
			}*/
			this.elem.html(this.content);
			if( o.action && o.objectId && this._name){
				this._saveContent();
			}
		},

		_saveContent: function(){
			var o = this.options,
				jsonData = {
					'ViewAction':	'JSONViewResponse',
					'ObjectID':		o.objectId,
					'ChangeAction':	o.action
				};

			jsonData[this._name] = this.content;
			// add additional parameters (e.g. CurrencyID)
			if(o.additional!==undefined){
				$.extend(jsonData, o.additional);
			}

			ep.ajax({
				dataType:	'json',
				data:		jsonData,
				type:		o.xhrType,
				context:	this
			});
		},

		destroy: function(){
			this.uiInput.remove();
			this.elem
				.off('.uiTextedit')
				.show();

			this.Inherited();
		}

	});

});
