/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.validate
 *
 * @revision		$Revision: 1.21.8.2 $
 */


jQuery.plugin( 'ep.ui.validate', ['ep.validate', 'ep.ui.input'], function( $, undefined ){

	// formular felder
	$.widget('ep.ui.Validate',ep.ui.Input,{

		options: {
	// ep.ui.Input
		//	big:				true,
		//	autofocus:			false,
		//	placeholder:		'text',
		//	info:				'text',

	// ep.ui.Validate
		//	accept:				'',
		//	min:				1,
		//	max:				20,
		//	minlength:			1,
		//	maxlength:			4,
		//	pattern:			[\w]+,
		//	required:			true,
		//	format:				'n0',
		//	currency:			'EUR',
			valid:				true,
			type:				'basic'
		},

		_create: function(){
			this.Inherited(arguments);

			this.options = $.extend({
					accept:		this.elem.attr('accept'),
					min:			parseInt( this.elem.attr('min'), 10 ),
					max:			parseInt( this.elem.attr('max'), 10 ),
					minlength:		parseInt( this.elem.attr('minlength'), 10 ),
					maxlength:		parseInt( this.elem.attr('maxlength'), 10 ),
					pattern:		this.elem.attr('pattern'),
					required:		this.elem.attr('required')
				},this.options);

			// wrap is also wrap of uiInput
			this.wrap.addClass('ep-uiValidate');
			// make .uiValidate as .uiInput also available to fix addClass for type radio
			this.elem
				.on( 'validate.uiValidate blur.uiValidate', this.Proxy('_validate') );

			// validate if invalid on default
			if( !this.options.valid ){
				this._setInstantly();
				this._setValidStatus(false);
			}

		},

		_init: function(){
			// init uiInput
			this.Inherited(arguments);
		},

		_setInstantly: function(){
			this._instantly = true;
			this.elem.on( 'changeValue.uiValidate', this.Proxy('_validate') );
		},

		_radioGroupCheck: function(){
			this.elem
				.formGroup(':radio')
				.trigger('validate')
				.uiInput('removeClass','checked')
				.filter(':checked')
					.uiInput('addClass','checked');
		},

		_changeAttr: function( event, attr ){
			var self = this;

			this.Inherited(arguments);

			$.each( attr, function( name, value ){
				if( (/^(accept|pattern|required)$/i).test(name) ){
					self.options[ name ] = value;
				}
				else if( (/^(min|max|minlength|maxlength)$/i).test(name) ){
					self.options[ name ] = parseInt( value, 10 );
				}
			});
		},

		_setOption: function( name, value ){
			if( name=='valid' && value!==undefined ){
				this._setValidStatus(value);
			}
			return this.Inherited(arguments);
		},
		
		_setValidStatus: function( valid ){
			this.elem[0].formInvalid = !valid;
			if( this.placeholder ){
				this.placeholder[0].formInvalid = !valid;
			}
			this[ valid ? 'removeClass' : 'addClass' ]('invalid');
		},

		_validate: function( event ){
			if( event.keyCode!==9 ){

				var val = this.elem.val();

				if( this.elem.is(':radio') ){
					val = this.elem
						.formGroup(':radio')
						.filter(':checked')
						.val() || '';
				}
				else if( this.elem.is(':checkbox') ){
					val = this.elem
						.filter(':checked')
						.val() || '';
				}

				this.valid = ep.validate[ this.options.type ]( val, this.options );

				var valid = this.valid===true;

				if( !valid && !this._instantly ){
					this._setInstantly();
				}

				this._setValidStatus(valid);

				if( event.type!=='changeAttr' ){
					$(this.elem[0].form).trigger('change');
				}

				this._tooltipShow();
			}
		},

		_tooltipShow: function(){
			if( this.valid!==true ){
				// create tooltip if not exists
				if( !this.tooltip ){
					this._tooltipInit(true);
					this.tooltip.on( 'show', this.Proxy('_tooltipHide') );
				}
				if( !this.errormsg ){
					this.errormsg = $('<div>')
						.addClass('ep-uiValidate-message')
						.insertBefore(this.tooltip);
					this.stack.push(this.errormsg[0]);
				}
				// options for errer msg
				var options = {
						min: this.options.min,
						max: this.options.max,
						minlength: this.options.minlength,
						maxlength: this.options.maxlength
					},
					format = this.options.format,
					opts = {region:this.options.region};

				// i18n for min + max
				if( this.options.type === 'number' ){
					opts.currency = this.options.currency;

					options.example = $.i18n.formatNumber( 21.45923, format, opts );

					if( options.min || options.max ){
						options.min = $.i18n.formatNumber( options.min, format, opts );
						options.max = $.i18n.formatNumber( options.max, format, opts );
					}
				}
				else if( this.options.type === 'date' ){
					options.example = $.i18n.formatDate( new Date(), format, opts );

					if( options.min || options.max ){
						options.min = $.i18n.formatDate( new Date(options.min), format, opts );
						options.max = $.i18n.formatDate( new Date(options.max), format, opts );
					}
				}
				// set tooltip msg
				this.errormsg.text( this.dict.translateData( this.valid, options ) );
				this.errormsg.show();

				if( this.elem.is(':focus') ){
					this.tooltip.uiTooltip('show');
				}
			}
			else{
				this._tooltipHide();
			}
		},

		_tooltipHide: function(){
			var self = this,
				hidemsg = function(){
					if( self.errormsg ){
						self.errormsg.hide();
					}
				};

			if( this.tooltip && this.valid===true ){
				if( !this.options.info ){
					this.tooltip.uiTooltip('hide',hidemsg);
				}
				else{
					hidemsg();
				}
			}
		},

		destroy: function(){
			this.elem
				.off('.uiValidate');
			this.wrap
				.removeClass(/ep-uiValidate[\w\d-]*/g);

			this.Inherited(arguments);
		}
	});

});
