/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.validate
 *
 * @revision		$Revision: 1.13.8.1 $
 */

jQuery.plugin( 'ep.validate', ['ep.core', 'ep.color', 'jQuery.mime', 'ep.i18n'], function( $, undefined ){

/*
 * @dictionary			ep.dict
 *
 * @translation			{NOT_VALID}
 *						{REQUIRED}
 *						{NOT_IN_RANGELENGTH}
 *						{LOWER_THAN_MINLENGTH}
 *						{GREATER_THAN_MAXLENGTH}
 *						{NOT_IN_RANGE}
 *						{LOWER_THAN_MIN}
 *						{GREATER_THAN_MAX}
 *						{NO_EMAIL}
 *						{NO_URL}
 *						{NO_MIME}
 *						{MIME_NOT_ACCEPT}
 *						{NO_CREDITCARD_NUMBER}
 *						{NO_NUMBER}
 *						{NO_CURRENCY}
 *						{NO_PERCENT}
 *						{NO_DATE}
 *						{NO_DATETIME}
 *						{NO_TIME}
 *						{NO_COLOR_VALUE}
 *						{NO_UNIT}
 *						{NO_UNIT_VALUE}
 *						{NO_CSS_NUMBER}
 *						{FLOAT_NOT_ALLOWED}
 *						{UNKOWN_UNIT}
 */

	var invalid = 'NOT_VALID',

		pattern = function( value, pattern ) {
			return pattern ? ( new RegExp( ((/^\^/).test(pattern)?'':'^') + pattern + ((/\$$/).test(pattern)?'':'$') , '' ).test( value ) || invalid ) : true;
		},

		required = function( value, req ) {
			return !value && req ? 'REQUIRED' : true;
		},

		rangelength = function( value, min, max ) {
			min = min<0 ? 0 : min;
			max = max<0 ? 0 : max;

			var val = $.trim(value).length;

			if( min && !isNaN(min) && min>val ){
				return max && !isNaN(max) ? 'NOT_IN_RANGELENGTH' : 'LOWER_THAN_MINLENGTH';
			}
			if( max && !isNaN(max) && max<val && max > 0){
				return min && !isNaN(min) ? 'NOT_IN_RANGELENGTH' : 'GREATER_THAN_MAXLENGTH';
			}
			return true;
		},

		range = function( value, min, max ) {
			var val = typeof value !== 'number' ? parseFloat(value,10) : value;

			if( min && !isNaN(min) && min>val ){
				return max && !isNaN(max) ? 'NOT_IN_RANGE' : 'LOWER_THAN_MIN';
			}
			if( max && !isNaN(max) && max<val && max > 0 ){
				return min && !isNaN(min) ? 'NOT_IN_RANGE' : 'GREATER_THAN_MAX';
			}
			return true;
		},

		i18nMap = {
			'date':					'd',
			'dateLong':				'D',
			'dateLong-time':		'f',
			'dateLong-timeLong':	'F',
			'date-time':			'l',
			'date-timeLong':		'L',
			'time':					't',
			'timeLong':				'T',
			'currency':				'c',
			'number':				'n',
			'percent':				'p'
		};

	$.extend( $.scope('ep.validate'), {

		email: function( value, options ){
			value = $.trim(value);

			var opt = $.extend({},options||{}),
				valid = required( value, opt.required );

			if( value && valid===true ){
				valid = /^((([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+(\.([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+)*)|((\x22)((((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(([\x01-\x08\x0b\x0c\x0e-\x1f\x7f]|\x21|[\x23-\x5b]|[\x5d-\x7e]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(\\([\x01-\x09\x0b\x0c\x0d-\x7f]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))))*(((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(\x22)))@((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.?$/i.test(value) || 'NO_EMAIL';
			}
			return valid===true ? range( value, opt.minlength, opt.maxlength ) : valid;
		},
		// url must be absolute
		url: function( value, options ){
			value = $.trim(value);

			var opt = $.extend({},options||{}),
				valid = required( value, opt.required );

			if( value && valid===true ){
				valid = /^(https?|ftp):\/\/(((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:)*@)?(((\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5]))|((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.?)(:\d*)?)(\/((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)+(\/(([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)*)*)?)?(\?((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|[\uE000-\uF8FF]|\/|\?)*)?(\#((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|\/|\?)*)?$/i.test(value) || 'NO_URL';
			}
			return valid===true ? range( value, opt.minlength, opt.maxlength ) : valid;
		},

		mimetype: function( value, options ){
			value = $.trim(value);

			var opt = $.extend({
					accept: undefined
				},options||{}),
				valid = required( value, opt.required );

			if( value && valid===true ){
				valid = /^[a-z]+\/[a-z\d\.-]+$/i.test(value) || 'NO_MIME';
			}
			if( value && opt.accept && valid===true ){
				valid = $.mime( value, opt.accept ) || 'MIME_NOT_ACCEPT';
			}
			return valid;
		},

		creditcard: function( value, options ) {
			value = $.trim(value);

			var opt = $.extend({},options||{}),
				valid = required( value, opt.required );

			if( value && valid===true ){
				if( /[^0-9-]+/.test(value) ){
					return false;
				}

				var nCheck = 0,
					nDigit = 0,
					bEven = false,
					cDigit;

				value = value.replace(/\D/g, "");

				for( var n=value.length-1 ; n>=0 ; n-- ){
					cDigit = value.charAt(n);
					nDigit = parseInt(cDigit, 10);
					if( bEven ){
						if( (nDigit *= 2) > 9 ){
							nDigit -= 9;
						}
					}
					nCheck += nDigit;
					bEven = !bEven;
				}

				valid = (nCheck % 10) === 0 || 'NO_CREDITCARD_NUMBER';
			}
			return valid;
		},

		date: function( value, options ){
			value = $.trim(value);

			var opt = $.extend({
					format: 'd',
					strict: false
				},options||{}),
				region = {region:opt.region},
				valid = required( value, opt.required );

			opt.format = i18nMap[opt.format] || opt.format;

			if( value && valid===true ){
				switch(true){
					case (/^(d|D)$/).test(opt.format):
						valid = 'NO_DATE';
					break;
					case (/^(f|F|l|L)$/).test(opt.format):
						valid = 'NO_DATETIME';
					break;
					case (/^(t|T)$/).test(opt.format):
						valid = 'NO_TIME';
					break;
					default:
						valid = invalid;
					break;
				}
				valid = new RegExp( '^' + $.i18n.expStrDate( opt.format, opt.strict, region ) + '$', '' ).test(value) || valid;

				if( valid===true && value ){
					var date = $.i18n.parseDate( value, opt.format, region );

					valid = date instanceof Date ? range( date.getTime(), opt.min, opt.max ) : invalid;
				}
			}
			return valid;
		},

		number: function( value, options ){
			value = $.trim(value);

			var opt = $.extend({
					format: 'n',
					strict: false
				},options||{}),
				region = {region:opt.region,currency:opt.currency},
				valid = required( value, opt.required );

			opt.format = i18nMap[opt.format] || opt.format;

			if( value && valid===true ){
				var msg = {
					n:'NO_NUMBER',
					c:'NO_CURRENCY',
					p:'NO_PERCENT'
				};
				opt.format.replace(/^(n|c|p)\d*$/,function(str,$1){
					valid = msg[$1] || invalid;
				});
				valid = new RegExp( '^' + $.i18n.expStrNumber( opt.format, opt.strict, region ) + '$', '' ).test(value) || valid;
			}
			if( value && valid===true ){
				var num = $.i18n.parseNumber( value, 10, region );

				valid = isNaN(num) ? invalid : range( num, opt.min, opt.max );
			}
			return valid;
		},

		cssColor: function( value ){
			value = $.trim(value);
			return ep.color.stringToHex(value)!==undefined || 'NO_COLOR_VALUE';
		},

		cssSize: function( value, options ){
			value = $.trim(value);

			var opt = $.extend({},options||{}),
				valid = required( value, opt.required );

			if( value && valid===true ){
				switch(true){
					case (/^[0-9]+(px|pt)$/i).test(value) || /^[0-9]*\.?[0-9]+(pc|%|mm|cm|em|in|ex)$/i.test(value) || /^[0]+$/.test(value):
						valid = true;
					break;
					case (/^[0-9]+$/).test(value):
						valid = 'NO_UNIT';
					break;
					case value === "":
						valid = 'NO_UNIT_VALUE';
					break;
					case (/^[^0-9]*\.?[^0-9]/).test(value):
						valid = 'NO_CSS_NUMBER';
					break;
					case (/^[0-9]*\.[0-9]+(?:px|pt)$/i).test(value):
						valid = 'FLOAT_NOT_ALLOWED';
					break;
					case (/^[0-9]*\.[0-9]$/).test(value):
						valid = 'NO_UNIT';
					break;
					case (/^[0-9]*\.?[0-9]+[a-z]+$/).test(value):
						valid = 'UNKNOWN_UNIT';
					break;
					default:
						valid = invalid;
					break;
				}
			}
			if( value && valid===true ){
				var num = parseFloat(value.replace(/\W+$/,''),10);

				valid = isNaN(num) ? invalid : range( num, opt.min, opt.max );
			}
			return valid;
		},

		basic: function( value, options ){
			var opt = $.extend({},options||{}),
				valid = required( value, opt.required );

			if( value && valid===true ){
				valid = pattern( value, opt.pattern);
			}

			if( value && valid===true ){
				valid = rangelength( value, opt.minlength, opt.maxlength );
			}

			return valid;
		}
/* TODO: validate by server,
		ajax: function( value, options ){

		}
*/
	});

});