/*
 * @author			Emanuel Vitzthum
 * @copyright		© 2011 jQuery SDK v1.1.1
 * @info			http://www.jquerysdk.com
 *
 * @license			Dual licensed under the MIT (http://www.opensource.org/licenses/mit-license.php) and GPL (http://www.gnu.org/licenses/gpl.html)
 *
 * @plugin			jQuery.event.special.mouse
 */

jQuery.plugin( 'jQuery.event.special.mouse', function( $, undefined ){

	var html = $('html'),
		
		dragNamespace	= '_specialMousedrag',
		
		wheelNamespace	= '_specialMousewheel',

		getId = function( node, obj ){
			return ( node[ $.expando ] || (node[ $.expando ] = ++$.guid) )+'.'+( obj.guid || (obj.guid = ++$.guid) );
		},

		mousedown = function( originalBind, namespace, id ){
			return (mousedown[ id ] = function( event ){
					html.on( 'mousemove.'+namespace, mousemove( originalBind, namespace, id ) )
						.on( 'mouseup.'+namespace, mouseup( originalBind, namespace, id ) );

					mousemove[ id ].pageX = event.pageX;
					mousemove[ id ].pageY = event.pageY;
				});
		},

		mousemove = function( originalBind, namespace, id ){
			return (mousemove[ id ] = function( event ){

					event.type = originalBind.type;
					event.target = originalBind.originalTarget;
					event.data = originalBind.data;

					event.dragDeltaX = event.pageX - mousemove[ id ].pageX;
					event.dragDeltaY = event.pageY - mousemove[ id ].pageY;
					event.dragDelta  = delta  = Math.sqrt( Math.pow(event.dragDeltaX,2) + Math.pow(event.dragDeltaY,2) );

					mousemove[ id ].pageX = event.pageX;
					mousemove[ id ].pageY = event.pageY;

					return originalBind.originalHandler.apply(originalBind.originalTarget,arguments);
				});
		},

		mouseup = function( originalBind, namespace, id ){
			return (mouseup[ id ] = function(event){

					html.off( 'mousemove.'+namespace, mousemove[ id ] )
						.off( 'mouseup.'+namespace, mouseup[ id ] );

					delete mousemove[ id ];
					delete mouseup[ id ];
				});
		},

		mousewheel = function( originalBind, namespace, id ){
			return (mousewheel[ id ] = function( event ){

					var originalEvent = event.originalEvent,
						divisor = Math.abs(originalEvent.wheelDelta || 0) >= 120 ? 120 : 1;

					event.wheelDelta  = originalEvent.wheelDelta  ? originalEvent.wheelDelta  / divisor : originalEvent.detail ? originalEvent.detail*3 : 0;
					event.wheelDeltaX = originalEvent.wheelDeltaX ? originalEvent.wheelDeltaX / divisor : 0;
					event.wheelDeltaY = originalEvent.wheelDeltaY!==undefined ? originalEvent.wheelDeltaY / divisor : event.wheelDelta;

					if( $.browser.opera ) {
						event.wheelDelta  *= -1;
						event.wheelDeltaX *= -1;
						event.wheelDeltaY *= -1;
					}

					event.type = originalBind.type;

					return originalBind.originalHandler.apply(this,arguments);
				});
		};

	$.extend( $.event.special, {
		mousedrag:{
			add: function( bind ) {
				bind.originalTarget = this;
				bind.originalHandler = bind.handler;

				$(this).on( 'mousedown.'+dragNamespace, mousedown( bind, dragNamespace, getId(this,bind) ) );
			},

			remove: function( unbind ) {
				var id = getId(this,unbind),
					handlerDown = mousedown[ id ],
					handlerMove = mousemove[ id ];

				$(this).off( 'mousedown.'+dragNamespace, handlerDown );
				delete( handlerDown );

				html.off( 'mousemove.'+dragNamespace, handlerMove );
				delete handlerMove;
			}
		},
		mousewheel:{
			add: function( bind ) {
				bind.originalTarget = this;
				bind.originalHandler = bind.handler;

				var handler = mousewheel( bind, wheelNamespace, getId(this,bind) );

				if( $.browser.mozilla )
					$(this).on( 'DOMMouseScroll.'+wheelNamespace, handler );
				else
					bind.handler = handler;
			},

			remove: function( unbind ) {
				var handler = mousewheel[ getId(this,unbind) ];

				if( $.browser.mozilla )
					$(this).off( 'DOMMouseScroll.'+wheelNamespace, handler );
				delete handler;
			}
		}
	});

});