/*
 * @author			Emanuel Vitzthum
 * @copyright		© 2011 jQuery SDK v1.1.1
  * @info			http://www.jquerysdk.com
 *
 * @license			Dual licensed under the MIT (http://www.opensource.org/licenses/mit-license.php) and GPL (http://www.gnu.org/licenses/gpl.html)
 *
 * @plugin			jQuery.event.special.touch
 */

jQuery.plugin( 'jQuery.event.special.touch', 'jQuery.event.fix', function( $, undefined ){

	var html = $('html'),

		getId = function( node, obj ){
			return ( node[ $.expando ] || (node[ $.expando ] = ++$.guid) )+'.'+( obj.guid || (obj.guid = ++$.guid) );
		},

		touchstart = function( originalBind, namespace, id ){
			return (touchstart[ id ] = function(event){
					var touches = event.touches,
						isA = originalBind.type==='touchdrag' && touches.length === 1 && event.originalTarget===originalBind.originalTarget,
						isB = originalBind.type==='touchzoom' &&  touches.length === 2 && touches[0].target===originalBind.originalTarget && touches[1].target===originalBind.originalTarget;
			
					if( isA || isB ){
						html.on( 'touchmove.'+namespace, touchmove( originalBind, namespace, id ) )
							.on( 'touchend.'+namespace, touchend( originalBind, namespace, id ) )
							.on( 'touchcancel.'+namespace, touchcancel( originalBind, namespace, id ) );

						touchmove[ id ].pageX = event.pageX;
						touchmove[ id ].pageY = event.pageY;
					}
					if( isB ){
						var deltaX = touchmove[ id ].deltaX = Math.abs( touches[0].pageX - touches[1].pageX ),
							deltaY = touchmove[ id ].deltaY = Math.abs( touches[0].pageY - touches[1].pageY );
							delta  = touchmove[ id ].delta  = Math.sqrt( Math.pow(deltaX,2) + Math.pow(deltaY,2) );
					}
				});
		},

		touchmove = function( originalBind, namespace, id ){
			return (touchmove[ id ] = function(event){
					event.type = originalBind.type;
					event.target = originalBind.originalTarget;
					event.data = originalBind.data;

					if( originalBind.type==='touchdrag' ){
						event.dragDeltaX = - touchmove[ id ].pageX + (touchmove[ id ].pageX = event.pageX);
						event.dragDeltaY = - touchmove[ id ].pageY + (touchmove[ id ].pageY = event.pageY);
						event.dragDelty  = Math.sqrt( Math.pow(event.dragDeltaX,2) + Math.pow(event.dragDeltaY,2) );

						return originalBind.originalHandler.apply(originalBind.originalTarget,arguments);
					}
					else if( originalBind.type==='touchzoom' &&  event.touches.length >= 2 ){
						var touches = event.touches,
							deltaX,
							deltaY;
						
						event.zoomDeltaX = - touchmove[ id ].deltaX + (touchmove[ id ].deltaX = deltaX = Math.abs( touches[0].pageX - touches[1].pageX ) );
						event.zoomDeltaY = - touchmove[ id ].deltaY + (touchmove[ id ].deltaY = deltaY = Math.abs( touches[0].pageY - touches[1].pageY ) );
						event.zoomDelta  = - touchmove[ id ].delta  + (touchmove[ id ].delta  = Math.sqrt( Math.pow(deltaX,2) + Math.pow(deltaY,2) ) );
						
						return originalBind.originalHandler.apply(originalBind.originalTarget,arguments);
					}
				});
		},

		touchend = function( originalBind, namespace, id ){
			return (touchend[ id ] = function(event){
					html.off( 'touchmove.'+namespace, touchmove[ id ] )
						.off( 'touchend.'+namespace, touchend[ id ] )
						.off( 'touchcancel.'+namespace, touchcancel[ id ] );

					delete( touchmove[ id ] );
					delete( touchend[ id ] );
					delete( touchcancel[ id ] );
				});
		},

		touchcancel = touchend;

	// touchdrag, touchzoom
	$.each({
		'touchdrag':'_specialTouchdrag',
		'touchzoom':'_specialTouchzoom'
	}, function( name, namespace ) {

		$.event.special[ name ] = {
			add: function( bind ) {
				bind.originalTarget = this;
				bind.originalHandler = bind.handler;

				html.on( 'touchstart.'+namespace, touchstart( bind, namespace, getId(this,bind) ) );
			},
			remove: function( unbind ) {
				var id = getId(this,unbind),
					handleStart = touchstart[ id ],
					handleMove = touchmove[ id ];
				
				html.off( 'touchstart', handleStart );
				delete( handleStart);
				
				html.off( 'touchmove', handleMove );
				delete( handleMove );
			}
		};

	});

});