/*
 * @author          Emanuel Vitzthum
 * @copyright       © 2011 jQuery SDK v1.1.1
 * @info            http://www.jquerysdk.com
 *
 * @license         Dual licensed under the MIT (http://www.opensource.org/licenses/mit-license.php) and GPL (http://www.gnu.org/licenses/gpl.html)
 *
 * @based			jQuery UI Widget (http://docs.jquery.com/UI/Widget) license (http://jquery.org/license)
 *
 * @plugin			jQuery.ui.widget
 */

jQuery.plugin( 'jQuery.ui.widget', ['jQuery.class', 'jQuery.each'], function( $, undefined ){

var camelCase = function( string ){
		return string.replace( /[\.\:\-]([a-z])/ig, function( all, letter ){
			return letter.toUpperCase();
		});
	};

// jQuery 1.4+
if ( $.cleanData ) {
	var _cleanData = $.cleanData;
	$.cleanData = function( elems ) {
		for ( var i = 0, elem; (elem = elems[i]) != null; i++ ) {
			$( elem ).triggerHandler( "remove" );
		}
		_cleanData( elems );
	};
} else {
	var _remove = $.fn.remove;
	$.fn.remove = function( selector, keepData ) {
		return this.each(function() {
			if ( !keepData ) {
				if ( !selector || $.filter( selector, [ this ] ).length ) {
					$( "*", this ).add( [ this ] ).each(function() {
						$( this ).triggerHandler( "remove" );
					});
				}
			}
			return _remove.call( $(this), selector, keepData );
		});
	};
}

$.widget = function( name, base , prototype ) {
	var tmp = name.split( "." ),
		namespace = tmp.shift(),
		Classname = name,
		api = window[ namespace ],
		dataName,
		fullName;
	name = camelCase( tmp.join(".") );
	fullName = namespace + "-" + name;

	if ( !prototype ) {
		prototype = base;
		base = $.Widget;
	}

	if( api && api.fn instanceof $ ) {
		api = namespace
		dataName = fullName
	}
	else {
		api = 'jQuery';
		dataName = name;
		Classname = 'jQuery.'+Classname;
	}

	// create selector for plugin
	$.expr[ ":" ][ fullName ] = function( elem ) {
		return !!$.data( elem, dataName );
	};

	var widget = $.Class( Classname, base, prototype );
	$.extend( widget.prototype, {
		namespace: namespace,
		widgetData: dataName,
		widgetName: name,
		widgetEventPrefix: widget.prototype.widgetEventPrefix || name,
		widgetBaseClass: fullName
	});

	$.widget.bridge( name, widget, api );
};

$.widget.bridge = function( name, object, api ){

	window[ api ].fn[ name ] = function( options ) {
		var isMethodCall = typeof options === "string",
			args = Array.prototype.slice.call( arguments, 1 ),
			returnValue = this,
			widgetData = object.prototype.widgetData;

		// allow multiple hashes to be passed on init
		options = !isMethodCall && args.length ?
			$.extend.apply( null, [ true, options ].concat(args) ) :
			options;

		// prevent calls to internal methods
		if( isMethodCall && (/^[A-Z_]/).test(options) && options!='Instance' ) {
			return returnValue;
		}

		if( isMethodCall ) {
			this.each(function() {
				var instance = $.data( this, widgetData ),
					methodValue = instance && $.isFunction( instance[options] ) ?
						instance[ options ].apply( instance, args ) :
						instance;

				if ( (methodValue !== instance && methodValue !== undefined) || options==='Instance' ) {
					returnValue = methodValue;
					return false;
				}
			});
		} else {
			this.each( true, function() {
				var instance = $.data( this, widgetData );
				if( instance ) {
					instance.option( options || {} )._init();
				}
				else {
					var instance = new object( options, this );
					$.data( this, widgetData, instance );
					return instance.element[0];
				}
			});
		}

		return returnValue;
	}
};

$.Class( 'jQuery.Widget', {

	widgetName: "widget",
	widgetEventPrefix: "",

	options: {
		disabled: false
	},

	Constructor: function( options, element ) {
		// $.widget.bridge stores the plugin instance, but we do it anyway
		// so that it's stored even before the _create function runs
		$.data( element, this.widgetData, this );
		this.element = $( element );

		// recursive inherited options extend
		var o = [ this.options, this._getCreateOptions(), options ],
			inh = this.Class.inherited,
			cur;
		while( inh ){
			cur = inh;
			inh = (cur.Class||{}).inherited;
			o.unshift( cur.options )
		}
		o.unshift({});
		o.unshift(true);

		this.options = $.extend.apply( $, o );

		var self = this;
		this.element.on( "remove." + this.widgetName, function() {
			self.destroy();
		});

		this._create();
		this._trigger( "create" );
		this._init();
	},
	_getCreateOptions: function() {
		return $.metadata && $.metadata.get( this.element[0] )[ this.widgetName ];
	},
	_create: function() {},
	_init: function() {},

	destroy: function() {
		this.element
			.off( "." + this.widgetName )
			.removeData( this.widgetName );
		this.widget()
			.off( "." + this.widgetName )
			.removeAttr( "aria-disabled" )
			.removeClass(
				this.widgetBaseClass + "-disabled " +
				"ui-state-disabled" );
	},

	widget: function() {
		return this.element;
	},

	option: function( key, value ) {
		var options = key;

		if ( arguments.length === 0 ) {
			// don't return a reference to the internal hash
			return $.extend( {}, this.options );
		}

		if  (typeof key === "string" ) {
			if ( value === undefined ) {
				return this.options[ key ];
			}
			options = {};
			options[ key ] = value;
		}

		this._setOptions( options );

		return this;
	},
	_setOptions: function( options ) {
		var self = this;
		$.each( options, function( key, value ) {
			self._setOption( key, value );
		});

		return this;
	},
	_setOption: function( key, value ) {
		this.options[ key ] = value;

		if ( key === "disabled" ) {
			this.widget()
				[ value ? "addClass" : "removeClass"](
					this.widgetBaseClass + "-disabled" + " " +
					"ui-state-disabled" )
				.attr( "aria-disabled", value );
		}

		return this;
	},

	enable: function() {
		return this._setOption( "disabled", false );
	},
	disable: function() {
		return this._setOption( "disabled", true );
	},

	_trigger: function( type, event, data ) {
		var callback = this.options[ type ];

		event = $.Event( event );
		event.type = ( type === this.widgetEventPrefix ?
			type :
			this.widgetEventPrefix + type ).toLowerCase();
		data = data || {};

		// copy original event properties over to the new event
		// this would happen if we could call $.event.fix instead of $.Event
		// but we don't have a way to force an event to be fixed multiple times
		if ( event.originalEvent ) {
			for ( var i = $.event.props.length, prop; i; ) {
				prop = $.event.props[ --i ];
				event[ prop ] = event.originalEvent[ prop ];
			}
		}

		this.element.trigger( event, data );

		return !( $.isFunction(callback) &&
			callback.call( this.element[0], event, data ) === false ||
			event.isDefaultPrevented() );
	}
});

});
