#-*- Makefile -*-

require DE_EPAGES::Core::API::Makefile;
__DATA__
#
#   makefile
#
#   Project : epages 6
#   Copyright   : ePages GmbH 2004
#   Version : Fri Dec 03 17:07:43 2010
#

!OS_SPECIFIC_DEFINES
DE_EPAGES   = .
COMPANY     = DE_EPAGES
STORE       = YourStore
SOURCEDOC   = $(MAKEDIR)\..\SourceDoc
SOURCEDOCDATE= Fri December 3 12:00:00 2010
EPAGES_VERSION_NUMBER= 6.13.1
EPAGES_VERSION= ePages $(EPAGES_VERSION_NUMBER)
EPAGES_TAG  = release_6_13_1
!ifndef PATCH_TAG
PATCH_TAG   = $(EPAGES_TAG)
!endif
SITE_NAME   = Site
STORE_TYPE  = Store
SITE_TYPE   = Site
BACKUP_NAME = Backup
PROVIDER_PATH = /Providers/Distributor
PROVIDER_USER = admin
PROVIDER_PASSWD = admin
DEMOSHOP_ALIAS = DemoShop
DEMOSHOP_SHOPTYPE = Demo
CARTRIDGE=ALL
# settings for multistore installations
MASTER_NAME  = Master
MASTER_TYPE  = Master
MULTISTORE_NAME  = MultiStore
MULTISTORE_TYPE  = MultiStore
MASTER_PROVIDER_PATH = $(PROVIDER_PATH)
MULTISTORE_PROVIDER_PATH = $(PROVIDER_PATH)
MASTER_PROVIDER_USER = $(PROVIDER_USER)
MULTISTORE_PROVIDER_USER = $(PROVIDER_USER)
MASTER_PROVIDER_PASSWD = $(PROVIDER_PASSWD)
MULTISTORE_PROVIDER_PASSWD = $(PROVIDER_PASSWD)
MASTERSHOP_ALIAS = MasterShop
MASTERSHOP_SHOPTYPE = Demo
SUBSHOP_ALIAS = SubShop
CACHESHOP_ALIAS = CacheShop
SUBSHOP_SHOPTYPE = DropShipping1000

!ifdef WINDOWS
IF_EXIST	= if exist
IF_NOT_EXIST	= if not exist
DO		=
!else
SHELL		= /bin/sh
IF_EXIST	= [ ! -e
IF_NOT_EXIST	= [ -e
DO		= ] ||
EP_WEB_GROUP  = $(shell echo $${EPAGES_WEBGROUP:-ep_web} | sed 's/:.*//')
!endif
MODULES_INI   = $(EPAGES_CARTRIDGES)/CPAN/EPAGES/PerlLib/$(OSNAME)/modules.ini

!ifndef PATCH_DIR
PATCH_DIR   = $(EPAGES_SHARED)\Patches\$(EPAGES_TAG)
!endif
DATA_DIR    = $(EPAGES_SHARED)\var-$(EPAGES_TAG)
BACKUP_DIR  = $(DATA_DIR)\backup\$(HOSTNAME)
LOG_DIR     = $(DATA_DIR)\logs\$(HOSTNAME)
TGT         = $(DATA_DIR)\tgt\$(HOSTNAME)

IMPORT      = $(PERL) $(DE_EPAGES)/Object/Scripts/import.pl -storename=$(STORE)
DELETE      = $(PERL) $(DE_EPAGES)/Object/Scripts/delete.pl -storename=$(STORE)
DBDUMPS     = $(PERL) $(DE_EPAGES)/Database/Scripts/DatabaseDumps.pl
PATCHFILES  = $(PERL) $(DE_EPAGES)/Core/Scripts/PatchFiles.pl
PATCHDB     = $(PERL) $(DE_EPAGES)/Installer/Scripts/patchDatabase.pl
CREATEDB    = $(PERL) $(DE_EPAGES)/Database/Scripts/createDB.pl
DROPDB      = $(PERL) $(DE_EPAGES)/Database/Scripts/dropDB.pl
INSTALLDB   = $(PERL) $(DE_EPAGES)/ShopConfiguration/Scripts/installDatabase.pl
REGISTER    = $(PERL) $(DE_EPAGES)/ShopConfiguration/Scripts/registercartridges.pl
SYNC        = $(PERL) $(DE_EPAGES)/ShopConfiguration/Scripts/synccartridges.pl
DROPTABLES  = $(PERL) $(DE_EPAGES)/Database/Scripts/dropAllTables.pl
STOREREDIRECT= $(PERL) $(DE_EPAGES)/Database/Scripts/redirectStoreConnections.pl
DEL_INI_SECTION= $(PERL) $(DE_EPAGES)/Core/Scripts/DelIniSection.pl
INSTALL_CPAN= $(PERL) $(DE_EPAGES)/Installer/Scripts/installCPAN.pl
START_EPAGES= $(PERL) $(DE_EPAGES)/Installer/Scripts/startService.pl epages
STOP_EPAGES = $(PERL) $(DE_EPAGES)/Installer/Scripts/stopService.pl epages
START_CRON  = $(PERL) $(DE_EPAGES)/Installer/Scripts/startService.pl cron
STOP_CRON   = $(PERL) $(DE_EPAGES)/Installer/Scripts/stopService.pl cron

# device size in MByte
DATSIZE = 100
LOGSIZE = 50

makefile: Makefile.PL
	$(PERL) Makefile.PL
	@$(ECHONL) "Please restart '$(MAKE)' again, makefile was updated." && $(PERL) -e "exit 1;"

test_store: makefile
	$(PERL) -e "exit ('$(STORE)' eq 'YourStore' ? 1 : 0);"

stop: makefile
	$(STOP_EPAGES)

start: makefile
!ifdef WINDOWS
	$(START_EPAGES)
!else
	IGNORE_INI=1 $(START_EPAGES)
!endif

stop_cron: makefile
	$(STOP_CRON)

start_cron: makefile
	$(START_CRON)

doc: makefile
	$(PERL) $(DE_EPAGES)/SourceDoc/Scripts/DocDir.pl -source $(EPAGES_CARTRIDGES) -target $(SOURCEDOC) -title "$(EPAGES_VERSION)" -ext pm -date "$(SOURCEDOCDATE)" $(EPAGES_CARTRIDGES)\DE_EPAGES

restart: makefile stop start

purge_configuration:
	$(DEL) $(EPAGES_CONFIG)\*.ept
	$(DEL) $(EPAGES_CONFIG)\*\*.ept
	$(DEL) $(EPAGES_CONFIG)\*.initial
	$(DEL) $(EPAGES_CONFIG)\*\*.initial
	$(DEL) $(EPAGES_CONFIG)\*~
	$(DEL) $(EPAGES_CONFIG)\*\*~

clean: makefile
!ifdef WINDOWS
	del /Q /S .*#* *.pdb
	dir /S /B /A:D | grep "Generated$$" | xargs "$(RMDIR)"
	dir /S /B /A:-D | grep "makefile$$" | xargs "$(DEL)"
!endif

test: makefile test_store
	$(PERL) $(DE_EPAGES)/Installer/Scripts/runTests.pl -storename $(STORE)

delete_tables: test_store
	$(DROPTABLES) -storename $(STORE)

cleanfiles_global: makefile
	$(EMPTY_DIR) $(EPAGES_WEBROOT)\_MONITOR_
	$(EMPTY_DIR) $(EPAGES_STORES)\_MONITOR_
	$(EMPTY_DIR) $(EPAGES_SHARED)\Static
	$(DEL) $(EPAGES_LOG)\*

install_storetype:
	$(PERL) $(DE_EPAGES)/Installer/Scripts/installStoreType.pl $(CARTRIDGE) -storetype $(STORE_TYPE) -version $(EPAGES_VERSION_NUMBER)
	$(PERL) $(DE_EPAGES)/JQuery/Scripts/DictionaryConverter.pl -cartridge $(CARTRIDGE) -storetype $(STORE_TYPE) -version $(EPAGES_VERSION_NUMBER)

!ifndef DOJOBUILD
	$(MAKE) build_dojo STORE_TYPE=$(STORE_TYPE)
!endif
!ifndef JQUERYBUILD
	$(MAKE) build_jquery STORE_TYPE=$(STORE_TYPE)
!endif
!ifndef CSSBUILD
	$(MAKE) build_css STORE_TYPE=$(STORE_TYPE)
!endif

cleanfiles_storetype:
	$(DEL) $(EPAGES_CONFIG)\StoreTypes.d\$(EPAGES_VERSION_NUMBER)\$(STORE_TYPE).conf
	$(RMDIR) $(EPAGES_WEBROOT)\StoreTypes\$(EPAGES_VERSION_NUMBER)\$(STORE_TYPE)

reinstall_storetype: makefile cleanfiles_storetype install_storetype

cleanfilesdb: makefile test_store
	$(RMDIR) $(EPAGES_WEBROOT)\$(STORE)
	$(RMDIR) $(EPAGES_STORES)\$(STORE)
	$(DEL) $(EPAGES_CONFIG)\Cartridges.d\$(STORE).conf

cleandb: makefile test_store
	$(PERL) $(DE_EPAGES)/WebInterfacePoolDB/Scripts/deleteStore.pl -storename $(STORE)
	$(MAKE) delete_tables STORE=$(STORE)

import_store: test_store
	@$(PERL) $(DE_EPAGES)/ShopConfiguration/Scripts/store.pl -sitename $(SITE_NAME) -action insert -storename $(STORE)

import_distributor:
	$(IMPORT) $(DE_EPAGES)/ShopConfiguration/Database/XML/Distributor.xml

import_shoptypes: makefile test_store
	$(IMPORT) -path $(PROVIDER_PATH) $(DE_EPAGES)/ShopConfiguration/Database/XML/DistributorShopTypes.xml

install_demoshop: makefile
	$(PERL) $(DE_EPAGES)/ShopConfiguration/Scripts/createShop.pl \
		-wsuser $(PROVIDER_PATH)/Users/$(PROVIDER_USER) -passwd $(PROVIDER_PASSWD) \
		-shoptype $(DEMOSHOP_SHOPTYPE) -storename $(STORE) \
		-alias $(DEMOSHOP_ALIAS) -scriptname $(DEMOSHOP_ALIAS)
	$(PERL) $(DE_EPAGES)/ShopWizard/Scripts/runWizard.pl -storename $(STORE) -shop $(DEMOSHOP_ALIAS) -config $(EPAGES_CONFIG)/ShopWizard.conf -nooutput
	$(MAKE) permissions PERM_ARGS=-u SECTION=webroot

uninstall_demoshop: makefile
	$(PERL) $(DE_EPAGES)/ShopConfiguration/Scripts/deleteShop.pl -wsuser $(PROVIDER_PATH)/Users/$(PROVIDER_USER) -passwd $(PROVIDER_PASSWD) -alias $(DEMOSHOP_ALIAS)

install_diagnostics: makefile test_store
	cd $(DE_EPAGES)\Diagnostics && $(PERL) Makefile.PL
	cd $(DE_EPAGES)\Diagnostics && $(MAKE) install STORE=$(STORE) EPAGES_VERSION=$(EPAGES_VERSION_NUMBER)

install_store: makefile test_store
	$(INSTALLDB) -storename $(STORE) -sitename $(SITE_NAME)
	$(MAKE) synchronize STORE=$(STORE)
	$(MAKE) permissions PERM_ARGS=-u SECTION=webroot
	$(MAKE) permissions PERM_ARGS=-u SECTION=stores

synchronize: makefile test_store
!ifndef EPAGES_IGNORE_SITE
	$(SYNC) -storename $(STORE) -sitename $(SITE_NAME)
!endif

create_store_conf:
	$(PERL) $(DE_EPAGES)/ShopConfiguration/Scripts/createStoreConf.pl -storename $(STORE) -storetype $(STORE_TYPE) -backupname $(BACKUP_NAME)

new_store:
	$(MAKE) create_store_conf create_db import_store install_store STORE=$(STORE) STORE_TYPE=$(STORE_TYPE) BACKUP_NAME=$(BACKUP_NAME)

install_pooldb: makefile stop stop_cron
	$(DEL) $(EPAGES_CONFIG)\ASPool.db
	$(DEL) $(EPAGES_CONFIG)\Cartridges.d\ASPoolDB.conf
	$(INSTALLDB) -storename ASPoolDB
	$(MAKE) permissions start PERM_ARGS=-u

install_cpan: makefile
	$(IF_EXIST) $(MODULES_INI) $(DO) $(MAKE) install_cpan_epages

install_cpan_epages:
	$(INSTALL_CPAN) CPAN::EPAGES
	$(MAKE) permissions PERM_ARGS=-u

install: makefile
!ifdef WINDOWS
	$(RMDIR) $(DE_EPAGES)\HSBC\bin\MSWin32\CVS
	$(RMDIR) $(DE_EPAGES)\Presentation\bin\MSWin32\CVS
!endif
	$(IF_EXIST) $(MODULES_INI) $(DO) $(MAKE) stop install_cpan start
!ifdef EPAGES_IGNORE_SITE
	$(MAKE) install_storetype STORE_TYPE=Store
	$(MAKE) create_store_conf create_db install_store STORE=Store
!else
	$(MAKE) install_storetype STORE_TYPE=Site
	$(MAKE) create_store_conf create_db install_store STORE=Site STORE_TYPE=Site DATSIZE=50 LOGSIZE=20
	$(MAKE) install_storetype STORE_TYPE=Store
	$(MAKE) create_store_conf create_db import_store install_store STORE=Store
	$(MAKE) import_distributor STORE=Site
	$(MAKE) import_shoptypes STORE=Site
!endif
!ifndef EPAGES_IGNORE_DEMOSHOP
	$(MAKE) install_demoshop STORE=Store
!endif
	$(MAKE) start_cron

reinstall: check_cartridges_xml update_os makefile uninstall
	$(MAKE) install
!ifndef EPAGES_IGNORE_SITE
	$(MAKE) install_diagnostics STORE=Site EPAGES_VERSION_NUMBER=$(EPAGES_VERSION_NUMBER)
!endif
	$(MAKE) install_diagnostics STORE=Store EPAGES_VERSION_NUMBER=$(EPAGES_VERSION_NUMBER)

check_cartridges_xml:
	$(PERL) $(EPAGES_CARTRIDGES)\DE_EPAGES\XML\Scripts\sortCartridgesXml.pl -check

uninstall: makefile cleanfiles_global
	$(MAKE) install_pooldb
	$(DEL) $(EPAGES_CONFIG)\EbayCategories.db
!ifndef EPAGES_IGNORE_SITE
	$(MAKE) drop_db cleanfilesdb cleanfiles_storetype STORE=Site STORE_TYPE=Site EPAGES_VERSION_NUMBER=$(EPAGES_VERSION_NUMBER)
!endif
	$(MAKE) drop_db cleanfilesdb cleanfiles_storetype STORE=Store STORE_TYPE=Store EPAGES_VERSION_NUMBER=$(EPAGES_VERSION_NUMBER)

build_dojo:
!ifndef DOJOBUILD
	$(PERL) $(EPAGES_CARTRIDGES)/DE_EPAGES/Dojo/Scripts/build.pl -storetype $(STORE_TYPE) -version $(EPAGES_VERSION_NUMBER)
!endif

build_jquery:
!ifndef JQUERYBUILD
	$(PERL) $(EPAGES_CARTRIDGES)/DE_EPAGES/Presentation/Scripts/buildJS.pl -storetype $(STORE_TYPE) -version $(EPAGES_VERSION_NUMBER)
!endif

build_css:
!ifndef CSSBUILD
	$(PERL) $(EPAGES_CARTRIDGES)/DE_EPAGES/Presentation/Scripts/buildCSS.pl -storetype $(STORE_TYPE) -version $(EPAGES_VERSION_NUMBER)
!endif


reinstall_store: makefile test_store cleandb cleanfilesdb install_store

# create databases

create_db: makefile
	$(CREATEDB) -storename $(STORE) -datsize $(DATSIZE) -logsize $(LOGSIZE) -connection $(BACKUP_NAME)

drop_db: makefile
	$(DROPDB) -storename $(STORE)

# patch

close_store:
	$(STOREREDIRECT) -storename $(STORE) -redirect StoreClosed.html

open_store:
	$(STOREREDIRECT) -storename $(STORE) -remove

patch_store: makefile test_store
	$(PATCHDB)   -nosync -storename $(STORE)
	$(INSTALLDB) -nosync -storename $(STORE)

update_latest: makefile
	cvs -q update -A -d -P
	$(MAKE) permissions SECTION=cartridges
	$(MAKE) restart

latest: makefile update_latest reinstall
renew: latest

update_stable: makefile
	cvs -q update -d -P -r stable
	$(MAKE) permissions SECTION=cartridges
	$(MAKE) restart

stable: makefile update_stable reinstall

# files (obsolete)

backup_local_files: etc\backup_dir Apache\backup_dir Sybase\backup_dir bin\backup_dir Config\backup_dir Monitor\backup_dir
copy_local_files:   etc\copy_dir Apache\copy_dir Sybase\copy_dir bin\copy_dir Config\copy_dir Monitor\copy_dir Patches\copy_dir
patch_local_files:  backup_local_files copy_local_files
restore_local_files: etc\restore_dir Apache\restore_dir Sybase\restore_dir bin\restore_dir Config\restore_dir Monitor\restore_dir

backup_dir:
!ifdef FORGETFULLY
	$(PATCHFILES) -differential -dir "$(DIR)" -bak "$(BACKUP_DIR)/$(NAME)" -new "$(PATCH_DIR)/$(NAME)"
!else
	$(PATCHFILES) -action $(TGT)\$(NAME)\backup -differential -dir "$(DIR)" -bak "$(BACKUP_DIR)/$(NAME)" -new "$(PATCH_DIR)/$(NAME)"
!endif

copy_dir:
!ifdef FORGETFULLY
	$(PATCHFILES) -dir "$(DIR)" -new "$(PATCH_DIR)/$(NAME)"
!else
	$(PATCHFILES) -action $(TGT)\$(NAME)\copy -dir "$(DIR)" -new "$(PATCH_DIR)/$(NAME)"
!endif

restore_dir:
	$(PATCHFILES) -action $(TGT)\$(NAME)\restore -dir "$(DIR)" -bak "$(BACKUP_DIR)/$(NAME)"

update_os:
!ifdef WINDOWS
	if not exist $(EPAGES_PERL)\graphviz $(MAKE) B_patch_6_13_0\graphviz\install
	if exist B_patch_6_13_0 $(RMDIR) B_patch_6_13_0
!else
	if [ -f "$(PATCH_DIR)/sh/patch.conf" ] ; then\
 PATH="$(PATCH_DIR)"/sh/EPAGES_RELEASE/var/epages:/var/epages:$$PATH patch-sh.sh -d "$(PATCH_DIR)"/sh ;\
 rpm --quiet -q graphviz || yum -y install graphviz ; fi
!endif

B_patch_6_13_0\graphviz\install:
	$(MAKE) $(@D)
	if exist $(@D)\setx.cmd $(@D)\setx.cmd
	if exist $(@D)\graphviz-2.28.0.msi msiexec /quiet /passive /norestart /a $(@D)\graphviz-2.28.0.msi TARGETDIR=$(EPAGES_PERL)\graphviz
B_patch_6_13_0\graphviz:
!ifdef CVSROOT
	$(MKDIR) $@
	cvs export -r $(@D) -d $@ RND/epages/epages5/WinNT/Build/Perl/graphviz-2.28.0.msi
	cvs export -r $(@D) -d $@ RND/epages/epages5/AllOS/Patch/src/Scripts/setx.cmd
!endif

build_patch_sh:
!ifndef WINDOWS
	if type -p build-patch-sh.sh ; then rm -rf "$(PATCH_DIR)"/sh ;\
 build-patch-sh.sh -r "$(EPAGES_TAG)" -P "$(PATCH_TAG)" -t "$(PATCH_DIR)"/sh ; fi
!endif

etc\backup_dir etc\copy_dir etc\restore_dir:
	$(MAKE) $(@F) DIR=/etc NAME="$(@D)"
Apache\backup_dir Apache\copy_dir Apache\restore_dir:
	$(MAKE) $(@F) DIR="$(EPAGES_APACHE)" NAME="$(@D)"
Sybase\backup_dir Sybase\copy_dir Sybase\restore_dir:
	$(MAKE) $(@F) DIR="$(SYBASE)" NAME="$(@D)"
bin\backup_dir bin\copy_dir bin\restore_dir:
	$(MAKE) $(@F) DIR="$(EPAGES)/bin" NAME="$(@D)"
Config\backup_dir Config\copy_dir Config\restore_dir:
	$(MAKE) $(@F) DIR="$(EPAGES_CONFIG)" NAME="Shared/$(@D)"
Monitor\backup_dir Monitor\copy_dir Monitor\restore_dir:
	$(MAKE) $(@F) DIR="$(EPAGES_SHARED)/Monitor" NAME="Shared/$(@D)"
Patches\copy_dir:
	$(MAKE) $(@F) DIR="$(EPAGES_SHARED)/Patches" NAME="Shared/$(@D)"

permissions:
!ifndef WINDOWS
	$(ETC_INITD) perm admin $(PERM_ARGS)
!ifdef SECTION
	$(ETC_INITD) perm $(SECTION) $(PERM_ARGS)
!endif
!endif

install_multistore: makefile
	$(MAKE) create_store_conf install_storetype STORE=$(MASTER_NAME) STORE_TYPE=$(MASTER_TYPE)
	$(MAKE) create_store_conf install_storetype STORE=$(MULTISTORE_NAME) STORE_TYPE=$(MULTISTORE_TYPE)
	cd $(DE_EPAGES)\MultiStoreProvider && $(PERL) Makefile.PL
	cd $(DE_EPAGES)\MultiStoreProvider && $(MAKE) install STORE=$(SITE_NAME) EPAGES_VERSION=$(EPAGES_VERSION_NUMBER)
	$(MAKE) new_store STORE=$(MASTER_NAME) STORE_TYPE=$(MASTER_TYPE)
	$(MAKE) new_store STORE=$(MULTISTORE_NAME) STORE_TYPE=$(MULTISTORE_TYPE)
	$(MAKE) import_multistore_shoptypes
	$(MAKE) import_multistore_distributor

uninstall_multistore: makefile
	$(MAKE) cleanfiles_storetype STORE_TYPE=$(MASTER_TYPE)
	$(MAKE) cleanfiles_storetype STORE_TYPE=$(MULTISTORE_TYPE)
	cd $(DE_EPAGES)\MultiStoreProvider && $(PERL) Makefile.PL
	cd $(DE_EPAGES)\MultiStoreProvider && $(MAKE) uninstall STORE=$(SITE_NAME) EPAGES_VERSION=$(EPAGES_VERSION_NUMBER)
	$(MAKE) drop_db cleanfilesdb STORE=$(MASTER_NAME)
	$(MAKE) drop_db cleanfilesdb STORE=$(MULTISTORE_NAME)

reinstall_multistore: makefile
	$(MAKE) uninstall_multistore
	$(MAKE) reinstall
	$(MAKE) install_multistore
	$(MAKE) install_multistore_demo
	$(MAKE) install_diagnostics STORE=$(MASTER_NAME)
	$(MAKE) install_diagnostics STORE=$(MULTISTORE_NAME)

install_multistore_demo: makefile
	$(MAKE) install_mastershop
	$(MAKE) install_subshop
	$(MAKE) install_cacheshop
	$(MAKE) assign_subshop_to_mastershop
	$(MAKE) clone_mastershop STORE=$(MULTISTORE_NAME)

install_mastershop: makefile
	$(MAKE) install_demoshop STORE=$(MASTER_NAME) DEMOSHOP_ALIAS=$(MASTERSHOP_ALIAS) PROVIDER_PATH=$(MASTER_PROVIDER_PATH) PROVIDER_USER=$(MASTER_PROVIDER_USER) DEMOSHOP_SHOPTYPE=$(MASTERSHOP_SHOPTYPE)
	$(PERL) $(DE_EPAGES)/Shop/Scripts/setMaxFeature.pl -storename $(MASTER_NAME) -shopname $(MASTERSHOP_ALIAS) -featurename MultiStoreMaster -featurevalue 1

install_subshop: makefile
	$(MAKE) install_demoshop STORE=$(MULTISTORE_NAME) DEMOSHOP_ALIAS=$(SUBSHOP_ALIAS) PROVIDER_PATH=$(MULTISTORE_PROVIDER_PATH) PROVIDER_USER=$(MULTISTORE_PROVIDER_USER) DEMOSHOP_SHOPTYPE=$(SUBSHOP_SHOPTYPE)

install_cacheshop: makefile
	$(PERL) $(DE_EPAGES)/MultiStore/Scripts/createCacheShop.pl -storename $(MULTISTORE_NAME) -masterstore $(MASTER_NAME) -mastershop $(MASTERSHOP_ALIAS) -cacheshop $(CACHESHOP_ALIAS)

import_multistore_shoptypes: makefile
	$(PERL) $(DE_EPAGES)/Object/Scripts/import.pl -storename $(SITE_NAME) -path $(MASTER_PROVIDER_PATH) $(DE_EPAGES)/MultiStoreProvider/Database/XML/DistributorShopTypesDropShipping.xml
	$(PERL) $(DE_EPAGES)/Object/Scripts/import.pl -storename $(SITE_NAME) -path $(MASTER_PROVIDER_PATH) $(DE_EPAGES)/MultiStoreProvider/Database/XML/DistributorShopTypesERetail.xml

import_multistore_distributor: makefile
	$(PERL) $(DE_EPAGES)/Object/Scripts/import.pl -storename $(SITE_NAME) $(DE_EPAGES)/MultiStoreProvider/Database/XML/DistributorMultiStore.xml

assign_subshop_to_mastershop: makefile
	$(PERL) $(DE_EPAGES)/MultiStoreProvider/Scripts/assignSubShopToMasterShop.pl -mastershop $(MASTERSHOP_ALIAS) -subshop $(SUBSHOP_ALIAS) -affiliateshopid='$(SUBSHOP_ALIAS)' -orderintegration BasketHandover

clone_mastershop: makefile test_store
	$(PERL) $(DE_EPAGES)/MultiStore/Scripts/cloneMasterShop.pl -storename $(STORE)

#
# END
#
