/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.presentation.ajaxTransport
 *
 * @revision		$Revision: 1.6.20.1 $
 */

jQuery.plugin( 'de_epages.presentation.ajaxTransport', [/*IFRAME: 'jQuery.support.multipleUpload', 'jQuery.uid',*/ 'jQuery.mime'], function( $, undefined ){

	// extend the ajax settings
	if( !$.ajaxSettings.fileSizeMax ){
		$.ajaxSettings.fileSizeMax = 2097152;	// set max size of files default: 2MB
	}

		// use html5 as default
	var html5 = true,
		// url query params
		rquery = /\?/,
		// to string method
		toString = Object.prototype.toString,
		// map of advanced errors
		statusMap = {
			102: { status:102, errorThrown:'Processing' },
			200: { status:200, errorThrown:'Ok' },
			290: { status:290, errorThrown:'Ok' },							// Item
			408: { status:408, errorThrown:'Request Timeout' },
			490: { status:490, errorThrown:'Bad Item Request' },
			491: { status:491, errorThrown:'Items of Request Failed' },
			492: { status:492, errorThrown:'Empty Request' },				// Item
			493: { status:493, errorThrown:'Mimetype Not Accept Error' },	// Item
			494: { status:494, errorThrown:'File Size Limit Error' },		// Item
			495: { status:495, errorThrown:'Feature Limit Exceeded' },		// Item
			496: { status:496, errorThrown:'Invalid Image File' },	// Item
			497: { status:497, errorThrown:'Image Exceeds Megapixels' },	// Item
			498: { status:498, errorThrown:'Invalid MP3 File' },	// Item
			499: { status:499, errorThrown:'Virus Found Error' },   // Item
			500: { status:500, errorThrown:'Internal Server Error' },
			590: { status:590, errorThrown:'Internal Server Error' },		// Item
			599: { status:599, errorThrown:'Unknow Error' }
		},
		// local Error handle
		errorGet = function( extended, code, errorThrown ){
			if( extended!==true ){
				errorThrown = code;
				code = extended;
				extended = false;
			}

			var data = statusMap[ code ] || { status:code, errorThrown: errorThrown || 'Unknow Error' },
				text;

			text = '{"status":'+data.code+',"errorThrown":"'+data.errorThrown+'"}}';

			return extended ? {text:'{"ERROR":'+text+'}',data:{ERROR: data}} : {text:text,data:	data};
		},
		// upload single file with ajax
		singleAJAX = function( o, originalOptions, jqXHR ){
			var xhr = o.xhr(),
				send = xhr.send,
				fileObj = o.file;

			// modify options
			o.data = null;
			o.type = 'post';
			o.async = true;
			o.xhr = function(){
				return xhr;
			};
			// add progress handle
			if( o.progress ){
				xhr.upload.onprogress = o.progress;
			}
			// modify send xhr
			xhr.send = function(){
				xhr.setRequestHeader( 'Content-Type', 'application/octet-stream' );
				xhr.setRequestHeader( 'If-Modified-Since', 'Mon, 26 Jul 1997 05:00:00 GMT' );
				xhr.setRequestHeader( 'Cache-Control', 'no-cache' );
				xhr.setRequestHeader( 'content-disposition', 'attachment; filename="'+encodeURIComponent(fileObj.fileName||fileObj.name)+'"; name="'+o['_inputName']+'"' );

				if(xhr.sendAsBinary && typeof fileObj.getAsBinary  == "function") {
					xhr.sendAsBinary(fileObj.getAsBinary());
				}
				else{
					send.call(xhr,fileObj);
				}
			};
		},
		// handle multiple files to upload with ajax
		multiAJAX = function( o, originalOptions, jqXHR ){
			// state whether xhr abort
			var isAbort = false,
			// state whether error-caused
				isError = false;
			// results send and abort handle
			return {
				send: function( headers, completeCallback ){
					var inputName = o['_inputName'],
						fileStack = $.merge([],o.file.files),
						fileStackLength = fileStack.length,
						// response data store
						responseText = [],
						responseData = [],
						headerStack = [],
						// completeCallback
						nextItem = function( i ){
							if( i<fileStackLength ){
								var fileObj = fileStack[i],
									fileItem = {
										name:		inputName,
										fileName:	fileObj.name || fileObj.fileName,
										fileSize:	fileObj.size || fileObj.fileSize,
										fileType:	fileObj.type
									},
									// setup options for AJAX item
									xhrItemOptions = $.extend( {}, originalOptions, {
										_inputName:	inputName,
										file:		fileObj,
										progress:	function( event ){
											if( isAbort ){
												// abort item if main xhr is abort
												xhrItem.abort();
											}
											else if( o.progress ){
												// set status of main xhr
												jqXHR.status = 102;
												jqXHR.statusText = 'progress-item';
												// call progress handle
												o.progress.call(
													o.context,
													// modify progress event object
													$.extend(event,{
														item:				i+1,
														itemTotal:			fileStackLength,
														itemLoaded:			i,
														data:				{item:fileItem}
													}),
													jqXHR.statusText,
													jqXHR
												);
											}
										},
										success:	function( data, textStatus, xhrItem ){
											// overwrite original status
											textStatus = 'success-item';
											var status = 290;

											if( isAbort ){
												// abort item if main xhr is abort
												xhrItem.abort();
											}
											else{
												if( data.ERROR ){
													data = {
														item: fileItem,
														ERROR: errorGet(data.ERROR.status || 590, data.ERROR.errorThrown).data
													};
													// set error status
													isError = true;
													status = data.ERROR.status;
													textStatus = 'error-item';
												}
												else{
													// store data
													responseText.push(xhrItem.responseText);
													responseData.push(data);
													data = {success:data, item:fileItem};
												}
												// call progress handle
												if( o.progress ){
													// set status of main xhr
													jqXHR.status = status;
													jqXHR.statusText = textStatus;
													// call progress handle
													o.progress.call(
														o.context,
														// simulate progress event object
														$.extend($.Event(),{
															type:				'progress',
															lengthComputable:	true,
															position:			fileItem.fileSize,
															total:				fileItem.fileSize,
															loaded:				fileItem.fileSize,
															item:				i+1,
															itemTotal:			fileStackLength,
															itemLoaded:			i+1,
															data:				data
														}),
														textStatus,
														jqXHR
													);
												}
											}
										},
										error:	function( xhrItem, textStatus, errorThrown ){
											// use the advanced error handling from success callback
											var error = errorGet( true, xhrItem.status, errorThrown );
											xhrItem.responseText = error.text;
											xhrItemOptions.success( error.data, textStatus, xhrItem );
										},
										complete:	function( xhrItem ){
											if( isAbort ){
												xhrItem.abort();
											}
											else{
												// store responseHeaders
												headerStack.push( xhrItem.getAllResponseHeaders() );
												// start xhr for next item
												nextItem(i+1);
											}
										}
									}),
									// xhr object for current item
									xhrItem;

								// check accept mimetype of file
								if( !$.mime(fileItem.fileType,o.fileMimeAccept) ){
									xhrItemOptions.dataType = 'ERROR';
									xhrItemOptions.ERROR = errorGet(493).data;
								}
								else if( (fileItem.fileSize) > o.fileSizeMax ){
									xhrItemOptions.dataType = 'ERROR';
									xhrItemOptions.ERROR = errorGet(494).data;
								}
								// call AJAX
								xhrItem = $.ajax(xhrItemOptions);

							}
							else{
								var status = 200,
									statusText = 'success';
								if(	isError ){
									status = 491;
									statusText = errorGet(491).data.errorThrown;
								}
								else if( !responseData.length ){
									status = 492;
									statusText = errorGet(492).data.errorThrown;
								}
								// build responseText from all responses
								jqXHR.responseText = '['+responseText.join(',')+']';
								jqXHR.responseXML = null;
								// execute callback
								completeCallback(
									status,
									statusText,
									{
										json:	responseData,
										text:	jqXHR.responseText
									},
									headerStack.join('')
								);
							}
						};

					// start first xhr for first item
					nextItem(0);
				},
				abort: function( statusText ){
					// set abort state
					isAbort = true;
				}
			};
		};

/************************************************************************
 * ALL BROWSER WE SUPPORT IN BACKOFFICE USES AJAX BINARY UPLAOD
 *-----------------------------------------------------------------------
 * So we disable the upload via iframe
 ************************************************************************

	// setup support of html5 upload
	html5 = $.support.multipleUpload;
		// container for iframes
	var	container,
		// handle single file upload with iframe
		singleIFRAME = function( o, originalOptions, jqXHR ){
			o.url = originalOptions.url + (rquery.test(originalOptions.url) ? '&' : '?') + $.param( originalOptions.data );
			// state whether xhr abort
			var isAbort = false,
				isTimeout = false,
				isComplete = false,
				isSuccess = false;
			// results send and abort handle
			return {
				send: function( headers, completeCallback ){
					var id = $.uid('upload'),
						// iframe
						io,
						ioNode,
						// form
						form,
						formNode,
						// file input
						file = $(o.file),
						fileNode = o.file,
						filePlaceholder,
						// simulate fileObj
						fileItem = {
							fileName: file.val(),
							fileSize: 204800
						};
						// get mime type of file
						var temp = /\.([^.]+)$/.exec(fileItem.fileName);
						fileItem.fileType = $.mime(temp ? temp[1] : '');
						// complete handle
					var	complete = function( event, responseText ){
							clearInterval(interval);
							if( !isAbort && !isComplete ){
								isComplete = true;

								var status,
									statusText,
									responseData;

								// get response text
								if( !responseText ){
									if( isTimeout ){
										responseText = errorGet(true,408).text;
									}
									else if( ioNode.contentWindow && ioNode.contentWindow.document ){
										responseText = ioNode.contentWindow.document.body ? ioNode.contentWindow.document.body.innerHTML : '';
									}
									else if( ioNode.contentDocument && ioNode.contentDocument.document ){
										responseText = ioNode.contentDocument.document.body ? ioNode.contentDocument.document.body.innerHTML : '';
									}
								}

								// try to parse JSON from response text
								try{
									responseData = $.parseJSON(responseText);
								}
								catch(error){
									error = errorGet( true, 490 );
									responseText = error.text;
									responseData = error.data;
								}

								// set status
								if( responseData && !responseData.ERROR ){
									status = 200;
									statusText = 'success';
								}
								else{
									var error = errorGet(490);
									status = responseData.ERROR.status || error.data.status;
									statusText = responseData.ERROR.errorThrown || error.data.errorThrown;
								}

								// set success state
								isSuccess = status===200;

								// handle progress item success/error
								if( o.progress ){
									// set date for progress event
									var progressEventData = {item:fileitem};
									if( responseData.ERROR ){
										progressEventData.ERROR = responseData.ERROR;
									}
									else{
										progressEventData.success = progressEventData;
									}
									// set status of main xhr
									jqXHR.status =		isSuccess ? 290 : status;
									jqXHR.statusText =	isSuccess ? 'success-item' : 'error-item';
									// call progress handle
									o.progress.call(
										o.context,
										// simulate progress event object
										$.extend($.Event(),{
											type:				'progress',
											lengthComputable:	true,
											position:			204800,
											total:				204800,
											loaded:				204800,
											item:				1,
											itemTotal:			1,
											itemLoaded:			1,
											data:				progressEventData
										}),
										jqXHR.statusText,
										jqXHR
									);
								}

								// set status of main xhr
								if( isSuccess ){
									jqXHR.status =		290;
									jqXHR.statusText =	'success';
								}
								else{
									jqXHR.status =		491;
									jqXHR.statusText =	errorGet(491).data.errorThrown;
								}
								// build responseText from all responses
								jqXHR.responseText = '['+responseText+']';
								jqXHR.responseXML = null;
								// execute callback
								completeCallback(
									jqXHR.status,
									jqXHR.statusText,
									{
										json:	responseData ? [responseData] : [],
										text:	jqXHR.responseText
									},
									''
								);
							}
						},
						// progress handle
						interval,
						progressLoaded = 0,
						progress = function(){
							if( !isAbort ){
								jqXHR.status = 102;
								var textStatus = jqXHR.statusText = 'progress-item';

								progressLoaded += 2048;

								if( progressLoaded > 203776 ){
									clearInterval(interval);
									progressLoaded = 203776;
								}

								o.progress.call(
									o.context,
									// simulate progress event object
									$.extend($.Event(),{
										type:				'progress',
										lengthComputable:	true,
										position:			progressLoaded,
										total:				204800,
										loaded:				progressLoaded,
										item:				1,
										itemTotal:			1,
										itemLoaded:			0,
										data:				{item:fileItem}
									}),
									textStatus,
									jqXHR
								);
							}
						};

					// check accept mimetype of file
					if( $.mime(fileItem.fileName,o.fileMimeAccept) ){
						complete( {}, errorGet(true,493).text );
					}
					else{
						// iframe container
						container = container || $('<div>')
							.css('display','none')
							.appendTo( document.body );
						// iframe to send form
						io = $('<iframe name="'+id+'"></iframe>').appendTo( container );
						ioNode = io[0];
						// bind success/error callback event
						if( window.attachEvent ){
							ioNode.attachEvent( 'onload', complete );
						}
						else{
							ioNode.addEventListener( 'load', complete, false );
						}
						// form
						form = $('<form>')
							.attr({
								target:		id,
								method:		'POST',
								action:		o.url,
								enctype:	'multipart/form-data',
								encoding:	'multipart/form-data'
							})
							.appendTo( container );
						formNode = form[0];

						if( o.timeout > 0 ){
							// Timeout checker
							setTimeout(function(){
								// Check to see if the request is still happening
								if( !isAbort ){
									isTimeout = true;
									complete();
								}
							}, o.timeout);
						}

						// run progress if defined
						if( o.progress ){
							// simulate xhr progress interval
							interval = setInterval(progress,25);
						}

						// send file
						filePlaceholder = file.clone().insertAfter(file).attr('disabled',true);
						form.append(file);
						formNode.submit();
						file.insertBefore(filePlaceholder);
						filePlaceholder.remove();
						form.remove();
					}
				},
				abort: function( statusText ){
					// set abort state
					isAbort = true;
				}
			};
		};
*/

	$.ajaxTransport( 'json', function( o, originalOptions, jqXHR ){
		// transport file
		if( o.file ){
				// be sure o.file is not modified
			var fileData = o.file = originalOptions.file,
				fileObj  = toString.call(fileData)==='[object File]',
				fileElem = fileObj ? $() : $(fileData);

			if( fileElem.is(':file') ){
				// prepare options
				o['_inputName'] = fileElem.attr('name') || 'NewFile';
				o.fileMimeAccept = o.fileMimeAccept || fileElem.attr('accept') || '*/*';
				originalOptions.data = $.extend( true, {ErrorAction:'JSONAjaxTransportFileError'}, originalOptions.data );
				if( html5 ){
					return multiAJAX( o, originalOptions, jqXHR );
				}
/*IFRAME: 		else{
					return singleIFRAME( o, originalOptions, jqXHR );
				}*/
			}
			else if( fileObj && o['_inputName'] ){
				return singleAJAX( o, originalOptions, jqXHR );
			}
		}
	});

	// handle client error to prevent call a real xhr
	$.ajaxTransport( 'ERROR', function( o, originalOptions, jqXHR ){
		return {
			send: function( headers, completeCallback ){
				var error = o.ERROR || errorGet(490).data;
				completeCallback( error.status, error.errorThrown, {}, '' );
			}
		};
	});

});