/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.presentation.ui.history
 *
 * @revision		$Revision: 1.7.6.1 $
 */

jQuery.plugin( 'de_epages.presentation.ui.history', {
	plugin:	['de_epages.presentation.history', 'jQuery.ui.widget', 'jQuery.tmpl', 'jQuery.dict', 'ep.alert', 'ep.ajax'],
	tmpl:	'de_epages.presentation.ui.history',
	dict:	'de_epages.presentation.dictionary'
}, function( $, undefined ){

/*
 * @dictionary		de_epages.presentation.dictionary
 *
 * @translation		{NoHistoryItems}
 *					{ObjectNoLongerExists}
 *					{AddToFavorites}
 *					{ClearHistory}
 */

	var historyHandle			= de_epages.presentation.history,
		presentationDict		= $.dict('de_epages.presentation.dictionary'),
		tNoHistoryItems			= presentationDict.translate('NoHistoryItems'),
		tObjectNoLongerExists	= presentationDict.translate('ObjectNoLongerExists'),
		tAddToFavorites			= presentationDict.translate('AddToFavorites'),
		tClearHistory			= presentationDict.translate('ClearHistory');

	// widget to display history
	$.widget( 'de_epages.presentation.ui.History', {

		_create: function(){
			this.element
				.addClass('de_epages-presentationUiHistory');
		},
		
		_init: function(){
			this._refreshList();
			if( window.dojo && window.$$ ){
				this._dojoFavorites = $$('favoritesWidget');
			}
		},
		
		_refreshList: function(){
			var tmpl = $.tmpl(
					'de_epages.presentation.ui.history',
					{
						tNoHistoryItems:	tNoHistoryItems,
						tAddToFavorites:	tAddToFavorites,
						tClearHistory:		tClearHistory,
						history:			historyHandle.get()
					},
					{
						scrunch: function( string ){
							return ep.scrunch( string, {ratio:0.75,length:30} );
						}
					}
				);
				
			this.element
				.empty()
				.append( tmpl )
				.find('li > a')
					.on( 'click', this.Proxy('_gotToHistoryPoint') );
		},
		
		_gotToHistoryPoint: function( event ){
			event.preventDefault();
			
			var self			= this,
				elem			= $(event.target),
				historyIndex	= elem.data('index'),
				historyItem		= ( historyHandle.get() )[ historyIndex ];

			if( elem.is('.de_epages-presentationUiHistory-clear *') ){
				historyHandle.remove();
				self._refreshList();
			}
			else if( elem.is('.addToFavorite') ){
				// add to favorites
				event.stopPropagation();
				if( this._dojoFavorites ){
					this._dojoFavorites.addHistoryToFavorites({
						Name:		historyItem.title,
						ObjectID:	historyItem.objectId,
						URI:		'?ViewAction='+historyItem.viewAction+'&ObjectID='+historyItem.objectId
					});
				}
			}
			else{
				// check object exists
				ep.ajax({
					dataType:	'json',
					data:		{
						'ViewAction':		'JSONObjectExists',
						'ObjectID':			ep.config.objectId,
						'CheckObjectID':	historyItem.objectId
					},
					success: function( check ){
						if( check.ObjectExists ){
							// got to history point
							location.href = '?ViewAction='+historyItem.viewAction+'&ObjectID='+historyItem.objectId;
						}
						else{
							// show error
							ep.alert({
								content:	tObjectNoLongerExists,
								ok:			function(){}
							});
							
							historyHandle.removeByObjectId(historyItem.objectId);
							self._refreshList();
						}
					}
				});
			}
		},
		
		destroy: function(){
			this.element
				.removeClass('de_epages-presentationUiHistory')
				.empty();
		
			this.Inherited(arguments);
		}

	});

});
