/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.presentation.ui.uploader
 *
 * @revision		$Revision: 1.15.8.1.4.1 $
 */

jQuery.plugin( 'de_epages.presentation.ui.uploader', {
	plugin: ['jQuery.dict', 'jQuery.tmpl','jQuery.i18n', 'jQuery.expr', 'jQuery.fn.form', 'jQuery.ui.widget', 'jQuery.ui.progressbar', 'ep.ui.dialog', 'de_epages.core', 'de_epages.presentation.ajaxTransport'],
	dict:	['ep.dict', 'de_epages.presentation.dictionary'],
	tmpl:	['de_epages.presentation.ui.uploaderProgress','de_epages.presentation.ui.uploaderError']
}, function( $, undefined ){

/*
 * @dictionary		ep.dict
 *
 * @translation		{Cancel}
 *					{Ignore}
 *
 * @dictionary		de_epages.presentation.dictionary
 *
 * @translation		{FileUploadSingle}
 *					{FileUploadMultiple}
 *					{UploaderError}
 *					{FixErrors}
 *					{FilesUploaded}
 *					{FileSizeLimitError}
 *					{InternalServerError}
 *					{MimetypeNotAcceptError}
 *					{UnknownError}
 *					{FeatureLimitExceeded}
 *					{InvalidMP3File}
 *					{InvalidImageFile}
 *					{ImageExceedsMegapixels}
 *					{VirusFoundError}
 */

	var epDict				= $.dict('ep.dict'),
		tCancel				= epDict.translate('Cancel'),
		tIgnore				= epDict.translate('Ignore'),
		presentationDict	= $.dict('de_epages.presentation.dictionary'),
		tUploadFile			= presentationDict.translate('FileUploadSingle'),
		tUploadFiles		= presentationDict.translate('FileUploadMultiple'),
		tUploaderError		= presentationDict.translate('UploaderError'),
		tFixErrors			= presentationDict.translate('FixErrors'),
		tFilesUploaded		= presentationDict.translate('FilesUploaded');

	$.widget( "de_epages.presentation.ui.Uploader", {
		options: {
		//	multiple:	false,
		//	accept:		'*/*',
			style:		'normal',
			big:		false,
			callback:	$.noop,
			url:		'?',
			data:		{
				'ViewAction':	'JSONViewResponse'
			}
		},

		_create: function(){
			var o = this.options;

			this.elem = this.element
				.sub(ep)
				.addClass('ep-uiInput-hidden');
			// create a label as wrap
			this.wrap = ep('<label>')
				.addClass(
					'ep-uiInput-wrap de_epages-presentationUiUploader-button'
					+ ( this.elem.is(':disabled') ? ' disabled' : '' )
					+ ( this.elem.is(':readonly') ? ' readonly' : '' )
					+ ( this.elem.is(':checked') ? ' checked' : '' )
				)
				.insertBefore(this.elem)
				.append(this.elem);
			// create pseudo button
			this.button = ep('<span></span>')
				.addClass('ep-uiInput ep-uiInput-base ep-uiInput-button')
				.insertBefore(this.elem);
			// init options
			this.option({
				'accept':	$.mime(o.accept || this.elem.attr('accept') || '*/*'),
				'multiple':	o.multiple || this.elem.attr('multiple') || false,
				'style':	o.big ? 'big' : o.style
			});
			// bind upload handle
			this.elem.on( 'change', this.Proxy('_upload') );
			// stack of item data
			this.dataStack = [];
		},

		_setOption: function( name, value ){
			var o = this.options;

			if( value!==undefined ){
				switch( name ){
					case 'accept':
						this.elem.attr( name, $.mime(value) );
					break;
					case 'multiple':
						this.elem.attr( name, value );
						this._setOption( 'style', o.style );
					break;
					case 'big':
						this._setOption( 'style', 'big' );
					break;
					case 'style':
						switch( value ){
							case 'big':
								this.button
									.html( '<span class="ep-sprite ep-sprite-s ico_s_fileupload"></span>'+(o.multiple ? tUploadFiles : tUploadFile) )
									.addClass('ep-uiInput-big');
							break;
							case 'icon':
								this.button
									.html( '<span class="ep-sprite ep-sprite-s ico_s_fileupload ep-uiInput-buttonSpriteOnly"></span>' )
									.removeClass('ep-uiInput-big');
							break;
							case 'big-icon':
								this.button
									.html( '<span class="ep-sprite ep-sprite-m ico_m_fileupload ep-uiInput-buttonSpriteOnly"></span>' )
									.addClass('ep-uiInput-big');
							break;
							default:
								this.button
									.html( '<span class="ep-sprite ep-sprite-s ico_s_fileupload"></span>'+(o.multiple ? tUploadFiles : tUploadFile) )
									.removeClass('ep-uiInput-big');
							break;
						}
					break;
				}
			}
			return this.Inherited( arguments );
		},

		_upload: function(){
			var self = this,
				o = this.options;

			this.statusText = '';

			this._errorClose({
				close: $.noop
			});
			this._progressOpen({
				close: function( event ){
					self._uploadCancel();
				}
			});

			for( var i=0,iLength=this.dataStack.length ; i<iLength ; i++ ){
				if( this.dataStack[i].ERROR ){
					this.dataStack.splice(i,1);
					iLength--;
					i--;
				}
			}

			this.uploadXHR = $.ajax({
				url:			o.url,
				data:			o.data,
				dataType:		'json',
				file:			this.elem[0],
				fileMimeAccept:	o.accept,
				progress:		this.Proxy('_uploadProgress'),
				success:		this.Proxy('_uploadSuccess'),
				error:			this.Proxy('_uploadError')
			});
		},

		_uploadCancel: function(){
			this.uploadXHR.abort();
			this._errorClose();
		},

		_uploadProgress: function( event, statusText, jqXHR ){
			var valueItem = event.loaded * 100 / event.total,
				valueAll = event.itemLoaded * 100 / event.itemTotal;

			this.progressbarItem.progressbar( 'value', valueItem );
			this.progresstextItem.text( ep.scrunch( event.data.item.fileName, {ratio:0.5,length:65} ) );
			this.progressbarAll.progressbar( 'value', valueAll );
			this.progresstextAll.text( tFilesUploaded+': '+event.itemLoaded+' / '+event.itemTotal );

			if( /(success|error)/.test(statusText) ){
				this.dataStack.push( event.data );
			}
		},

		_uploadSuccess: function( data, statusText, jqXHR ){
			this.statusText = statusText;
			this._progressClose({
				close: $.noop
			});
			this._uploadComplete();
		},

		_uploadError: function( jqXHR, statusText, errorThrown ){
			var self = this;
			this.statusText = statusText;
			this._errorCreate();
			this.error.element
				.html(
					$.tmpl(
						'de_epages.presentation.ui.uploaderError',
						[
							{items: this.dataStack}
						],
						{
							i: 0,
							even: function(){
								return (this.i++)%2==0;
							},
							scrunch: function( string ){
								return ep.scrunch( string, {ratio:0.5,length:40} );
							},
							translate: function( string ){
								var before = string.replace(/ /g,''),
									after = presentationDict.translate( before );

								return after!==before ? after : presentationDict.translate('UnknownError');
							}
						}
					)
				);
			this._progressClose({
				close: $.noop
			});
			this._errorOpen({
				close: function(){
					self.elem.appendTo(self.wrap);
					self._uploadComplete();
				}
			});
		},

		_uploadComplete: function(){
			var o			= this.options,
				dataStack	= this.dataStack,
				statusText	= this.statusText;

			this.elem.formReset();
			this.dataStack = [];
			this.statusText = '';

			if( $.type(o.callback)=='string' ){
				location.href = o.callback;
			}
			else{
				o.callback( dataStack, statusText );
			}
		},

		_progressOpen: function( options ){
			if( !this.progress ){
				this._progressCreate();
			}
			if( options ){
				this.progress.option(options);
			}
			this.progress.open();
		},

		_progressClose: function( options ){
			if( this.progress ){
				if( options ){
					this.progress.option(options);
				}
				this.progress.close();
			}
		},

		_progressCreate: function(){
			if( !this.progress ){
				var self = this;

				this.progress = ep('<div class="de_epages-presentationUiUploader">')
					.html( $.tmpl('de_epages.presentation.ui.uploaderProgress') )
					.uiDialog({
						width:		350,
						modal:		true,
						autoOpen:	false,
						title:		self.options.multiple ? tUploadFiles : tUploadFile,
						buttons:	{
							cancel:	{text:tCancel,	click:function(){ self._progressClose(); }}
						}
					})
					.find('.de_epages-presentationUiUploader-progressBar')
						.progressbar()
						.each(function(i){
							var elem = $(this);
							self[ i ? 'progressbarAll' : 'progressbarItem'] = elem;
							self[ i ? 'progresstextAll' : 'progresstextItem'] = elem.next();
						})
					.end()
					.uiDialog('Instance');
			}
		},

		_errorOpen: function( options ){
			if( !this.error ){
				this._errorCreate();
			}
			if( options ){
				this.error.option(options);
			}
			this.error.open();
		},

		_errorClose: function( options ){
			if( this.error ){
				if( options ){
					this.error.option(options);
				}
				this.error.close();
			}
		},

		_errorCreate: function(){
			if( !this.error ){
				var self = this;

				this.error = ep('<div class="de_epages-presentationUiUploader">')
					.html( $.tmpl('de_epages.presentation.ui.uploaderError') )
					.uiDialog({
						width:		350,
						modal:		true,
						autoOpen:	false,
						open:		function(){
							self.elem.insertAfter(self.error.options.buttons.fixit);
						},
						title:		tUploaderError,
						buttons:	{
							fixit:	{text: tFixErrors,'class':'epWidth-160'},
							ignore:	{text: tCancel,	click:function(){ self._errorClose(); }}
						}
					})
					.uiDialog('Instance');
			}
		},

		destroy: function(){
			this.Inherited();
			if(this.error){
				ep(this.error).uiDialog('destroy');
			}
			if(this.progress){
				ep(this.progress).uiDialog('destroy');
			}
		}

	});

});
