/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.product.ui.appointmentpicker
 *
 * @revision		$Revision: 1.13.4.2 $
 */

jQuery.plugin( 'de_epages.product.ui.appointmentpicker', {
	plugin:	['de_epages.core', 'ep.ui.datepicker', 'ep.alert', 'ep.ajax', 'ep.dict'],
	dict:	'de_epages.product.dictionary'
}, function( $, undefined ){

/*
 * @dictionary		de_epages.product.dictionary
 *
 * @translation		{Loading}
 *					{BookedUp}
					{CALENDAR_DATA_NOT_LOAD}
 */

 	var dict			= ep.dict('de_epages.product.dictionary'),
 		tLoading		= dict.translate('Loading')+'...',
 		tBookedOut		= dict.translate('BookedUp'),
 		tDataNotLoad	= dict.translate('CALENDAR_DATA_NOT_LOAD');

	$.widget('de_epages.product.ui.Appointmentpicker',ep.ui.Datepicker,{

		options: {
	// ep.ui.Input
		//	big:				true,
		//	autofocus:			false,
		//	placeholder:		'text',

	// ep.ui.Validate
		//	accept:				'',
		//	min:				1,
		//	max:				20,
		//	minlength:			1,
		//	maxlength:			4,
		//	pattern:			[\w]+,
		//	required:			true,
		//	tooltipOffset:		[0,-7],
		//	valid:				true,
		//	type:				'basic',
		//	format:				'',

	// ep.ui.Datepicker
		//	type:				'date',
		//	format:				'l',
		//	showOn:				'both',
		//	duration:			'normal',
		//	show:				'fadeIn',
		//	hide:				'fadeOut',
		//	weekDaysAvailable:	[0,0,1,1,1,1,1],
		//	numberOfMonths:		1,
		//	showOtherMonth:		true,
		//	selectOtherMonth:	false,
		//	changeMonth:		true,
		//	changeYear:			true,
		//	positionAdjust:		[-1,2],
		//	region:				ep.config.region,
		//	time:				true,
		//	timeStep:			15,

	//	ep.ui.DatepickerApp
			time:				true,
			appointment: {							// hash to perform appointment available
				id:					0,
				quantityMin:		1,
				quantityRequested:	1,
				quantityAvailable:	1,
				minutesRequested:	60
			}
		},

		_create: function(){
			this.Inherited();
			this._appointmentSort = {};
			this._appointmentStack = {};

			this._loadDayList( this._date.clone().addDate(-7), true );
		},

		_createTimeSelect: function(){
			this.timeSelect = ep('<select>')
				.attr({
					'class': 'epWidth100'
				})
				.appendTo( this.timeBar )
				.uiInput()
				.uiInput('addClass','ep-uiDatepicker-time')
				.on( 'changeValue.uiAppointmentpicker', this.Proxy('_changeTimeVal') );

			this.stack.push( this.timeSelect[0]	);
		},

		_createCalendars: function(){
			this.Inherited(arguments);
			this._createTimeSelectData();
		},

		_calendarData: function(){
			var self			= this,
				o				= this.options,
				opt				= o.appointment,
				calendarsData	= this.Inherited(arguments),
				appData			= $.scope( opt.minutesRequested+'', this._appointmentStack ),
				load			= false;

			$.each( calendarsData, function( i, calendarData ){
				$.each( calendarData.days, function( j, day ){
					if( this.clickable ){
						var date = day.date,
							dateString = date.getFormat('S'),
							dayObj = appData[ dateString ];

						if( !dayObj && !load ){
							appData	= self._loadDayList( date.clone() );
							dayObj = appData[ dateString ];
							load = true;
						}

						this.clickable = !!(dayObj && dayObj.IsAvailable);
					}
				});
			});

			return calendarsData;
		},

		_createTimeSelectData: function(){
			var self			= this,
				opt				= this.options.appointment,
				dateString		= this._dateSelected.getFormat('S'),
				appStack		= $.scope(  opt.minutesRequested+'', this._appointmentStack ),
				dayObj			= appStack[ dateString ],
				placeholder		= $('<option/>')
					.attr('disabled', true)
					.text(tLoading);


			self.timeSelect
				.empty()
				.append(placeholder);

			if( dayObj ){
				// load TimeList for selected day if not cached
				if( !dayObj.DayList ){
					appData		= self._loadTimeList(self._dateSelected);
					return;
				}

				var region		= {region: self.options.region};

				// add TimeList to select field
				if( dayObj.DayList && dayObj.DayList.length ){
					self.timeSelect.empty();
					$.each( dayObj.DayList, function( i, data ){
						if( (opt.quantityAvailable-data[1]) >= opt.quantityRequested ){
							var date = new ep.Date(data[0]);
							$('<option>')
								.attr( 'value', date.setFullYear(0,0,0).getTime() )
								.text( date.getFormat( 't', region ) )
								.appendTo( self.timeSelect );
						}
					});
				}
				// show times are booked out
				else{
					placeholder.text(tBookedOut);
				}

			}

			self._setTimeVal();
		},

		_setTimeVal: function(){
			var self			= this,
				selectedTime	= this._timeSelected.getTime()+'',
				selectedNode,
				selectedIndex;

			$.each( this.timeSelect[0].options, function( i, node ){
				if( node.value === selectedTime ){
					node.selected = true;
					selectedNode = node;
					selectedIndex = i;
					return false;
				}
			});

			if( !selectedNode && $(this.timeSelect[0].options).not(':disabled').length ){
				selectedNode = this.timeSelect[0].options[0];
				selectedIndex = 0;
				selectedNode.selected = true;
				selectedTime = selectedNode.value;
			}

			this._timeSelected = new ep.Date(selectedTime);

			if( !this.elem.is(':focus') ){
				this._setElemVal(true);
			}
		},

		_getTimeVal: function(){
			var opt = this.timeSelect[0].selectedIndex >= 0 ?
						this.timeSelect[0].options[ this.timeSelect[0].selectedIndex ] :
						null,
				date = opt ? new ep.Date(opt.value) : new ep.Date();
			return date.setFullYear(0,0,0);
		},
		// attention the date object will be modified, be sure to use an cloned date object
		_loadDayList: function( date, preload ){
			var self		= this,
				o			= this.options,
				opt			= o.appointment,
				dateStart	= date,
				dateEnd		= dateStart.clone().addMonth( o.numberOfMonths + 2 ),
				ret;


			ep.ajax({
				data: {
					'ViewAction':		'JSON-AppointmentCalendar',
					'ObjectID':			opt.id,
					'StartDate':		dateStart.getFormat('S'),
					'EndDate':			dateEnd.getFormat('S'),
					'RequestedMinutes':	opt.minutesRequested,
					'Available':		opt.quantityAvailable,
					'UseTimestamps':	1,
					'IsAvailableOnly':	1
				},
				async: !!preload,
				dataType: 'json',
				success: function( data ){
					ret = data;
				},
				error: function( xhr, status, error ){
					if( !preload ){
						self.hide();
						ep.alert({
							type:		'error',
							content:	tDataNotLoad
						});
					}
				}
			});

			ret = $.extend( true, $.scope( opt.minutesRequested+'', this._appointmentStack ), ret );
			return ret;
		},

		// attention the date object will be modified, be sure to use an cloned date object
		_loadTimeList: function( date ){
			var self		= this,
				o			= this.options,
				opt			= o.appointment,
				dateStart	= date,
				dateEnd		= dateStart.clone().addDate(1),
				ret;

			ep.ajax({
				data: {
					'ViewAction':		'JSON-AppointmentCalendar',
					'ObjectID':			opt.id,
					'StartDate':		dateStart.getFormat('S'),
					'EndDate':			dateEnd.getFormat('S'),
					'RequestedMinutes':	opt.minutesRequested,
					'Available':		opt.quantityAvailable,
					'UseTimestamps':	1
				},
			//	async:		false,
				dataType:	'json',
				success:	function( data ){
					ret = $.extend( true, $.scope( opt.minutesRequested+'', self._appointmentStack ), data );

					self._createTimeSelectData();
				},
				error:		function( xhr, status, error ){
					self.hide();
					ep.alert({
						type:		'error',
						content:	tDataNotLoad
					});
				}
			});
		},

		updateAppointment: function( options ){
			$.extend( this.options.appointment, options );
			// preload data
			// this._loadDayList( this._date.clone().addDate(-7), true );
		}
	});
});
