/*
 * @copyright		Copyright 2006-2011, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.seo.ui.seoCheckWidget
 *
 * @revision		$Revision: 1.4.14.1 $
 */

jQuery.plugin( 'de_epages.seo.ui.seoCheckWidget', {
	plugin:	['de_epages.core', 'ep.ui.dialog', 'ep.ajax', 'jQuery.ui.tabs', 'de_epages.seo.ui.seoGaugeWidget', 'jQuery.metaparse']
}, function( $, undefined ){
	$.widget( "de_epages.seo.ui.seoCheckWidget", {

		_gaugeWidget : undefined,

		options : {
			caption : '{SEOCheck}',
			currentSEOQuality : 0,
			objectId : ep.config.objectId,
			languageId : ep.config.languageId
		},

		_create : function() {
			this.elem = this.element.sub(ep);
			this.elem.uiDialog('option', 'width', 920);
			this.elem.uiDialog('option', 'height', 550);
			this.elem.uiDialog('option', 'modal', true);
		},

		_init : function() {
			this.element.html('');
			this._getDialogContent();
			this._buildTitleGauge();
		},

		_getDialogContent : function() {
			ep.ajax({
				data: {
					'ViewAction': 'MBO-ViewSEOCheck',
					'ObjectID': this.options.objectId,
					'LanguageID': this.options.languageId
				},
				context: this,
				success : function(data, textStatus, XMLHttpRequest) {
					data = this._sortTestsBySeverity(data);
					this.element.empty().append(data).find('.ep-metaparse').metaparse();
					var newQual = this.elem.children('.seoCheck').data('newseoquality');
					if(newQual && newQual != this.options.currentSEOQuality) {
						this._setSEOQuality(newQual);
						this.elem.trigger("SEOCheckNewQuality", newQual);
					}
				}
			});
		},

		_sortTestsBySeverity : function(data) {
			var cache = jQuery(data);
			cache.find('ul.seoTest').each(function() {
				var ul = $(this);
				ul.children('.yellow').prependTo(ul);
				ul.children('.orange').prependTo(ul);
				ul.children('.red').prependTo(ul);
			});
			return cache;
		},

		_buildTitleGauge : function() {
			this._gaugeWidget = de_epages('<div>').seoUiSeoGaugeWidget({
				'caption' : this.options.caption,
				'showStatusImage' : false,
				'seoQuality' : this.options.currentSEOQuality
			})
			.seoUiSeoGaugeWidget('Instance');
			this.elem.uiDialog('option','title', this._gaugeWidget.element );
		},

		_refreshTitleGauge : function(newValue) {
			if(this._gaugeWidget) {
				this._gaugeWidget.setPercentage(newValue);
			}
		},

		_setSEOQuality : function(newValue) {
			ep.ajax({
				data: {
					'ViewAction' : 'JSONViewResponse',
					'ChangeAction' : 'MBO-SaveSEOQuality',
					'ObjectID' : this.options.objectId,
					'SEOQuality' : newValue
				},
				context: this,
				cache: false,
				type: "post",
				dataType: "json",
				success : function(data) {
					this.options.currentSEOQuality = newValue;
					this._refreshTitleGauge(newValue);
				}
			});
		}

	});
});
