/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.dashboard
 *
 * @revision		$Revision: 1.27.4.1.2.1 $
 */

jQuery.plugin( 'de_epages.shop.ui.dashboard', [
	'de_epages.core',
	'jQuery.effects.clip',
	'jQuery.ui.widget',
	'jQuery.ui.sortable',
	'ep.ajax',
	'ep.dict',
	'ep.alert',
	'ep.ui.dialog',
	'de_epages.shop.ui.dashboardWidget',
	'ep.fn.busy'
], function( $, undefined ){

	/*
	 * @dictionary				ep.dict
	 *
	 * @translation				{WIDGET_LOADING_ERROR}
	 *							{WIDGET_DELETE_ERROR}
	 *							{WIDGET_DELETE_ERROR_TITLE}
	 *							{WIDGET_ADD_ERROR}
	 *							{WIDGET_ADD_ERROR_TITLE}
	 *							{WIDGET_POSITION_ERROR}
	 * 							{WIDGET_POSITION_ERROR_TITLE}
	 */
	$.widget( "de_epages.shop.ui.Dashboard",/* my.parent.widget,*/ {
		_widgetCount: 0,
		_reloadWidgetStack: [],
		SIMULTAN_REQUESTS: 2,
		_runningRequests: 0,
		_dict: ep.dict('ep.dict'),
		_dialogDiv: null,

		options: { //define options of widget formally know as widget properties in dojo
			userId: undefined
		},

		_create: function(){
			this.element.sortable({
				placeholder: 'ep-PlaceholderNode ep-gradient',
				handle: '.ep-TitleNode',
				opacity: 0.9,
				containment: '#DashboardContent',
				revert: 'true',
				items: 'li',
				cancel: '.ep-FixedNode', //apply this class to elements that should not be draggable
				stop: function(evt,ui){
					de_epages(this).shopUiDashboard('saveWidgetPositions');
				}
			});
			var self = this;
			this.element
				.sub(de_epages)
				.find('.ep-DashboardWidget')
				.each(function(index,value){
					if(value){
						var widgetInstance = de_epages(value).shopUiDashboardWidget('Instance');
						if(widgetInstance){
							self.registerWidget(de_epages(value).shopUiDashboardWidget('Instance'));
							self._widgetCount++;
						}
					}
				});
				if(this._widgetCount == 0){
					this.element.trigger('dashboardEmpty',[this]);
				}
				else{
					this.element.trigger('dashboardUsed',[this]);
				}

			//bind events
			this.element
				.on('deleteWidget.dashboard',$.proxy(this,'_onDeleteWidget'))
				.on('addWidget.dashboard',$.proxy(this,'_onAddWidget'))
				.on('showAddWidgetDialog.dashboard',$.proxy(this,'_onShowAddWidgetDialog'));
			for(var i = 0; i < this.SIMULTAN_REQUESTS;i++){
				this._reloadWidgetContent();
			}
		},

		registerWidget: function(dashboardWidget){
			var widgetData = dashboardWidget.options;
			if(widgetData && widgetData.reload && widgetData.action){
				var reloadWidget = {
					element: dashboardWidget.element,
					action: widgetData.action,
					objectId: widgetData.objectId
				};
				this._reloadWidgetStack.push(reloadWidget);
			}
			this._reloadWidgetContent();
		},

		_reloadWidgetContent: function(){
			if(this._reloadWidgetStack.length){
				if(this._runningRequests < this.SIMULTAN_REQUESTS){
					this._runningRequests++;
					var reloadWidget = this._reloadWidgetStack.shift();
					ep.ajax({
						data: {
							'ViewAction': reloadWidget.action,
							'ChangeAction': 'BlockMonitor',
							'ObjectID': ep.config.siteId
						},
						url : ep.config.baseUrl,
						type: 'GET',
						cache: false,
						context: this,
						success :  function(data, textStatus, XMLHttpRequest){
							this._runningRequests--;
							this._reloadWidgetContent();
							reloadWidget.element.find('.ep-ContentNode').html(data);
						},
						error : function(data, textStatus, XMLHttpRequest){
							this._runningRequests--;
							this._reloadWidgetContent();
							reloadWidget.element.find('.ep-ContentNode').html(this._dict.translate('WIDGET_LOADING_ERROR'));
						}
					});
				}
			}
		},

		_onDeleteWidget: function(evt,dashboardWidget){
			var deleteObjectId = dashboardWidget.options.objectId;
			if(deleteObjectId){
				//Do async request to server
				ep.ajax({
					dataType : 'json',
					data: {
						'ViewAction': 'JSONViewResponse',
						'ChangeAction': 'JSONRemoveWidget',
						'ObjectID': deleteObjectId,
						'ViewObjectID': this.options.userId
					},
					context: this,
					success :  function(data, textStatus, XMLHttpRequest){
						var dashboardWidgetData = dashboardWidget.options;

						var widgetTypeId = dashboardWidgetData.widgetTypeId;
						var placeholderWidget = $('#DashboardWidget_'+widgetTypeId,this._dialogDiv);

						var animationPlaceHolder = dashboardWidget.element.clone(false);
						animationPlaceHolder.insertBefore(dashboardWidget.element);
						animationPlaceHolder.hide('clip',{},500,
						function() {
							$(this).remove();
						});

						if(placeholderWidget.length){
							dashboardWidget.element.insertBefore(placeholderWidget);
							placeholderWidget.remove();
						}
						else{
							dashboardWidget.destroy();
						}

						dashboardWidgetData.objectId = dashboardWidgetData.widgetTypeId;
						dashboardWidget.setStatus('preview');
						this._widgetCount--;
						if(this._widgetCount == 0){
							this.element.trigger('dashboardEmpty',[this]);//trigger empty event when nomore widgets are available
						}
					},
					error : function(data, textStatus, XMLHttpRequest){
						ep.alert({
							content: this._dict.translate('WIDGET_DELETE_ERROR'),
							title: this._dict.translate('WIDGET_DELETE_ERROR_TITLE'),
							ok: function(){}
						});
					}
				});
			}
			else{
				dashboardWidget.destroy();
			}

		},

		_onAddWidget: function(evt,dashboardWidget){
			var dialogNode = ep(this._dialogDiv).uiDialog('widget');
			ep(dialogNode).busy('show');
			var newWidgetTypeId = dashboardWidget.options.widgetTypeId;
			ep.ajax({
				dataType: 'json',
				data: {
					'ViewAction': 'JSONViewResponse',
					'ChangeAction': 'JSONAddWidget',
					'ObjectID': this.options.userId,
					'WidgetTypeId': newWidgetTypeId
				},
				context: this,
				success :  function(data, textStatus, XMLHttpRequest){
					var dashboardWidgetData = dashboardWidget.options;
					var placeholderWidget = dashboardWidget.element.clone(false);
					placeholderWidget.attr('id','DashboardWidget_'+dashboardWidgetData.widgetTypeId);

					placeholderWidget.insertBefore(dashboardWidget.element);

					dashboardWidgetData.objectId = data.widgetId;
					dashboardWidget.element.attr('id',data.widgetId);
					if(this.element.find('.ep-DashboardWidget:first').length){
						dashboardWidget.element.insertBefore(this.element.find('.ep-DashboardWidget:first'));
					}
					else{
						this.element.append(dashboardWidget.element);
					}
					this.element.sortable('refresh');

					de_epages(placeholderWidget).shopUiDashboardWidget({
						'dashboard': this.element,
						'status': 'used',
						'reload': dashboardWidgetData.reload,
						'objectId': dashboardWidgetData.widgetTypeId,
						'widgetTypeId': dashboardWidgetData.widgetTypeId
					});
					de_epages(placeholderWidget).shopUiDashboardWidget('refresh');

					this._widgetCount++;
					if(this._widgetCount == 1){
						this.element.trigger('dashboardUsed',[this]);//trigger used event when widgets are available
					}

					dashboardWidget.setStatus();
					this.saveWidgetPositions();
					ep(dialogNode).busy('hide');
				},
				error : function(){
					ep.alert({
						content: this._dict.translate('WIDGET_ADD_ERROR'),
						title: this._dict.translate('WIDGET_ADD_ERROR_TITLE'),
						ok: function(){}
					});
					ep(dialogNode).busy('hide');
				}
			});
		},

		_onShowAddWidgetDialog: function(evt){

			if(this._dialogDiv){
				ep(this._dialogDiv).uiDialog('open');
			}
			else{
			var closeButtonText = this._dict.translate('Close');
				ep.ajax({
				  dataType : 'json',
				  cache: false,
					data: {
						'ViewAction': 'JSONSnippet',
						'ObjectID': ep.config.siteId,
						'TemplateAction': 'MBO-ViewAddWidgetDialogSnippet',
						'TemplateLanguageID': ep.config.languageId,
						'TemplateName': 'Page'
					},
					context: this,
					success :  function(data, textStatus, XMLHttpRequest){
						this._dialogDiv = $('<div>')
							.html(data.snippet);
						var dialogTitle = $(this._dialogDiv).children().first().attr('title');
						$(this._dialogDiv).children().first().removeAttr('title');
						ep(this._dialogDiv).uiDialog({
							autoOpen: false,
							title: dialogTitle,
							modal: true,
							width: '980',
							height: '550',
							buttons: { "Close": {
								text: closeButtonText,
								click: function() { ep(this).uiDialog("close");}
							}}
						});
						ep(this._dialogDiv).find('.ep-metaparse').metaparse();
						// TODO: call custom event on body to build djo widgets inside dashboard widgets
						// please remove this call if there are no longer dojo widgets inside dashboard
						$('body').trigger('DashboardDialogReadyDojoConnector');
						//TODO END
						ep(this._dialogDiv).uiDialog('open');

						ep('.dashboardBoxGroups li').on('click', $.proxy(this,'_toggleWidgetTabs'));

						// add initial css-"Active"-class and trigger a click for showing widgets of the first widget-group
						ep(this._dialogDiv).find('.dashboardBoxWidgets ul:gt(0)')
						  .hide();
						ep(this._dialogDiv).find('.dashboardBoxGroups li:first')
							.addClass('Active')
							.children()
							.addClass('Active')
							.trigger('click');

					}
				});
			}
		},

		_toggleWidgetTabs: function(evt) {
			var nodeClass = ep(evt.currentTarget).attr('class');
			if( ep(evt.currentTarget).hasClass('Active') == false) {
				var dashboardBoxWidgets = ep(this._dialogDiv).find('.dashboardBoxWidgets');
				var tmp = 'ul:not(#'+nodeClass+')';
				dashboardBoxWidgets.find(tmp).hide(200);
				tmp = 'ul#'+nodeClass;
				dashboardBoxWidgets.find(tmp).show(200);

				// Adding/ Deleting CSS-Classes
				ep(this._dialogDiv)
					.find('.dashboardBoxGroups .Active')
					.removeClass('Active')
					.children()
					.removeClass('Active');
				ep(evt.currentTarget)
					.addClass('Active')
					.children()
					.addClass('Active');
			}
			return false;
		},

		saveWidgetPositions: function(){
			if(this.options.userId){
				var positionsString = '';
				this.element.find('.ep-DashboardWidget').each(function(index,elm){
					var widgetInstance = de_epages(elm).shopUiDashboardWidget('Instance');
					if(widgetInstance && widgetInstance.options && widgetInstance.options.objectId){
						var objectId = widgetInstance.options.objectId;
						positionsString += objectId + ';';
					}
				});

				ep.ajax({
					dataType : 'json',
					type: 'POST',
					data: {
						'ViewAction': 'JSONViewResponse',
						'ChangeAction': 'JSONSaveWidgetPositions',
						'ObjectID': this.options.userId,
						'Positions': positionsString
					},
					context: this,
					error : function(){
						ep.alert({
							content: this._dict.translate('WIDGET_POSITION_ERROR'),
							title: this._dict.translate('WIDGET_POSITION_ERROR_TITLE'),
							ok: function(){}
						});
					}
				});
			}
		},

		destroy: function(){
			this.element.off('.dashboard');
			$(this).remove();
		}

	});

});