/*
 * @copyright		Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @odule			de_epages.shop.ui.dashboardWidget
 *
 * @evision		$Revision: 1.14.6.1 $
 */

jQuery.plugin( 'de_epages.shop.ui.dashboardWidget', [
	'de_epages.core',
	'jQuery.effects.slide',
	'jQuery.ui.widget',
	'ep.fn.busy'
], function( $, undefined ){

	$.widget( "de_epages.shop.ui.DashboardWidget", {
		_buttonsNode: null,
		_deleteButtonNode: null,
		_addButtonNode: null,
		_titleNode: null,
		_previewLayerNode: null,
		_previewLayerAddNode: null,
		_contentNode: null,
		_bindStack: null,
		_dashboard: null,

		options: { //define options of widget formally know as widget properties in dojo
			dashboard: null,
			status: null,
			reload: null,
			action: null,
			objectId: null

		},

		_create: function(){
			this.element = this.element.sub(ep);
			this._dashboard = $(this.options.dashboard);
			this._bindStack = $();
			this._deleteButtonNode = this.element.find('.ep-DeleteButtonNode');
			this._addButtonNode = this.element.find('.ep-AddButtonNode');
			this._buttonsNode = this.element.find('.ep-ButtonsNode');
			this._titleNode = this.element.find('.ep-TitleNode');
			this._previewLayerNode = this.element.find('.ep-PreviewLayerNode');
			this._previewLayerAddNode = this.element.find('.ep-PreviewLayerAddNode');
			this._contentNode = this.element.find('.ep-ContentNode');

			//bind events
			this._bindStack.push(
				this._deleteButtonNode[0],//first Element in jQuery array like object is dom node it self
				this._addButtonNode[0],
				this._buttonsNode[0],
				this._titleNode[0],
				this.element[0]
			);

			if(this._previewLayerNode.length){
				this._bindStack.push(this._previewLayerNode[0]);
			}
			if(this._previewLayerAddNode.length){
				this._bindStack.push(this._previewLayerAddNode[0]);
			}

			this.setStatus(this.options.status);

			if(this.options.reload && this.options.action){
				this._contentNode.busy();
			}
			de_epages(this._dashboard).shopUiDashboard('registerWidget',this);
		},

		_showButtons: function(evt){
			this._buttonsNode.stop(true,true).fadeIn();
		},

		_hideButtons: function(evt){
			this._buttonsNode.stop(true,true).fadeOut();
		},

		_onClickDeleteButtonNode: function(evt){
			this._dashboard.trigger('deleteWidget',[this]);
		},

		_onClickAddButtonNode: function(evt){
			this._dashboard.trigger('addWidget',[this]);
		},

		setStatus: function(status){
			this.options.status = status;
			this._buttonsNode.hide();
			this._bindStack.off();

			if(this._previewLayerNode.length){
				this._previewLayerNode.hide();
			}

			if(this._previewLayerAddNode.length){
				this._previewLayerAddNode.hide();
			}

			this._deleteButtonNode.hide();
			this._addButtonNode.hide();

			switch(status){
				case "preview":
					this._addButtonNode.show();
					this._addButtonNode
						.one("click.dashboardWidget",$.proxy(this,'_onClickAddButtonNode'));
					this._previewLayerAddNode
						.show()
						.on("mouseover.dashboardWidget",$.proxy(this,'_showButtons'))
						.one("click.dashboardWidget",$.proxy(this,'_onClickAddButtonNode'));
					this.element
						.on("mouseleave.dashboardWidget",$.proxy(this,'_hideButtons'));
				break;
				case "used":
						this._previewLayerNode.show();
				break;
				default:
					this._deleteButtonNode.show();
					this._titleNode
						.on("mouseover.dashboardWidget",$.proxy(this,'_showButtons'))
						.on("mouseleave.dashboardWidget",$.proxy(this,'_hideButtons'));
					this._deleteButtonNode
						.on('mousedown.dashboardWidget',function( event ){ event.stopPropagation(); })
						.one("click.dashboardWidget",$.proxy(this,'_onClickDeleteButtonNode'));
				break;
			}
		},

		destroy: function(){
			this._bindStack.off();
			this.element.remove(undefined, true);
		}

	});

});
