use strict;

use DE_EPAGES::Core::API::File qw ( GetFileContent );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Shopzilla::API::XML::AccountActivationResponseHandler;

use Readonly;
use Test::More;

# directory to Shopzilla sample files.
Readonly my $XMLSAMPLEDIR => $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Shopzilla/t/';

sub Main {
    TestActivationResponse200();
    TestActivationResponse401();
    TestActivationResponse403();
    TestActivationResponse404();
    TestActivationResponse411();
    TestActivationResponseMulti();
    done_testing();
}

sub TestActivationResponse200 {
    my $Result
      = DE_EPAGES::Shopzilla::API::XML::AccountActivationResponseHandler->importXML(
        ${GetFileContent($XMLSAMPLEDIR . 'example_act_resp_200.xml', {'encoding' => 'utf-8'})});
    my $Merchant = $Result->[0];
    isa_ok($Merchant, 'DE_EPAGES::Shopzilla::API::DTO::Merchant', 'Merchant');
    is($Merchant->name(), 'Testmerchant1', 'Merchant name');
    is($Merchant->mgid(), '1234567', 'Merchant mgid');
    is($Merchant->mid(), '123456', 'Merchant mid');
    is($Merchant->site(), '12345', 'Merchant site');
    is($Merchant->status_code(), '200', 'Merchant status_code');
}

sub TestActivationResponse401 {
    my $Result
      = DE_EPAGES::Shopzilla::API::XML::AccountActivationResponseHandler->importXML(
        ${GetFileContent($XMLSAMPLEDIR . 'example_act_resp_401.xml', {'encoding' => 'utf-8'})});
    my $Merchant = $Result->[0];
    isa_ok($Merchant, 'DE_EPAGES::Shopzilla::API::DTO::Merchant', 'Result');
    is($Merchant->status_code(), '401', 'Merchant status_code');
    is($Merchant->status_text(), 'Unauthorized', 'Merchant status_text');
}

sub TestActivationResponse403 {
    my $Result
      = DE_EPAGES::Shopzilla::API::XML::AccountActivationResponseHandler->importXML(
        ${GetFileContent($XMLSAMPLEDIR . 'example_act_resp_403.xml', {'encoding' => 'utf-8'})});
    my $Merchant = $Result->[0];
    isa_ok($Merchant, 'DE_EPAGES::Shopzilla::API::DTO::Merchant', 'Result');
    is($Merchant->status_code(), '403', 'Merchant status_code');
    is($Merchant->status_text(), 'EINSTID does not authorize UMID', 'Merchant status_text');
}

sub TestActivationResponse404 {
    my $Result
      = DE_EPAGES::Shopzilla::API::XML::AccountActivationResponseHandler->importXML(
        ${GetFileContent($XMLSAMPLEDIR . 'example_act_resp_404.xml', {'encoding' => 'utf-8'})});
    my $Merchant = $Result->[0];
    isa_ok($Merchant, 'DE_EPAGES::Shopzilla::API::DTO::Merchant', 'Result');
    is($Merchant->status_code(), '404', 'Merchant status_code');
    is($Merchant->status_text(), 'Not Setup', 'Merchant status_text');
}

sub TestActivationResponse411 {
    my $Result
      = DE_EPAGES::Shopzilla::API::XML::AccountActivationResponseHandler->importXML(
        ${GetFileContent($XMLSAMPLEDIR . 'example_act_resp_411.xml', {'encoding' => 'utf-8'})});
    my $Merchant = $Result->[0];
    isa_ok($Merchant, 'DE_EPAGES::Shopzilla::API::DTO::Merchant', 'Result');
    is($Merchant->status_code(), '411', 'Merchant status_code');
    is($Merchant->status_text(), 'UMID and / or EINSTID is / are invalid', 'Merchant status_text');
}

sub TestActivationResponseMulti {
    my $Result
      = DE_EPAGES::Shopzilla::API::XML::AccountActivationResponseHandler->importXML(
        ${GetFileContent($XMLSAMPLEDIR . 'example_act_resp_multi.xml', {'encoding' => 'utf-8'})});
    is(scalar @{$Result},3,'count result');
    foreach my $Merchant (@{$Result}){
        isa_ok($Merchant, 'DE_EPAGES::Shopzilla::API::DTO::Merchant', 'Result');
        ok( defined $Merchant->name(), 'Merchant name');
        ok( defined $Merchant->mid(), 'Merchant mid');
        ok( defined $Merchant->status_code(), 'Merchant status_code');
        ok( defined $Merchant->status_text(), 'Merchant status_text');
    }
}


RunScript( Sub => \&Main );

