use Test::More;
use Time::HiRes qw( time );
use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );


plan tests => 3;

sub Main {
    my $ServerName = 'localhost';
    my $URL = "http://$ServerName/epages/TestRequest.sf";
    my $FakeRequest = MyFakeRequest->new;
    my $Response = $FakeRequest->get( $URL );

    cmp_ok( $Response->code, '==', 200, '200 OK' );
    cmp_ok( $Response->content, 'eq', 'TestRequest: OK', 'content' );

    # Run test once before measuring the performance.
    $FakeRequest->get( $URL ) for 1..8000;
    my $time0 = time();
    $FakeRequest->get( $URL ) for 1..8000;
    my $time1 = time();
    my $diff = sprintf '%0.3f', $time1 - $time0;
    my $avg = sprintf '%1.0f', 8000 / $diff;
    cmp_ok( $time1 - $time0, '<', 10, "at least 800 test requests per second (8,000 requests took $diff seconds, average requests per second: $avg)" );
    return;
}

RunScript(Sub=>\&Main, Debug=>1);

package MyFakeRequest;
use base qw( DE_EPAGES::WebInterface::API::FakeRequest );

sub lookupPoolDB {
    my $self = shift;
    my ($Request) = @_;

    # set the X-Headers
    $Request->header('X-ePages-Site' => 'DummyGUID');
    $Request->header('X-ePages-Store' => 'DummyStore');
    $Request->header('X-ePages-Pool' => 'DummyPool');
    return;
}
