use strict;
use Test::More;
use Perl::Critic;
use DE_EPAGES::Core::API::Script qw( RunScript );
use lib ("$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/XML/Scripts");
use Perl::Critic::Policy::DE_EPAGES::Style::RequireCorrectImports;

# Tests Perl::Critic::Policy::DE_EPAGES::Variables::RequireCorrectImports

sub Main { ## no critic (ProhibitExcessComplexity, DE_EPAGES::Style::ProhibitFunctionSize)
    my $critic = Perl::Critic->new(
        -profile  => 'NONE',
        -severity => 2
    );
    ok( defined $critic, 'Perl::Critic initialized' );
    $critic->add_policy( -policy => 'Perl::Critic::Policy::DE_EPAGES::Style::RequireCorrectImports' );
    my @policies = $critic->policies();
    my $policyLoaded = scalar grep { $_ =~ 'DE_EPAGES::Style::RequireCorrectImports' }
    @policies;
    ok( $policyLoaded, 'Policy loaded' );

    my @violations;
    my $sourceCode;
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
RunScript( $Test );
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'ok - use correct ') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Object::API::Factory qw ( RunScript );
RunScript( $Test );
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 1, 'duplicate import ') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadClassByAlias );
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 3, 'unused imports ') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use strict;
use DE_EPAGES::Object::API::Factory qw ( LoadObject );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );
LoadObject();
LoadClassByAlias();
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 1, 'package on multiple lines') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );

EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 1, 'qw ( )') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use strict;
use DE_EPAGES::Object::API::Factory qw ( LoadObject2 );
LoadObject2();
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 1, 'LoadObject2 not in export list') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use strict;
LoadObject();
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 1, 'LoadObject is not imported') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use strict;
use DE_EPAGES::Object::API::Factory;
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 1, 'unused general import.') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use strict;
use Test::More;
pass('ok');
done_testing();
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'used general import.') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use strict;
$a->LoadObject();
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'object method') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
package DE_EPAGES::Test::Abc;
use DE_EPAGES::Content::DAL::Content;

DAL_GetInfoShortURLsByContentObjectID(1);

EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'EXPORT test') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
package DE_EPAGES::Test::Abc;
use DE_EPAGES::Content::API::Cartridge;

DE_EPAGES::Content::API::Cartridge->new();

EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'package test') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use DE_EPAGES::Core::API::CartridgeConfig qw (CartridgeConfig);
CartridgeConfig();
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'nochmal qw ()') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use Sort::Versions qw(versioncmp);
versioncmp();
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, '& in export.') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
sub RunScript ($);
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'sub prototype declaration') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
package DE_EPAGES::Test;
use base DE_EPAGES::Core::API::CartridgeConfig;

EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'child class') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
sub RunScript {}
RunScript(1);
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'sub declaration and use') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use constant    LOG_INFO  => (23<<3);
my %SYSLOGLEVELS = (FATAL   => LOG_INFO);
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'constant declaration and use') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use DE_EPAGES::Test;
my $x = 'DE_EPAGES::Test';
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'use package name in string.') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use Config;
$Config{Test} = 1;
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'use package that exports variables.') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';

use constant    LOG_INFO  => (23<<3);
use constant    LOG_INFO2  => LOG_INFO;
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'use constant referring to constant.') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use DE_EPAGES::Core::API::Process;
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'use in exclude list.') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use Readonly;
Readonly::Scalar my $DESC => q{UNIVERSAL::can should not be used as a function};
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'using fully qualified name.') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use DE_EPAGES::Amazon::API::Constants qw ( %AMAZON_ONLINESTATUS );
my $x = $AMAZON_ONLINESTATUS{};
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'skip variables.') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
package DE_EPAGES::Order::API::Object::MyOrder;
use DE_EPAGES::Amazon::API::Constants qw ( %AMAZON_ONLINESTATUS );
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 1, 'dependency violation.') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
package DE_EPAGES::Amazon::API::Constants;
use DE_EPAGES::Order::API::Constants qw ( CLASS_MAP );
CLASS_MAP();
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'dependency OK.') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
package DE_EPAGES::Object::API::XML::ObjectDeleteHandler;
use strict;
use DE_EPAGES::Payment::API::Table::ShippingPaymentMap qw ( DeleteShippingPaymentMap );
DeleteShippingPaymentMap();
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'dependency of same package.') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
package DE_EPAGES::Test;
use strict;
use DE_EPAGES::Core::API::Html;
DE_EPAGES::Core::API::Html::BalanceHTMLTags();
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'qualified use.') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
package DE_EPAGES::Test;
use strict;
use DE_EPAGES::Core::API::Html qw ( BalanceHTMLTags );
DE_EPAGES::Core::API::Html::BalanceHTMLTags();
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 1, 'import and qualified use.') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
package DE_EPAGES::Test;
use strict;

use constant {
    BULKGOODS_ATTRIBUTE_ID     => 123,  #ID for Sperrgut
    PROOF_ATTRIBUTE_ID         => 13,   #ID for Einschreiben
    PICKUPSERVICE_ATTRIBUTE_ID => 17,   #ID for Abholung
    GOGREEN_ATTRIBUTE_ID       => 999,  #ID for GoGreen
    ROLEPACKAGE_ATTRIBUTE_ID   => 998,  #ID for Rollenversand
    INSURANCE_ATTRIBUTE_ID     => 20,   #ID for Versicherung/ Haftung
};
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'use constant as hash') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
package DE_EPAGES::Test;
use strict;
push (@errors , {
    Error => 'Not imported function',
});
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'use Error as hash key.') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
package DE_EPAGES::Test;
use strict;
use Time::HiRes qw( time );
time();
1;
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'overwrite perl internal time') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use strict;
use DE_EPAGES::Core::API::IDN qw(
    EncodeDomain
);

sub Main {
    RunTests( \&EncodeDomain );
}
1;
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'Method reference.') or diag(@violations);
#-------------------------------------------------------
$sourceCode = <<'EOL';
use strict;
package MyPackage;
sub x {
    my $hResult;
    return if($hResult->{Error});
}
1;
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'Bareword hash keys.') or diag(@violations);
#-------------------------------------------------------
$sourceCode = <<'EOL';
use DE_EPAGES::Installer::API::CPAN qw( TestInstallCPAN );

foreach my $hModule (@{TestInstallCPAN($var)}) {
}

EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'array cast') or diag(@violations);

$sourceCode = <<'EOL';
use strict;
use DE_EPAGES::Object::API::Language;
{
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = 1;
}

EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'fully-qualified variable access') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
RunScript( $Test );
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'ok - use correct ') or diag(@violations);
#-------------------------------------------------------
$sourceCode = <<'EOL';
package DE_EPAGES::Core::API::Constants;

use constant KILO => 10 ** 3;
use constant MEGA => KILO ** 2;
use constant GIGA => KILO ** 3;
use constant TERA => KILO ** 4;
use constant PETA => KILO ** 5;
use constant EXA  => KILO ** 6;
use constant ZETA => KILO ** 7;
use constant YOTTA => KILO ** 8;

use constant KIBI => 2 ** 10;
use constant MEBI => KIBI ** 2;
use constant GIBI => KIBI ** 3;
use constant TEBI => KIBI ** 4;
use constant PEBI => KIBI ** 5;
use constant EXBI => KIBI ** 6;
use constant ZEBI => KIBI ** 7;
use constant YOBI => KIBI ** 8;

1;
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'constant used') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
package DE_EPAGES::Test;
use strict;
use Sort::Versions qw(versioncmp);
versioncmp();
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'ok - EXPORT list contains &.') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use strict;
use DE_EPAGES::Core::API::Script qw( &RunScript );
RunScript( $Test );
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'ok - use contains &.') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use strict;
use Moose;
no Moose;
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'ok - stop complaining about no Moose statement') or diag(@violations);
#-------------------------------------
    $sourceCode = <<'EOL';
﻿use utf8;
use strict;
use Test::Simple tests => 7;
use DE_EPAGES::XML::API::Constants;

my @Good = ('alpha-ville', 'A2', 'Größe', 'a:b', ':b');
my @Bad = ('2A', '-5');
my $RegEx = XML_SYNTAX_Name;
foreach my $TestString (@Good) {
    ok ( $TestString =~ /^$RegEx$/ , $TestString);
}
foreach my $TestString (@Bad) {
    ok ( $TestString !~ /^$RegEx$/ , $TestString);
}
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'ok - Test::Simple') or diag(@violations);
#-------------------------------------------------------
    $sourceCode = <<'EOL';
use strict;
use DE_EPAGES::Core::API::Script 'RunScript';
RunScript( $Test );
EOL
    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, 'ok - single import correct') or diag(@violations);
}

RunScript( Sub => \&Main );
done_testing();