Option Explicit

ConfigureIIS()

Function ConfigureIIS

    Dim WshShell : Set WshShell = WScript.CreateObject("WScript.Shell")
    Dim WshSysEnv : Set WshSysEnv = WshShell.Environment("SYSTEM")
    Dim sWA_InstallDir : sWA_InstallDir = WshSysEnv("EPAGES") & "\WebAdapter\"

    ' add ".mobile" script extension
    Dim VirtDir_epages : Set VirtDir_epages = GetObject("IIS://LocalHost/W3SVC/1/Root/epages")
    Dim scriptMaps : scriptMaps = VirtDir_epages.ScriptMaps
    SetMapping scriptMaps, ".mobile," & sWA_InstallDir & "ISAPIAdapter.dll,1"
    VirtDir_epages.ScriptMaps = scriptMaps
    VirtDir_epages.SetInfo

    ' add JSON mime type mapping
    SetMimeType ".json", "application/json"

    Dim VirtDir_WebRoot : Set VirtDir_WebRoot = Nothing

    ' set Expiration date for /WebRoot/StoreTypes
    Dim VirtDir_StoreTypes : Set VirtDir_StoreTypes = Nothing
    On Error Resume Next
        Set VirtDir_StoreTypes = GetObject("IIS://LocalHost/W3SVC/1/Root/WebRoot/StoreTypes")
        Err.Clear
    On Error Goto 0

    If VirtDir_StoreTypes Is Nothing Then
        On Error Resume Next
            Set VirtDir_WebRoot = GetObject("IIS://LocalHost/W3SVC/1/Root/WebRoot")
            Err.Clear
        On Error Goto 0
        If Not VirtDir_WebRoot Is Nothing Then
            Set VirtDir_StoreTypes = VirtDir_WebRoot.Create("IISWebVirtualDir", "StoreTypes")
        End If
    End If

    If Not VirtDir_StoreTypes Is Nothing Then
        VirtDir_StoreTypes.HttpExpires = "D, 0x1E13380" ' expire content after 365 days (EPG-16323)
        VirtDir_StoreTypes.SetInfo
    End If

    ' set Expiration date for /WebRoot/_MONITOR_
    Dim VirtDir_WebRoot_Monitor : Set VirtDir_WebRoot_Monitor = Nothing
    On Error Resume Next
        Set VirtDir_WebRoot_Monitor = GetObject("IIS://LocalHost/W3SVC/1/Root/WebRoot/_MONITOR_")
        Err.Clear
    On Error Goto 0

    If VirtDir_WebRoot_Monitor Is Nothing Then
        On Error Resume Next
            Set VirtDir_WebRoot = GetObject("IIS://LocalHost/W3SVC/1/Root/WebRoot")
            Err.Clear
        On Error Goto 0
        If Not VirtDir_WebRoot Is Nothing Then
            Set VirtDir_WebRoot_Monitor = VirtDir_WebRoot.Create("IISWebVirtualDir", "_MONITOR_")
        End If
    End If

    If Not VirtDir_WebRoot_Monitor Is Nothing Then
        VirtDir_WebRoot_Monitor.HttpExpires = "S, Fri, 18 Mar 2011 0:00:00 UTC" ' expire content immediately (EPG-16183)
        VirtDir_WebRoot_Monitor.SetInfo
    End If

End Function

' Adds a script mapping or replaces an existing one with the same extension
Function SetMapping( mappings, value )

    Dim index, params, ext
    params = Split( value, "," )
    ext = params(0)
    For index=LBound(mappings) To UBound(mappings)
        params = Split( mappings(index), "," )
        If( params(0) = ext ) Then
            Exit For
        End If
    Next
    If index <= UBound(mappings) Then
        mappings(index) = value
    Else
        Redim Preserve mappings( UBound(mappings) - LBound(mappings)+1 )
        mappings(index)=value
    End If

    SetMapping = mappings

End Function


' Adds a MIME type mapping or updates an existing one
Function SetMimeType( extension, mimeType )
    Dim oMimeMap : Set oMimeMap = GetObject("IIS://LocalHost/MimeMap")
    Dim aMimeMap : aMimeMap = oMimeMap.GetEx("MimeMap")

    Dim exists : exists = False
    Dim item
    For Each item in aMimeMap
        If item.Extension = extension Then
            exists = True
            item.MimeType = mimeType
        End If
    Next

    If Not exists Then
        Dim i : i = UBound(aMimeMap) + 1
        Redim Preserve aMimeMap(i)
        Set aMimeMap(i) = CreateObject("MimeMap")
        aMimeMap(i).Extension = extension
        aMimeMap(i).MimeType = mimeType
    End If

    Const ADS_PROPERTY_UPDATE = 2
    oMimeMap.PutEx ADS_PROPERTY_UPDATE, "MimeMap", aMimeMap
    oMimeMap.SetInfo
End Function
